#!/usr/bin/perl -w

package BaseTest::EDGLifecycle::Stack::GfalStorm;

use FindBin qw($Bin);
use lib $Bin;

use File::Basename;
use EDGUtils;

use strict;
use vars qw(@ISA);
@ISA=qw(BaseTest::EDGLifecycle::Stack);

=head1 NAME

GfalStorm.pm - Check that globusrun authentication works on the hosts
# $Id: GfalStorm.pm,v 1.1.1.2 2005/02/08 16:09:42 mbarroso Exp $

=head1 SYNOPSIS

Check that globusrun authentication works on the hosts

=head1 DESCRIPTION 

B<GfalStorm.pm> checks that the Globusrun authentication works
for all hosts of the list for the testing user. 
 This test requires that a valid GSI proxy is available for the latter.

=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
    my $self = shift;

    $self->SUPER::setContext();

#    print "DataStorm: setContext\n";
    $self->printLocation( (caller(0))[3] );

    $self->addOption('maxSubs=i',
                     'Number of jobs.',
                     '6');
    $self->addOption('maxStack=i',
                     'Number of simultaneous jobs in the stack.',
                     '2');
    $self->addOption('reqLapse=i',
                     'Delay between 2 submissions (in sec).',
                     '15');
    $self->addOption('serie=i',
                     'Serial number of the storm.',
                     '635');
    $self->addOption ("gfalSize=i", 
		      "Size of the file for checksum test (Mb)", 
		      2000);
#     $self->addOption('circular',
#                      'To scan circularly the CEList built with useCEList');
    $self->addOption('SEtype=s',
		     'To select the SE type where the file is going to be first created and registered (Classic, Castor, dCache, All=NOselection)',
		     'All');


    # Additionnal Commands necessary to run this test.
    my @commands = (
#			  "edg-vo-env",
#			  "edg-gridftp-rm",
#			  "edg-gridftp-ls",
#			  "edg-gridftp-mkdir",
#			  "gdmp_host_subscribe",
#			  "gdmp_job_status",
#			  "globus-replica-catalog",
#			  "ldapsearch",
#			  "edg-replica-manager-replicateFile",
#			  "edg-replica-manager-copyAndRegisterFile",
#			  "edg-replica-manager-deleteFile",
#			  "edg-replica-manager-listReplicas"
		   );
    push @{$self->{commands}}, @commands;

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::Stack::GfalStorm [options]

END
    ;

    $self->{DescText} = <<'END'

 The GfalStorm is similar to a DataStorm in several regards: from
multiple jobs sent to the WNs, a small program, sent thru the input sandbox
and compiled and linked on the WN, exercises several C APIs from the GFAL
library: gfal_open, gfal_read, gfal_write, gfal_close, gfal_unlink,
gfal_stat for reading and writing a rather small file. This does not depend
on the type of the SE choosen (randomly or not) to store the file.
Four --SEtypes are currently available: Classic, Castor, dCache (default: All).

 The created file is currently not replicated to any other SE, and the job 
is declared successful if it was able to write, reread and unlink the file.

 All options available to DataStorm are valid in this case as well,
apart from --lcgcr. In addition, the --gfalSize option allows to specify 
the size (in MB) of the file when writing it.

 A typical submission command used in a regression test is:
opt/edg/bin/MainScript --forcingTB=CertTB --TList=GS_dCache --verbose --forcingVO=dteam \
  --addOptionList="--keepTempDir --maxSubs=10 --useCEList --circular --SEtype=dCache --serie=11018"

END
    ;

	$self->{type} = "GfalStorm";
	$self->{brfinfo} = "GFAL from WNs";
#	 $self->{subDir} = "";
#	 $self->{location} = "";
#	 $self->{VO} = "";
#	 $self->{dataFL} = [];      ## List
#	 $self->{SPs} = {};         ## Hash
	 $self->{SEs} = [];         ## List
#	 $self->{SEpairs} = {};     ## Hash
	$self->{CEs} = [];         ## List
    $self->{bigFile} = "";

    return 0;

}

# Build any necessary setup for the storm.
sub setupEnvironment {
    my $self = shift;

#    print "DataStorm: setupEnvironment\n";
    $self->printLocation( (caller(0))[3] );

## /opt/edg/etc/edg-replica-manager/edg-replica-manager.conf:defaultCE=adc0015.cern.ch

    my @m;
    if ( -e "/opt/edg/var/etc/edg-replica-manager/edg-replica-manager.conf" ) {
	chomp(my $res = `grep defaultSE /opt/edg/var/etc/edg-replica-manager/edg-replica-manager.conf`);
	@m = split /=/, $res;
	print " Found 1: $m[1] \n";
    } 
    if ( ! $m[1] ) {
    if ( -e "/opt/edg/etc/edg-replica-manager/edg-replica-manager.conf.lcfg" ) {
	chomp(my $res = `grep 'DEFAULT.SE' /opt/edg/etc/edg-replica-manager/edg-replica-manager.conf.lcfg`);
	@m = split /\|/, $res;
	print " Found 2: $m[1] \n";
    }
}

## Look up for overall SE declared in the local BDII (assumed to be the first one)...
    my ($bdii, @rest) = split/\s+/, $ENV{IIhostname__};
#    my ($bdii, @rest) = $ENV{IIhostname__};
    my @SEs = `ldapsearch -x -H ldap://$bdii:2170 -b mds-vo-name=local,o=grid | grep 'GlueSEUniqueID:' `;
    my $nSE = scalar(@SEs);
    my $rnd = int(rand($nSE));
    my @s = split/\s+/, $SEs[$rnd];
    print " bdiiSE: $bdii m[1]: $m[1] s[1]: $s[1] SEs: @SEs \n";

## New way ...
    my $snwy;
    my $cetree;
    my $count = -1;
    open NWY, "ldapsearch -x -H ldap://$bdii:2170 -b mds-vo-name=local,o=grid | grep -E 'objectClass: GlueCESEBind\$\|GlueCESEBindCEUniqueID:\|GlueCESEBindSEUniqueID:' |";
    while (<NWY>) {
#p	print " ldapline: $_";
	my @m = split /[:\s]+/;
	if ( /objectClass/ ) { $count++; }
	elsif ( /GlueCESEBindCEUniqueID/ ) { $snwy->{$count}->{CE} = $m[1]; }
	elsif ( /GlueCESEBindSEUniqueID/ ) { $snwy->{$count}->{SE} = $m[1]; }
	if ( $snwy->{$count}->{CE} && $snwy->{$count}->{SE} ) {
	    $cetree->{$snwy->{$count}->{CE}}->{$snwy->{$count}->{SE}}++;
	}
    }
    close NWY;
    foreach my $cet ( sort keys %{$cetree} ) {
	print " CETree: $cet =>";
	foreach my $set ( sort keys %{$cetree->{$cet}} ) {
	    print " $set ";
	}
	print " \n";
    }
## End New Way |

    if ( "@SEs" =~ /$m[1]/ ) {
	$self->{targetSE} = "$m[1]";
    } else {
	$self->{targetSE} = "$s[1]"; ## || "AHAHAH";
    }

    print " TargetSE: =$self->{targetSE}= \n";
    if ( ! $self->{targetSE} ) {  ##  || "=$self->{targetSE}=" eq "==" ) {
	print STDERR " TargetSE: =$self->{targetSE}= \n";
	$self->setErrorMessage('setupEnvironment failed while setting the targetSE which returned empty');
	return 1;
    }	

    return 0;
}

sub dgjobUSELESS {
    my $cmdn = shift;
    my $root = "dg-job-";
    if ( defined $ENV{EDG_FLAVOR} && $ENV{EDG_FLAVOR} ne 'edg1' ) {
	$root = "e".$root;
    }
    return "$root"."$cmdn";
}

# Add any necessary arguments to the existing local JDL file before submission.
sub addArguments {
    my ($self, $jdlTemp, $index) = @_;

#    print "DataStorm: addArguments\n";
    $self->printLocation( (caller(0))[3] );

## Create the checksum file ...
#     $self->{reqs}->{$index}->{bigo} = $self->{tempdir}."/BigOrig-$index.crap";
#     my $checksum1 = $self->createBig($self->{reqs}->{$index}->{bigo});
#     $checksum1 =~ s/ /-/;
#     $checksum1 = "=$checksum1=";
    my $findex = sprintf("%04d", $index);

    my $bid = `echo 'Arguments = "S$ENV{STACK_SERIE} I${findex} $self->{targetSE} $self->{opts}->{SEtype}"; ' >> $jdlTemp`;
#     $bid = `echo '#BigFileName = "mumbojombo"; ' >> $jdlTemp`;
#     $bid = `echo 'InputSandbox = {"$self->{shFile}","$self->{reqs}->{$index}->{bigo}"}; ' >> $jdlTemp`;

if (0) {
    my $nCE = scalar(@{$self->{CEs}});
    my $rnd = int(rand($nCE));
    my $ce;
    if ( $self->{opts}->{circular} ) {
	$ce = ${$self->{CEs}}[($index%$nCE)];
    } else {
	$ce = ${$self->{CEs}}[$rnd];
    }
##    my $ce = "";

    if ( $self->{opts}->{useCEList} ) {
#glue    my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
#norm    my $reqs = 'Requirements = other.CEId=="'.$ce.'"';
	my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
	print STDERR " Requirements: $reqs \n";
	$bid = `echo '$reqs; ' >> $jdlTemp`;
	my @m = split /:/, $ce;
	$self->{reqs}->{$index}->{mace} = "CE:$m[0]";
    } else {
	$self->{reqs}->{$index}->{mace} = "free";
    }
} else {
    $self->chooseCE($index, $jdlTemp);
}
    
    return 0;
}

# Create a BigFile for checksuming ...
sub createBigUNUSED {
    my $self = shift;
    my $bigOrig = shift;

    my $size = $self->{opts}->{cksumSize};  ## in Mb
##    $size = 2;  ## temp
##    my $time = $size*70000;
    open BGF, "> $bigOrig";
    my $bgf = "";
    for ( my $t=0; $t<$size; $t++ ) {
	my $r = rand($t);
	print BGF "$r";
	$bgf .= "$r";
    }
    print BGF "\n";
    $bgf .= "\n";
    my $cumul = 70000;
    for ( my $t=0; $t<$cumul; $t++ ) {
	print BGF "$bgf\n";
    }
    close BGF;
    #system("ls -l $bigOrig; cksum $bigOrig");
    my $checksum = `cksum $bigOrig | cut -d" " -f1-2`;
    chomp($checksum);

    return $checksum;

}


# Prepare everything for running the test itself ...
sub setupTest {
    my $self = shift;

#    print "DataStorm: setupTest\n";
    $self->printLocation( (caller(0))[3] );

    @{$self->{SEs}} = @{$self->{args}};

#    print " Defaults: @{$self->{optionDefaults}} \n";
#    print " Linkage:  @{$self->{optionLinkage}}  \n";
#    exit 0 if (1);

    my $rc = $self->SUPER::setupTest();
    return $rc if $rc;

    ## Possible only when jdlFile and shFile get created
    $rc = $self->buildCEList();
    return $rc if $rc;

#     if ( $self->{opts}->{useCEList} ) {
# 	if ( ! scalar(@{$self->{CEs}}) ) {
# 	    $self->setErrorMessage('Setup failed while building the CEList which returned empty');
# 	    return 1;
# 	}	
#     }

    $self->{zombSlep} = ( $self->{zombSlep} > $self->{reqLapse} ) ? $self->{reqLapse} : $self->{zombSlep};
    print "\nOPTIONS: \n", map { "$_ => $self->{opts}->{$_}\n" } keys %{$self->{opts}};
    print "\n";

    return $rc;
}


sub createJDL {
    my ($self, $file, $script) = @_;

#    print "DataStorm: createJDL\n";
    $self->printLocation( (caller(0))[3] );
    print " file script: $file $script \n";

    my ($sname, $spath, $suffix) = fileparse($script, []);

#    my $file = "$dir/JobDescription.jdl";

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
Executable = "$sname";
InputSandbox = {"$script"};
StdOutput = "result.out";
StdError = "result.err";
#OutputSandbox = {"result.out","result.err", "BrokerInfo"};
OutputSandbox = {"result.out","result.err", "BrokerInfo", "gilb", "core"};
VirtualOrganisation = "$self->{opts}->{vo}";
    Requirements = other.GlueCEPolicyMaxCPUTime<600000;
    Rank = other.GlueCEPolicyMaxCPUTime;
END
    ;

    print JDL "RetryCount = 3;\n";   ##  Special ?

    close JDL;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}

sub createScript {
#   my $dir = shift;
#   my $file = "$dir/JobScript.sh";
    my ($self, $file) = @_;

#    print "DataStorm: createScript\n";
    $self->printLocation( (caller(0))[3] );

    # Write out the job description.
    open SCRIPT, ">$file";
    print SCRIPT <<'END'
#!/usr/bin/perl -w

my ($a, $b, $c, $d) = @ARGV;
print STDERR "GfalStorm: runTest ARGS: $a $b $c $d\n";
print STDOUT "GfalStorm: runTest ARGS: $a JobIndex: $b returnSE: $c SEtype: $d\n";
my $returnSE = $c;
my $SEtype   = $d;

my $res = `cp .BrokerInfo BrokerInfo 2>&1`;
my (@CSE) = `edg-brokerinfo getCloseSEs 2>&1`;
    chomp(my $theCE = `edg-brokerinfo getCE 2>&1 | cut -d":" -f 1`);
    my @t = split /\./, $theCE;
    $theCE = $t[0];
    chomp(my $theWN = `hostname --short 2>&1`);
chomp(my $closeSE = $CSE[0]);

my %cseType = ();
my %cseMP = ();
my %cseFS = ();
$closeSE = "";
foreach my $mycse ( @CSE ) {
    chomp $mycse;
    chomp(my $SEfs = `edg-brokerinfo getSEFreeSpace  $mycse 2>&1`);
    chomp(my $SEmp = `edg-brokerinfo getSEMountPoint $mycse 2>&1`);
    ## flatfile search is useless here ...
    my @ldap = `ldapsearch -x -H ldap://$mycse:2135 -b "mds-vo-name=local,o=grid" | egrep 'castor|pnfs|flatfile' 2>&1`;

my $pnfsO = grep { /pnfs/ } @ldap;
my $castorO = grep { /castor/ } @ldap;
print " MYCSE: $mycse $pnfsO $castorO $SEmp \n";
$cseMP{$mycse} = $SEmp;
$cseFS{$mycse} = $SEfs;

    $cseType{$mycse} = "NADA";
#     if ( $SEfs =~ /\d+/ ) {
#         if ( $SEmp =~ /flatfiles/ || $SEmp !~ /dummy/ ) { $cseType{$mycse} = "Classic"; }
#         else { $cseType{$mycse} = "Castor"; }
#     } else { $cseType{$mycse} = "dCache"; }

    if ( $SEmp =~ /flatfiles/ || $SEmp !~ /dummy/ ) { $cseType{$mycse} = "Classic"; }
    else {
        if ( $pnfsO ) { $cseType{$mycse} = "dCache"; }
        if ( $castorO ) { $cseType{$mycse} = "Castor"; }
    }

    if ( "$mycse" eq "castorgrid.cern.ch" ) { $cseType{$mycse} = "Castor"; }

    print STDOUT " CSEType: $mycse >$cseType{$mycse}< \n";
    if ( $cseType{$mycse} eq "NADA" ) { next; }

###    if ( $closeSE eq "" && $cseType{$mycse} ne "NADA" ) { $closeSE = $mycse; }
##    if ( $closeSE eq "" && $cseType{$mycse} ne "dCache" ) { $closeSE = $mycse; }
    if ( $closeSE eq "" && $cseType{$mycse} eq "dCache" ) { $closeSE = $mycse; }
#    if ( $closeSE eq "" && $cseType{$mycse} ne "Classic" ) { $closeSE = $mycse; }

}

    if (0) {
	my %cseF = (
		    'lxshare0235.cern.ch' => 'lxshare0282.cern.ch',
		    'lxshare0241.cern.ch' => 'lxshare0302.cern.ch',
		    'lxshare0277.cern.ch' => 'lxshare0247.cern.ch',
		    'lxshare0290.cern.ch' => 'lxshare0302.cern.ch'
		    );
	$closeSE = $cseF{$theCE};
    }

    if (0) {
	my %cseF = (
		    'lxshare0235.cern.ch' => 'lxshare0282.cern.ch',
		    'lxshare0241.cern.ch' => 'lxshare0303.cern.ch',
		    'lxshare0277.cern.ch' => 'lxshare0247.cern.ch',
		    'lxshare0290.cern.ch' => 'lxshare0229.cern.ch'
		    );
	$closeSE = $cseF{$theCE};
    }

my %SEofType = reverse %cseType;

if ( $SEtype eq 'All' ) {  ##  GOOD
    my $nCE = scalar(@CSE);
    my $rnd = int(rand($nCE));
    $closeSE = $CSE[$rnd];
} else {
    if ( $SEofType{$SEtype} ) { $closeSE = $SEofType{$SEtype}; }
    elsif ( $SEofType{'Classic'} ) { $closeSE = $SEofType{'Classic'}; }
}

if (1) {
    print STDOUT " CSE: >$closeSE< @CSE ( SEtype: $SEtype TypeOf: $cseType{$closeSE} ) \n";
##TEMP
    if ( scalar(@CSE) >= 2 && $closeSE =~ /lxshare0247|lxshare0282|lxshare0302/ ) {
##REMOVED??   chomp($closeSE = $CSE[1]);
    }
    if ( $closeSE eq "" ) { 
        print STDERR " FAILed: NO closeSE \n";
        exit 1; 
    }
}

chomp(my $myVO = `edg-brokerinfo getVirtualOrganization 2>&1`);
    my $config = "";  ## Still empty
    my $edgRMopts = "$config --insecure --vo=$myVO";
    print STDOUT " GfalStorm: edgRMopts: $edgRMopts WN: $theWN CE: $theCE closeSE: $closeSE returnSE: $c \n";

my ($cres, $file) = createNewData($edgRMopts, $closeSE, $closeSE, $a, $b, "CE$theCE-WN$theWN");

unless ( $cres ) { 
    print " GfalLFN: lfn:$file \n";
    $cres = createCfile("gfal_test.c");
}

unless ( $cres ) {
    print " GfalC: gfal_test.c \n";
    my $res = `gcc -I /opt/lcg/include -L/opt/lcg/lib -lgfal -o gilb gfal_test.c`;
    if ( $res ) { print " GfalC: compil failed\n$res\n"; }
    $cres = $?;
}

unless ( $cres ) {
    print " GfalC: compiled fine \n";
    ## grid or home ?
#OLD    $ENV{RFIO_TRACE} = "3";
    alarm 180;

    my $command = "VIDE";
#NO    my $res = `./gilb -g 56 -R -W $file rfio:////castor/cern.ch/home/dteam/$file`;
#     print " GfalC: cmd: ./gilb -g 56 -R -W $file rfio:////castor/cern.ch/grid/dteam/$file \n";
#     my $res = `./gilb -g 56 -R -W $file rfio:////castor/cern.ch/grid/dteam/$file 2>&1`;
#     $ENV{STAGE_HOST} = "wacdr002d.cern.ch";
#     $ENV{CNS_HOST} = "cnsgrid.cern.ch";
#GOODrfio     my $command = "./gilb -g 56 -R -W $file rfio:////castor/cern.ch/grid/dteam/$file";
    if ( $cseType{$closeSE} eq "Castor" ) {
#<<<<castor      
        $ENV{RFIO_TRACE} = "3";
        ## with / or // after 8443: both work
        $command = "./gilb -g 56 -R -W $file srm://$closeSE:8443//castor/cern.ch/home/dteam/$file";

        if ( $closeSE =~ /castorgrid/ ) {
            $command =~ s/home/grid/;
        }
    } elsif ( $cseType{$closeSE} eq "Classic" ) {
#<<<<classic
# #         my $out = `edg-gridftp-ls gsiftp://$closeSE/$cseMP{$closeSE}/dteam 2>&1`;
# #         print " OUT1: under dteam:\n$out\n";
# #         $out = `edg-gridftp-ls gsiftp://$closeSE/$cseMP{$closeSE}/dteam/generated 2>&1`;
# #         print " OUT2: under generated:\n$out\n";
#old        $command = "./gilb -g 56 -R -W $file sfn://$closeSE//flatfiles/LCG-CERT-SE01/dteam/generated/2004-04-17/$file";
        $command = "./gilb -g 56 -R -W $file sfn://$closeSE/$cseMP{$closeSE}/dteam/$file";
    } elsif ( $cseType{$closeSE} eq "dCache" ) {
#<<<<dcache
        $ENV{DCACHE_DEBUG} = "3";
#         my $out = `edg-gridftp-ls --noauth gsiftp://$closeSE//pnfs/cern.ch/data/dteam 2>&1`;
#         print " OUT1: under dteam:\n$out\n";
        ## with / or // after 8443: both work (hummm: at least currently: both DO NOT work)
        $command = "./gilb -g 56 -R -W $file srm://$closeSE:8443//pnfs/cern.ch/data/dteam/$file";
        $command = "./gilb -g 56 -R -W $file srm://$closeSE:8443/pnfs/cern.ch/data/dteam/$file";
        $command = "./gilb -g 56 -R -W $file srm://$closeSE//pnfs/cern.ch/data/dteam/$file";
        $command = "./gilb -g 56 -R -W $file srm://$closeSE/pnfs/cern.ch/data/dteam/$file";
        $command = "./gilb -g 56 -R -W $file srm://$closeSE:22128/pnfs/cern.ch/data/dteam/$file";
        $command = "ulimit -Sc unlimited; ./gilb -g 56 -R -W $file srm://$closeSE:8443//pnfs/cern.ch/data/dteam/$file";
#1        $command = "./gilb -g 56 -R -W $file srm://lxshare0282.cern.ch:8443//pnfs/cern.ch/data/dteam/$file";
#2        $command = "./gilb -g 56 -R -W $file srm://lxshare0302.cern.ch:8443//pnfs/cern.ch/data/dteam/$file";
#3        $command = "./gilb -g 56 -R -W $file srm://lxshare0247.cern.ch:8443//pnfs/cern.ch/data/dteam/$file";
        ## This work currently works (YES gfal_unlink works) ==> still works fine 26/06/04 !
       $command = "./gilb -g 56 -R -W $file gsidcap://$closeSE:22128//pnfs/cern.ch/data/dteam/$file";
    } else {
        print STDERR " GFAL FAILed: NO closeSE: >$closeSE< Type: >$cseType{$closeSE}< \n";
        exit 1; 
    }

#     my $tmpdir = "$ENV{PWD}";
# #?    my $SE1 = "lxshare0247.cern.ch";
#     my $SE1 = "lxshare0278.cern.ch";
#     my $command = "grid-proxy-info -all; ls -l /tmp; edg-replica-manager $edgRMopts copyAndRegisterFile file:$tmpdir/${file} -l lfn:${file} -d srm://$SE1//castor/cern.ch/grid/dteam";
    print " GfalC: cmd: $command \n";
    my $res = `$command 2>&1`;

    $cres = $?;
    alarm 0;
    if ( $cres ) { 
        print " GfalC: exec failed\n>$cres<\n$res\n"; 
        print " JobLFNF: lfn:$file $cseType{$closeSE} $closeSE \n";
    } else { 
        print " GfalC: exec successed\n$res\n"; 
        print " JobLFN0: lfn:$file $cseType{$closeSE} $closeSE \n";
    }
#     system("ls -l /etc/shift.conf");
#     system("cat /etc/shift.conf");
}

# unless ( $cres ) {
# }

## New to target all WNs ?
sleep 60;

#LIMITmy $service = `echo; date; pwd; host $ENV{HOSTNAME}; host $ENV{HOST}; echo EDG_WL_JOBID: $ENV{EDG_WL_JOBID}; uname -a; echo; ls -l`;
my $service = `echo "soft then hard"; ulimit -Sa; ulimit -Ha; date; pwd; host $ENV{HOSTNAME}; host $ENV{HOST}; echo EDG_WL_JOBID: $ENV{EDG_WL_JOBID}; uname -a; echo; ls -l`;
print STDOUT $service;

exit $cres;

sub createNewData {
    my ($edgRMopts, $SE1, $SE2, $serie, $index, $label) = @_;

#    $self->printLocation( (caller(0))[3] );

    ## Create another temporary data file ...
    chomp (my $date = `date '+%y%m%d-%H%M%S'`);
    my $file = "$ENV{USER}-gfal-$serie-$index-$label-$$-$date.jpb";

    ## Some Pre-Cleaning
#     my $rc = removeGhostFile($edgRMopts, $file);
#     return $rc if $rc;

    # Retrieve the temporary directory.
#    my $tmpdir = $self->{tempdir};
    my $tmpdir = "$ENV{PWD}";

    open TMP, ">$tmpdir/$file";
    ## Useful factor: for a 1Mb file, cumul = 15000.
#    my $cumul = 7000;
    my $cumul = 1;
    for ( my $t=0; $t<$cumul; $t++) {
        print TMP "Good Morning to all Everybody ! This is a GfalStorm. Job# $index $t\n";
    }
    close TMP;

#    my $filer = `ls -l $tmpdir/$file; ls -l /etc/shift.conf; cat /etc/shift.conf; ssh cnsgrid.cern.ch uname -a`;
    my $filer = `ls -l $tmpdir/$file; ls -l /etc/shift.conf; cat /etc/shift.conf;`;
    print " GfalLFN0: $filer \n";

#     $rc = createReplicas ($edgRMopts, $SE1, $SE2, $subDir, $file);
#     return $rc if $rc;

    return (0, "$file");
}

## Dynamic creation of Replicas job by job ...
sub createReplicasUNUSED {

#     my ($self, $SE1, $SE2, $subDir, $file) = @_;
#     my $edgRMopts = $self->{edgRMopts};
    my ($edgRMopts, $SE1, $SE2, $subDir, $file) = @_;

    # Retrieve the temporary directory.
#    my $tmpdir = $self->{tempdir};
    my $tmpdir = "/tmp";

#     my $command = "edg-replica-manager-copyAndRegisterFile -e -l ${subDir}/${file} -s $self->{hostname}//tmp/${file} -d $SE1/$SP1/$file";
#     print " CMD: $command\n";
#     my $resRcCARF = `$command 2>&1`;
#     my $stat = $?;
#     print "\n resRcCARF1: $file -$stat- \n$resRcCARF";
#     if ( $stat != 0 ) {
# 	print STDERR " File: ${subDir}/${file} on: $SE1 not correctly registered or created \n";
# 	return 1;
#     }

    my $command = "edg-replica-manager $edgRMopts copyAndRegisterFile file:$tmpdir/${file} -l lfn:${file} -d $SE1";
    print "\n CMD: $command\n";
    my $resRcCARF = `$command 2>&1`;
    my $stat = $?;
    print "\n resRcCARF1: $file -$stat- \n$resRcCARF";
    if ( $stat != 0 ) {
	print STDERR " File: ${file} on: $SE1 not correctly registered or created \n";
	return 1;
    } else {
	chomp($resRcCARF);
	if ( $resRcCARF !~ "guid:" ) {
	    print STDERR " File: ${file} on: $SE1 : Wrong GUID returned: $resRcCARF \n";
	    return 1;
	}
#	$self->{reqs}->{$stepI}->{olout} = $resRcCARF;
    }

    my %PFNs = ();
    $PFNs{"$SE1/$file"} = "created";

#    my @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
    $command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
    my @resRcLR = `$command 2>&1`;
    print "\n resRcLR1: $file -$?- =$#resRcLR= \n@resRcLR";
    foreach my $l ( @resRcLR ) {
        chomp $l;
        my @i = split /[:\/]/, $l;
#prev	if ( $i[0] ne "srm" ) {
	if ( $i[0] !~ /srm|sfn/ ) {
	    print STDERR " File: ${file} on: $SE1 : Wrong SRM returned: $l \n";
	    return 1;
	} 

	my $gsrm = "$i[3]/${file}";
	my $csrm = "$SE1/${file}";
	if ( $csrm ne $gsrm ) {
	    print STDERR " File: ${file} on: $SE1 : A ghost file still existed: $gsrm \n";
	    return 1;
	}

        if ( ! exists $PFNs{$gsrm} || $PFNs{$gsrm} !~ /created|existing/ ) {
            print STDERR " Step: $SE1 File: $file did not previously existed: $gsrm ! \n";
	    return 1;
        } else {
	    $PFNs{$gsrm} = "existing";
	    print " SRM keys SE1: $SE1 $file value: $l \n";
	}
    }
    if ( $PFNs{"$SE1/$file"} ne "existing" ) {
	print STDERR " Step: $SE1 File: $file not correctly copied/registered ! \n";
	return 1;
    }

if ( "$SE1" ne "$SE2" ) {
#    my $resRcRF = `edg-replica-manager-replicateFile -e -l ${subDir}/${file} -s $SE1 -d $SE2 2>&1`;
    $command = "edg-replica-manager $edgRMopts replicateFile lfn:${file} -d $SE2";
    my $resRcRF = `$command 2>&1`;
    $stat = $?;
    print "\n resRcRF2: $file -$stat- \n$resRcRF";
    if ( $stat != 0 ) {
        print STDERR " File: ${file} on: $SE2 not correctly registered or created \n";
	return 1;
    }
    $PFNs{"$SE2/$file"} = "created";
}

#       @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
    $command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
    @resRcLR = `$command 2>&1`;
    print "\n resRcLR2: $file -$?- =$#resRcLR= \n@resRcLR";
    foreach my $l ( @resRcLR ) {
        chomp $l;
        my @i = split /[:\/]/, $l;
#	if ( $i[0] ne "srm" ) {
	if ( $i[0] !~ /srm|sfn/ ) {
	    print STDERR " File: ${file} on: $SE2 : Wrong SRM returned: $l \n";
	    return 1;
	} 

	my $gsrm = "$i[3]/${file}";
	my $csrm = "$SE2/${file}";
	if ( $csrm ne $gsrm ) {
	    next;
# 	    print STDERR " File: ${file} on: $SE2 : A ghost file still existed: $gsrm \n";
# 	    return 1;
	}

        if ( ! exists $PFNs{$gsrm} || $PFNs{$gsrm} !~ /created|existing/ ) {
            print STDERR " Step: $SE2 File: $file did not previously existed: $gsrm ! \n";
	    return 1;
        } else {
	    $PFNs{$gsrm} = "existing";
	    print " SRM keys SE2: $SE2 $file value: $l \n";
	}
    }

    print "\nPFN0: \n", map { "$_ => $PFNs{$_}\n" } keys %PFNs;
    print "-----------------------------------\n";
    if ( $#resRcLR < 1 || scalar(keys(%PFNs)) != 2 ) {
	print STDERR " Replicates not correctly created on: $SE1 $SE2 \n";
	return 1;
    }

    return 0;

}

sub createCfile {
    my $file = shift;

    # Write out the job C code.
    open GFAL, ">$file";
    print GFAL <<'EOC'
/*
 * Copyright (C) 2003 by CERN Jean-Philippe Baud
 */

#include <fcntl.h>
#include <stdio.h>
#include "gfal_api.h"
#define BLKLEN 65536

#include <unistd.h>
//#include <getopt.h>

extern char* optarg;
extern int optind, opterr, optopt;


main(int argc, char **argv)
{
	int in_fd;
	int out_fd;
	int i, j;
	char ibuf[BLKLEN];
	char obuf[BLKLEN];
	int rc;
	int count;

	int in_fdc, out_fdc;
	char* grogfal;
	int ic;

        struct stat statbuf;

	char gstring[128];
	//char* hstring;

	// Getting options ...
	int iget;
	//	optind = 1;
	char* optiong;

        //fflush(NULL);

	while ( (iget = getopt(argc, argv, "+g:RW")) != EOF) {
	  switch (iget) {
	  case 'g':
	    {
	      optiong = optarg;
	      printf ("option %c %s\n", iget, optiong);
	      break;
	    }
	  case 'R':
	  case 'W':
	    {
	      printf ("option %c\n", iget);
	      break;
	    }
	  default:
	    {
	      /*
	      dummy = optarg;
	      */
	    }
	  }
	}

	/*
	if (optind < argc) {
	  printf ("non-option ARGV-elements: ");
	  while (optind < argc)
	    printf ("%s ", argv[optind++]);
	  printf ("\n");
	}
	*/
	// End of Options stuff

        fflush(NULL);

	if (argc != 2+optind) {
		fprintf (stderr, "usage: %s unix-filename gfal-filename [%d]\n", argv[0], argc);
		exit (1);
	}

	// Test string read ...
        /*
	scanf("%s", gstring);
	printf (" Some string to read: %s \n", gstring);
        */
	// End of Read test

	for (i = 0; i < BLKLEN; i++)
		obuf[i] = i;

	printf ("reading file %s\n", argv[optind]);
	if ((in_fdc = open (argv[optind], O_RDONLY)) < 0) {
		perror ("unix_open");
		exit (1);
	}
	printf ("unix open successful, in_fdc = %d\n", in_fdc);

	//grogfal = "/tmp/grogfal";
	grogfal = "grogfal";
	printf ("creating file %s\n", grogfal);
	if ((out_fdc = open (grogfal, O_WRONLY|O_CREAT, 0644)) < 0) {
		perror ("grogfal_open_write");
		exit (1);
	}
	printf ("grogfal open_write successful, out_fdc = %d\n", out_fdc);

	while ((i = read(in_fdc, obuf, BLKLEN)) > 0 ) {

	  count++;

	  //for ( ic = 0; ic<15000; ic++ ) {
	  for ( ic = 0; ic<150; ic++ ) {
	    if ((rc = write (out_fdc, obuf, i)) != i) {
	      if (rc < 0)
		perror ("write");
	      else
		fprintf (stderr, "write %d returns %d\n", count, rc);
	      (void) close (out_fdc);
	      exit (1);
	    }
	    printf ("grogfal write %d successful_ic\n", ic);
	  }
	  printf ("grogfal write %d successful_count\n", count);

	}

	if ((rc = close (in_fdc)) < 0) {
		perror ("unix_close");
		exit (1);
	}
	printf ("unix close successful\n");

	if ((rc = close (out_fdc)) < 0) {
		perror ("close");
		exit (1);
	}
	printf ("grogfal close successful\n");

	printf ("reading file %s\n", grogfal);
	if ((in_fd = open (grogfal, O_RDONLY)) < 0) {
		perror ("grogfal_open_read");
		exit (1);
	}
	printf ("grogfal open_read successful, in_fd = %d\n", in_fd);

	printf ("writing file %s\n", argv[1+optind]);
	if ((out_fd = gfal_open (argv[1+optind], O_WRONLY|O_CREAT, 0644)) < 0) {
		perror ("gfal_open");
		exit (1);
	}
	printf ("gfal_open successful, out_fd = %d\n", out_fd);
	count = 0;

	while ((i = read(in_fd, obuf, BLKLEN)) > 0 ) {

	  count++;

	  if ((rc = gfal_write (out_fd, obuf, i)) != i) {
	    if (rc < 0)
	      perror ("gfal_write");
	    else
	      fprintf (stderr, "gfal_write %d returns %d\n", count, rc);
	    (void) gfal_close (out_fd);
	    exit (1);
	  }

	}
	printf ("gfal_write %d successful\n", count);

	if ((rc = close (in_fd)) < 0) {
		perror ("unix_close");
		exit (1);
	}
	printf ("unix close successful\n");

	if ((rc = gfal_close (out_fd)) < 0) {
		perror ("gfal_close");
		exit (1);
	}
	printf ("gfal_close successful\n");

	count = 0;

	printf ("reading back %s\n", grogfal);

	if ((in_fd = open (grogfal, O_RDONLY, 0)) < 0) {
		perror ("unix_open");
		exit (1);
	}
	printf ("unix open successful, in_fd = %d\n", in_fd);

	if ((out_fd = gfal_open (argv[1+optind], O_RDONLY, 0)) < 0) {
		perror ("gfal_open");
		exit (1);
	}
	printf ("gfal_open successful, out_fd = %d\n", out_fd);

	while ((i = read(in_fd, ibuf, BLKLEN)) > 0 ) {

	  count++;

	  if ((rc = gfal_read (out_fd, obuf, i)) != i) {
	    if (rc < 0)
	      perror ("gfal_read");
	    else
	      fprintf (stderr, "gfal_read %d returns %d\n", count, rc);
	    (void) gfal_close (out_fd);
	    exit (1);
	  }
	  //printf ("gfal read %d successful\n", count);

	  for (j = 0; j < i; j++) {
	    if (ibuf[j] != obuf[j]) {
	      fprintf (stderr, "compare %d failed at offset %d\n", count, j);
	      exit (1);
	    }
	  }

	}
	printf ("compare %d successful\n", count);

	if ((rc = close (in_fd)) < 0) {
		perror ("unix_close");
		exit (1);
	}
	printf ("unix close successful\n");

	if ((rc = gfal_close (out_fd)) < 0) {
		perror ("gfal_close");
		exit (1);
	}
	printf ("gfal_close successful\n");

    // grid or home ?
//	rc = system("nsls -l /castor/cern.ch/home/dteam");
//	rc = system("nsls -l /castor/cern.ch/grid/dteam");

        if (gfal_stat (argv[1+optind], &statbuf) < 0) {
                perror ("gfal_stat");
                exit (1);
        }

        printf ("\ngfal_stat successful\n");
        printf ("mode = %o\n", statbuf.st_mode);
        printf ("nlink = %d\n", statbuf.st_nlink);
        printf ("uid = %d\n", statbuf.st_uid);
        printf ("gid = %d\n", statbuf.st_gid);
        printf ("size = %ld\n", statbuf.st_size);

	rc = unlink(grogfal);

	if ((rc = gfal_unlink(argv[1+optind])) < 0) {
		perror ("gfal_unlink");
		exit (1);
	}
	printf ("gfal_unlink successful\n");

	exit (0);
}

EOC
    ;
    close GFAL;

    # Ensure that the script is executable.
#    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
#        return $file;
        return 0;
    } else {
        return 1;
    }
}

sub removeGhostFileUNUSED {

#     my ($self, $file) = @_;
#     my $edgRMopts = $self->{edgRMopts};
    my ($edgRMopts, $file) = @_;

#    $self->printLocation( (caller(0))[3] );

##	    foreach my $loop ( 0..2 ) {
##		my $file = "$ENV{USER}-$upid-$loop-rmstorm.txt";

		my $command = "edg-replica-manager $edgRMopts listGUID lfn:${file}";
		chomp(my $resRcLR = `$command 2>&1`);
		my $stat = $?;
		print "$stat\t                                   $resRcLR\n";
#		if ( $stat ) { next; }
    return 0 if $stat;

		$command = "edg-replica-manager $edgRMopts deleteFile --all-available --all-lfn $resRcLR";
		my $resRcDF = `$command 2>&1`;
		$stat = $?;
		print "\n resRcDF: $file -$stat- \n$resRcDF\n";
    return $stat if $stat;

		$command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
		my $resRcLRafter = `$command 2>&1`;
		$stat = $?;
		print "\n resRcLRafter: $file -$stat- \n$resRcLRafter";
#    return $stat if $stat;

		$command = "edg-replica-manager $edgRMopts listReplicas $resRcLR";
		my $resRcLGafter = `$command 2>&1`;
		$stat = $?;
		print "\n resRcLGafter: $file -$stat- \n$resRcLGafter";
#    return $stat if $stat;
##	    }

    return 0;

}

__DATA__

#additional storage here

END
    ;
    close SCRIPT;

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}

# This will run the evaluateOutput method on all of the jobs.  The
# evaluateOutput method may be overridden to achieve a different
# evaluation. 
sub evaluateTest {
    my $self = shift;
    my %cseLab = (
		   'castor'    => 'Castor',
		   'pnfs'      => 'dCache',
		   'flatfiles' => 'Classic'
		  );

#    print "DataStorm: evaluateTest\n";
    $self->printLocation( (caller(0))[3] );

    my $config = "";  ## Still empty
    my $edgRMopts = "$config --insecure --vo=$self->{opts}->{vo}";

    ## Implement proper parsing of output files, and subsequent evaluation here.
#     foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
# 	print " JOBL\t$index :\n$self->{reqs}->{$index}->{olout} \n";
#     }

    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
	print " JOBL\t$index :\n$self->{reqs}->{$index}->{olout} \n";
#p	print " JOUT\t$index :\n".`cat $self->{reqs}->{$index}->{olout}`." \n" if $self->{reqs}->{$index}->{olout};
	next unless $self->{reqs}->{$index}->{olout};
	chomp(my $res = `grep JobLFN0 $self->{reqs}->{$index}->{olout}`);
	chomp(my $ref = `grep JobLFNF $self->{reqs}->{$index}->{olout}`);

	my @m = ();
	if ( $res ) {
	@m = split /\s+/, $res;
	print " JLFN:\t$m[2] \n";
	$self->{reqs}->{$index}->{stat1} = '[OK]';
	$self->{reqs}->{$index}->{stat2} = "$m[3]";
	$self->{reqs}->{$index}->{cse} = "$m[4]";
    } else {
	my @mm = split /\s+/, $ref;
	$self->{reqs}->{$index}->{stat1} = '[FAIL]';
	$self->{reqs}->{$index}->{stat2} = "$mm[3]";
	$self->{reqs}->{$index}->{cse} = "$mm[4]";
		$self->{reqs}->{$index}->{olog} = $self->{reqs}->{$index}->{olout};
    }

	if (0) {
	my $file = $m[2];
	my $match = 0;
	my $command = "edg-replica-manager $edgRMopts listReplicas ${file}";
	my @resRcLR = `$command 2>&1`;
	print "\n resRcLR0: $file -$?- =$#resRcLR= \n@resRcLR";
	if ( ! $? ) {
	foreach my $l ( @resRcLR ) {
	    chomp($l);
	    if ( $l =~ "$self->{targetSE}" ) {
		print " Matching: [OK] $l \n";
		$match++;
	    }
	}
	chomp($res = `grep 'SRM keys SE1:' $self->{reqs}->{$index}->{olout}`);
	my @m = split /\s+/, $res;
#	@m = split /\./, $m[4];
	$self->{reqs}->{$index}->{cse} = $m[4];
	$res =~ /.*\/(castor|pnfs|flatfiles)\/.*/;
	$self->{reqs}->{$index}->{stat2} = $cseLab{$1};
    }
##test	if ( $index == 1 ) { $match = 10; }  ##  TEMP TEST
	if ( $match != 1 ) { print " Matching: [FAILED] \n"; }
#old	$self->{reqs}->{$index}->{match} = $match;
	if ( $self->{reqs}->{$index}->{stat1} eq '[OK]' ) {
	    if ( $match != 1 ) {
		$self->{reqs}->{$index}->{stat1} = '[FAIL]';
		$self->{reqs}->{$index}->{stat2} = 'MATCHING';
		$self->{reqs}->{$index}->{olog} = $self->{reqs}->{$index}->{olout};
	    }
	}
    }

#p	print " SUMM\t$index :\n", map { "\t$_\t=>\t$self->{reqs}->{$index}->{$_} \n" } sort keys %{$self->{reqs}->{$index}};
    }

    my $rc = $self->SUPER::evaluateTest();
    return $rc if ($rc);

    return 0;

}

sub buildXMLFileNOTANYMORE {
 
    my $self = shift;

#    print "Stack: buildXMLFile\n";
#    $self->{debug} = 1;
    $self->printLocation( (caller(0))[3] );

    my @mopts = split /,/, $self->{opts}->{display};

    ## Default: several columns, prefixed:mace as a col index, mawn as a row index
    if ( ! scalar(@mopts) ) { $self->SUPER::buildXMLFile(); }
    elsif ( $mopts[0] eq "1Col" ) {
	## this arg arg may be: "jind", "mace", "mawn" (as a row index)
	## "jind" stands for Job_index
	$self->SUPER::buildXMLFile1Col($mopts[1]);
    } else {
	## Now, 1st arg may be: "mace", "mawn" (as a col index)
	## while 2nd arg may be: "jind", "mace", "mawn" (as a row index)
	$self->SUPER::buildXMLFile($mopts[0], $mopts[1]);
    }
    return 0;

}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
