#!/usr/bin/perl

package BaseTest::RMSConsistencyCheck;

use strict;
use vars qw(@ISA);
use EDGUtils;
use TestUtils;
@ISA=qw(BaseTest);

=head1 NAME

RMSConsistencyCheck.pm - Check that reverse DNS lookups of host names works

=head1 SYNOPSIS

blah

=head1 DESCRIPTION 

blah

=head1 AUTHOR

blah

=head1 LICENSE

blah

=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    $self->printMethod("setContext");

    $self->addOption('log=s','Logfile','output.log');
    $self->addOption('vo=s','Virtual Organisation','wpsix');
    $self->addOption('debug:i','Debug flag: 0=>Silent, 1=>Verbose (Default 0)',0);
    $self->addOption('rmc=s','RMC host');
    $self->addOption('port=s','RMC port','8080');

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::RMSConsistencyCheck [options] <hostname> [hostname ...]

END
    ;

    $self->{DescText} = <<'END'

    blah
END
    ;

}

sub checkPrerequisites {
  my $self = shift;
  my $rc = $self->SUPER::checkPrerequisites();
  return $rc if ($rc);

  $self->printMethod("checkPrerequisites");

  # Ensure that the mandatory options have been given
  if (!defined $self->{opts}->{vo}) {
      $self->setErrorMessage("Mandatory option -vo was not provided");
      return 1;
  };

  if (!defined $self->{opts}->{rmc}) {
      $self->setErrorMessage("Mandatory option -rmc was not provided");
      return 1;
  }
  # Check that Grid proxy exists
  my $rc = checkProxyExists();
  $self->setErrorMessage('user proxy does not exist') if ($rc);

  return $rc;
}

sub setupTest {
    my $self = shift;

    $self->printMethod("setupTest");

    system ("rm -rf $self->{opts}->{log} ; touch  $self->{opts}->{log}");
    print "Logfile $self->{opts}->{log} created\n";

    return 0;
}


sub runTest {
    my $self = shift;

    print "RMSConsistencyCheck: runTest\n";

    my $href=$self->runCommand("edg-rmc -i mappingsByGuid '*' -h $self->{opts}->{rmc} -p $self->{opts}->{port}");

# Okies, ara tenim els mappings dels Guid aqui... Toca fer un split

    my $hraliases;
    my $retval=0;
    foreach my $line (split(/\n/,$href->{out})) {if ($line =~ /([^,]+),\s*(.+)/) {$hraliases->{$1}=$2};};
    foreach my $GUID (keys %$hraliases)
    {
        my $schema;
        my $lfn_schema;
        my $valid_guid=0;

	print ("\nTesting... $GUID\n");
	if ($GUID =~ /([^:]+):/) {$schema=$1};
	if ($GUID =~ /[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}/) {$valid_guid=1};
	if ($schema ne "guid") {print("ERROR GUID schema is invalid\n"); $retval=1;};
	if ($valid_guid == 0) {print("ERROR GUID is invalid\n"); $retval=1;};

	print ("- $hraliases->{$GUID}\n");
        if ($hraliases->{$GUID} =~ /([^:]+):/) {$lfn_schema=$1};
	if ($lfn_schema ne "lfn") {print("ERROR LFN schema is invalid\n"); $retval=1;};

#	my $href=$self->runCommand("edg-rm -i --vo=$self->{opts}->{vo} lg $hraliases->{$GUID}");
#	if ($GUID ne $href->{out}) {print("ERROR GUID LFN inconsistency\n"); $retval=1;};
#	print ("$href->{out}\n");
#	my $href=$self->runCommand("edg-rm -i --vo=$self->{opts}->{vo} lr $GUID");
#	print ("$href->{out}\n");

	my $hrefsfns=$self->runCommand("edg-lrc -i pfnsForGuid $GUID -h $self->{opts}->{rmc} -p $self->{opts}->{port}");
	if ($hrefsfns->{out} =~ "No such guid") { print ("ERROR No SFN for this GUID\n"); $retval=1;}
	else {
	    foreach my $sfn_location (split(/\n/,$hrefsfns->{out})) 
	    {
		my $sfn_schema;
		print ("* $sfn_location\n");
		if ($sfn_location =~ /([^:]+):/) {$sfn_schema=$1};
		if ($sfn_schema ne "sfn") {
		    print("ERROR SFN schema \"$sfn_schema\" is invalid\n");
		    $retval=1;
		    }
		else {
		    my $host_name="";
		    if ($sfn_location =~ /sfn:\/\/([^\/]+)\//) {$host_name=$1};
		    if ($host_name eq "") { print ("ERROR SRM Server invalid\n"); $retval=1;}
		}
	    }
	}
#	print ("$href->{out}\n");
    };
    return $retval;
}

# Utility function to issue a command
# What it does:
#   1-Writes the issued command to a tmp logfile
#   2-Issues the command with a timeout
#   3-Appends the output of this command to the main test logfile $self->{opts}->{log}
# Arguments:
#   $cmd: String with the command to be issued
#   $logflag: Flag to indicate wether to append the result to the main log (logflag=1, default) or not (logflag=0)
# Returns a hash ref with keys:
#   rc: Return code of the issued command
#   out: stdout+stderr of the issued command

sub runCommand {
  my $self = shift;
  my ($cmd,$logflag) = @_;

  if (!defined $logflag) {$logflag = 1};

  return 1 if ( ! defined $cmd );

#  print "runCommand(): Now running: \"$cmd\" \n";

  my $head = "\n";
  $head .= "*************************************** \n";
  $head .= "* Log for: $cmd \n";
  $head .= "*************************************** \n";

  my $href = mytimeout($cmd);

  # Finally, if temporary log file name was given, append it to the main test logfile
  if ( $logflag != 0 ) {
    open LOG, ">> $self->{opts}->{log}";
    print LOG $head;
    print LOG $href->{out};
    close LOG;
  }

  return $href;
}

sub printMethod {
  my $self = shift;

  if ($self->{debug} > 0){
    my ($method) = @_;
    my $class = ref $self;
    print "${class}::${method}()\n";
  }

  return 0;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
