#!/usr/bin/perl -w

package BaseTest::EDGLifecycle::RMSetupTest;

use strict;
no strict 'refs';

use vars qw(@ISA);
use EDGUtils;
use TestUtils;

use File::Basename;

@ISA=qw(BaseTest::EDGLifecycle);

=head1 NAME

RMSetupTest.pm - Check that globusrun authentication works on the hosts
# $Id: RMSetupTest.pm,v 1.1.1.2 2005/02/08 16:09:42 mbarroso Exp $

=head1 SYNOPSIS

Check that globusrun authentication works on the hosts

=head1 DESCRIPTION 

B<RMSetupTest.pm> checks that the Globusrun authentication works
for all hosts of the list for the testing user. 
 This test requires that a valid GSI proxy is available for the latter.

=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "RMSetupTest: setContext\n";

    # Add the resource option.
#     $self->addOption('mds=s',
#                      'The top-level mds node.',
#                      'gppinfo06.gridpp.rl.ac.uk');
#     $self->addOption('mdsport=i',
#                      'The top-level mds port number.',
#                      '2135');
#     $self->addOption('mdsname=s',
#                      'The mds-vo-name for MDS.',
#                      'local');
    $self->addOption('bdii=s',
                     'The default BDII node.',
                     'ccedgli02.in2p3.fr');
    $self->addOption('bdiiport=i',
                     'The BDII port number.',
                     '2170');
    $self->addOption('bdiiname=s',
                     'The mds-vo-name for BDII.',
                     'local');
    $self->addOption('VO=s',
		     'VO to use in the repStorm.',
                     'dteam');

  # Add the filesize option
    $self->addOption('filesize=i',
		     'Size in kB of the file used for replication. (Default: 256)',
		     256);
    $self->addOption('lcgcr',
                     'To use lcg-cr command instead of edg-rm for the cr operation when set.');

    # The command needed to run this test.
    $self->{commands} = ["grid-proxy-info",
#			 "edg-vo-env",
			 "edg-gridftp-rm",
                         "edg-gridftp-ls",
			 "edg-gridftp-mkdir",
#p			 "gdmp_ping",
# 			 "gdmp_host_subscribe",
# 			 "gdmp_job_status",
# 			 "globus-replica-catalog",
# 			 "ldapsearch",
# 			 "edg-replica-manager-replicateFile",
# 			 "edg-replica-manager-copyAndRegisterFile",
# 			 "edg-replica-manager-deleteFile",
# 			 "edg-replica-manager-listReplicas"
 			 "edg-replica-manager"
			 ];

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::RMSetupTest [options] <server node> [server node...] 

END
    ;

    $self->{DescText} = <<'END'
NEEDS TO BE REWRITTEN ...
Check the basic setup of an SE.  This test checks:
 * That the SE GRIS is running.
 * Supported VO(s) are defined in the GRIS.
 * The GridFTP server is running.
 * The storage path exists.
 * The "magic" softlink exists for NFS access (via BrokerInfo).
 * The edg_query_vo_storage cron job is running.
This does not check the data management software or configuration.

This test was supplied by Stephen Burke. 

END
    ;

}


sub checkPrerequisites {
    my $self = shift;
    my $rc = $self->SUPER::checkPrerequisites();
    return $rc if ($rc);

    print "RMSetupTest: checkPrerequisites\n";

    my $rc = checkProxyExists();
    $self->setErrorMessage('user proxy does not exist') if ($rc);

    return $rc;
}


sub runTest {
    my $self = shift;

    if ( $self->{opts}->{xml} ) {

 	my $base = "/tmp/$ENV{USER}-$$";

	$self->{reqs} = {};    ## Hash
	$self->{steps} = {};    ## Hash
	$self->{mach} = {};    ## Hash
	$self->{case} = {};    ## Hash

	my $package = __PACKAGE__;
	my @m = split /::/, $package;
	$self->{type} = "$m[$#m]";
	$self->{xmlfile}  = "$base-".$self->{type}.".XML";
	## To trigger the processing thru buildXMLParser
	$self->{xmlxist}  = "$self->{type}";

	my $html_dir = ( defined $ENV{HTML_DIR} ) ? "$ENV{HTML_DIR}" : "$ENV{HOME}/TSTG/HTML";
	$html_dir = "${html_dir}/$self->{type}";
	my $resmk = `mkdir -p $html_dir`;
	$self->{htmldir}  = "$html_dir";

    }

    print "RMSetupTest: runTest\n";

    my $rc;
    my @SEs;
    my %SPs = {};
    if (0) {
	($rc, @SEs) = getSEs($self);
	return $rc if $rc;

	($rc, %SPs) = getSPs($self, @SEs);
	return $rc if $rc;
    } else {
	@SEs = @{$self->{args}};
    }

#   exit 0;

    # Keep a list of all SEs which fail
    my @failed;

#     # Loop over each SE pair doing a full test for each.
#     for my $n ( 0 .. ($#SEs-1) ) {
# 	my @SEpair = ($SEs[$n], $SEs[$n+1]);
#         my $error = $self->testRMSetupTest(\@SEpair, \%SPs);
#         push @failed, @SEpair if ($error);
#     }

  @{$self->{ARGSLIST}} = @{$self->{args}};
  my @aColsList = ();
  my @aRowsList = ();

    # 2D test
    @aColsList = @{$self->{ARGSLIST}};
    @aRowsList = @{$self->{ARGSLIST}};

  # Build the table:
    my $index = 0;
  foreach my $row (@aRowsList) {
    foreach my $col (@aColsList) {
      if ($row eq $col) {
	# For the diagonal elements, skip for the time being
	  next;
      } else {
	  my @pair = ($row, $col, $index++);
	  push @{$self->{pairh}}, \@pair;
#p	  print STDOUT " Adding: @pair \n";
      }
    }
  }


  my ($pidref, $phref) = $self->parallelSubmit(@{$self->{pairh}});
  my $resMonit = $self->parallelMonit($pidref, $phref);

    # If there are any servers in the list, then the test has failed.
    if ($#failed>=0) {
        my $names = join(",",@failed);
        $self->setErrorMessage("RMSetupTest test failed for $names");
        return 1;
    }

    return 0;
}


## 
## The actual test is here.  It just takes a server name, extracts the
## supported VOs and runs checks. 
##

sub testRMSetupTestOLD {
    my ($self, $arrayref, $hashref) = @_;
    my @SEpair = @$arrayref;
    my %SPs = %$hashref;
    my $rc = 0;
    print "  Working this SE pair: @SEpair with these mount points: \n";
    foreach my $se ( @SEpair ) { 
	print "     FS: $se => $SPs{$se} \n";
    }

    my $subDir = "testGG";
    my $edgRMopts = "--config=edg-replica-manager-CertTB.conf --insecure --vo=wpsix";
#    my $edgRMopts = "--vo=atlas --verbose";

    $rc = mutualSPs($subDir, \@SEpair, \%SPs) if (0);
    return $rc if $rc;

    ## Create a temporary file ...
    my $file = "$ENV{USER}-$$-rmstorm.txt";
    open TMP, ">/tmp/$file";
    print TMP "Good Morning to all Everybody !\n";
    close TMP;

    my $nn = int(rand(2));
    my $SE1 = $SEpair[$nn];
    my $SE2 = $SEpair[1-$nn];
    my $SP1 = "$SPs{$SE1}/$subDir" if (0);
    my $SP2 = "$SPs{$SE2}/$subDir" if (0);
    chomp(my $hostname = `hostname`);

    ## Initialization of the location stuff ...
    my $needed = 0;
    if ( $needed ) { 
	$rc = locationStuff ($SE1, $SE2, $SP1, $SP2, $subDir, $file, $hostname);
	return $rc if $rc;
    }

##edg2.0    my $command = "edg-replica-manager-copyAndRegisterFile -e -l ${subDir}/${file} -s ${hostname}//tmp/${file} -d $SE1/$SP1/$file";
    my $command = "edg-replica-manager $edgRMopts copyAndRegisterFile file:/tmp/${file} -l lfn:${file} -d $SE1";
    print " CMD: $command\n";
    my $resRcCARF = `$command 2>&1`;
    my $stat = $?;
    print "\n resRcCARF1: $file -$stat- \n$resRcCARF";
    if ( $stat != 0 ) {
##edg2.0	print STDERR " File: ${subDir}/${file} on: $SE1 not correctly registered or created \n";
	print STDERR " File: ${file} on: $SE1 not correctly registered or created \n";
	return 1;
    } else {
	chomp($resRcCARF);
	if ( $resRcCARF !~ "guid:" ) {
	    print STDERR " File: ${file} on: $SE1 : Wrong GUID returned: $resRcCARF \n";
	    return 1;
	}
    }
    my %PFNs = ();
    my %GUIDs = ();
    my %SRMs = ();
    $GUIDs{$file} = $resRcCARF;
#    $PFNs{"$SE1/$SP1/$file"} = "created";
    $PFNs{"$SE1/$file"} = "created";

##edg2.0    my @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
    my @resRcLR = `edg-replica-manager $edgRMopts listReplicas lfn:${file} 2>&1`;
    print "\n resRcLR1: $file -$?- =$#resRcLR= \n@resRcLR";
    foreach my $l ( @resRcLR ) {
        chomp $l;
##edg2.0        my @i = split /: /, $l;
        my @i = split /[:\/]/, $l;
	if ( @i[0] ne "srm" ) {
	    print STDERR " File: ${file} on: $SE1 : Wrong SRM returned: $l \n";
	    return 1;
	} 

	my $gsrm = "$i[3]/${file}";
        if ( ! exists $PFNs{$gsrm} || $PFNs{$gsrm} !~ /created|existing/ ) {
            print STDERR " Step: $SE1 File: $gsrm previously existed: $l ! \n";
	    return 1;
        } else {
	    $PFNs{$gsrm} = "existing";
	    $SRMs{$l} = "existing";
	}
    }
    if ( $PFNs{"$SE1/$file"} ne "existing" ) {
	print STDERR " Step: $SE1 File: $file not correctly copied/registered ! \n";
	return 1;
    }

##edg2.0    my $resRcRF = `edg-replica-manager-replicateFile -e -l ${subDir}/${file} -s $SE1 -d $SE2 2>&1`;
    my $resRcRF = `edg-replica-manager $edgRMopts replicateFile lfn:${file} -d $SE2 2>&1`;

#GOOD    my $resRcRF = `edg-replica-manager-replicateFile -e -l ${subDir}/${file} -s $SE1/$SP1/$file -d $SE2/$SP2/$file 2>&1`;
#GOOD!    my $resRcRF = `edg-replica-manager-replicateFile -e -l ${subDir}/${file} -s $SE1 -d $SE2/$SP2/$file 2>&1`;
#BAD    my $resRcRF = `edg-replica-manager-replicateFile -e -l ${subDir}/${file} -s $SE1/$SP1/$file -d $SE2 2>&1`;
    $stat = $?;
    print "\n resRcRF2: $file -$stat- \n$resRcRF";
    if ( $stat != 0 ) {
##edg2.0        print STDERR " File: ${subDir}/${file} on: $SE2 not correctly registered or created \n";
        print STDERR " File: ${file} on: $SE2 not correctly registered or created \n";
	return 1;
    } else {
	chomp($resRcRF);
	if ( $resRcRF !~ "srm:" ) {
	    print STDERR " File: ${file} on: $SE2 : Wrong SRM returned: $resRcRF \n";
	    return 1;
	} else {
	    $PFNs{"$SE2/$file"} = "created";
	    $SRMs{$resRcRF} = "existing";
	}
    }
##    $PFNs{"$SE2/$SP2/$file"} = "created";

##edg2.0       @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
       @resRcLR = `edg-replica-manager $edgRMopts listReplicas lfn:${file} 2>&1`;
    print "\n resRcLR2: $file -$?- =$#resRcLR= \n@resRcLR";
    foreach my $l ( @resRcLR ) {
	chomp $l;
##edg2.0	my @i = split /: /, $l;
        my @i = split /[:\/]/, $l;
	if ( @i[0] ne "srm" ) {
	    print STDERR " File: ${file} on: $SE2 : Wrong SRM returned: $l \n";
	    return 1;
	} 

	my $gsrm = "$i[3]/${file}";
	if ( ! exists $PFNs{$gsrm} || $PFNs{$gsrm} !~ /created|existing/ ) {
	    print STDERR " Step: $SE2 File: $gsrm previously existed: $l ! \n";
	    return 1;
	} else {
	    $PFNs{$gsrm} = "existing";
	    if ( ! $SRMs{$l} ) {
		print STDERR " Step: $SE2 SRM: $l not previously existing ! \n";
		return 1;
	    }
	}
    }

    print "\nPFN0: \n", map { "$_ => $PFNs{$_}\n" } keys %PFNs;
    print "-----------------------------------\n";
    if ( $#resRcLR < 1 || scalar(keys(%PFNs)) != 2 ) {
	print STDERR " Replicates not correctly created on: $SE1 $SE2 \n";
	return 1;
    }

    exit 0 if (0);  ##  TEMP


    ## Section where the JDL job is being run ...

    ## Adding required options to JDL file ...
    my $resE = `echo '    InputData = "lfn:${subDir}/${file}";' >> $self->{jdl}`;
    $resE =    `echo '    Arguments = "${file}";'    >> $self->{jdl}`;
##edg2.0    $resE =    `echo '    ReplicaCatalog = "$ENV{RC_LOGICAL_COLLECTION}";' >> $self->{jdl}`;


    my $resJob = $self->SUPER::runTest();
    if ( $resJob ) { print " RMSetupTest::Job : Failed ! \n"; }
    ## Temp ...
    if (1) { exit 0;}

    my @resRcDF = grep { /deleting/ } `edg-replica-manager-deleteFile -e -a -l ${subDir}/${file} 2>&1`;
    print "\n resRcDF3: $file -$?- =$#resRcDF= \n@resRcDF";
    foreach my $l ( @resRcDF ) {
	chomp $l;
        my @i = split /deleting /, $l;
        $PFNs{$i[1]} = "deleting";
    }

       @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
    print "\n resRcLR3: $file -$?- =$#resRcLR= \n";
    if ( $#resRcLR > -1 ) {
	foreach my $l ( @resRcLR ) {
	    chomp $l;
	    my @i = split /: /, $l;
	    $PFNs{$i[1]} = "still-existing";
	}
    }

    print "\nPFNs: \n", map { "$_ => $PFNs{$_}\n" } keys %PFNs;
    print "-----------------------------------\n";
    foreach my $se ( $SE1, $SE2 ) {
	my $resDel = `edg-gridftp-rm gsiftp://${se}$SPs{$se}/$subDir`;
    }

## To delete a location:
## source grc.conf
## 1183  globus-replica-catalog -host "$RC_LOGICAL_COLLECTION" -manager "$RC_REP_CAT_MANAGER_DN" -password pwd.conf  -location lxshare0232.cern.ch -delete
## 1184  globus-replica-catalog -host "$RC_LOGICAL_COLLECTION" -manager "$RC_REP_CAT_MANAGER_DN" -password pwd.conf  -location lxshare0303.cern.ch -delete
    unlink "/tmp/$file";

    return $rc;
}


# The setup creates a temporary directory to be used to hold the job's
# files (JDL and scripts).  It will also be used to hold the retrieved
# job output.  (The key in the context is 'tempdir'.)
sub setupTest {
    my $self = shift;

    print "RMSetupTest: setupTest\n";

    # Call the parent method to ensure that the temporary directory is
    # created.
    my $error = $self->SUPER::setupTest();
    if ($error) {
        $self->setErrorMessage('default setup failed');
        return 1;
    }

    # Retrieve the temporary directory.
    my $tmpdir = $self->{tempdir};
#p  print " tmpdir: $tmpdir \n";

    # Create the script for this job.
    my $script = createScript($tmpdir);
    if (! $script) {
        $self->setErrorMessage('could not create job script');
        return 1;
    }

    # Create the JDL file.
    my $file = $self->createJDL($tmpdir, $script);
    if ($file) {
        $self->{jdl} = $file;
    } else {
        $self->setErrorMessage('could not create JDL file');
        return 1;
    }

    return 0;
}


##
## Below here are utility functions for supporting the methods above.
## These should not need to be called by subclasses.
##


sub createJDL {
    my ($self, $dir, $script, $site) = @_;

    my ($sname, $spath, $suffix) = fileparse($script, []);

    my $file = "$dir/JobDescription.jdl";

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
    Executable    = "$sname";
    StdOutput     = "std.out";
    StdError      = "std.err";
    InputSandbox  = {"$script"};
    OutputSandbox = {"std.out","std.err"};
#   Executable = "World.sh";
#   Arguments = "test/HelloWorld";
##  Arguments = "$ENV{HW_INP}";
    StdInput = "/dev/null";
#   StdOutput = "HW.out";
#   StdError = "HW.err";
#   InputSandbox = "World.sh";
#   OutputSandbox = {"HW.out", "HW.err"};
#   InputData = "LF:test/HelloWorld";
##  InputData = "LF:$ENV{HW_INP}";
#   ReplicaCatalog = "ldap://testbed014.cern.ch:12389/lc=TestCollection,rc=ITeamRC,dc=testbed014,dc=cern,dc=ch";
#    ReplicaCatalog = "$ENV{RC_LOGICAL_COLLECTION}";
    DataAccessProtocol = "file";
END
    ;

    print JDL "Requirements = $self->{opts}->{requirements};\n"
        if $self->{opts}->{requirements};
    print JDL "Rank = $self->{opts}->{rank};\n"
        if $self->{opts}->{rank};
    print JDL "RetryCount = $self->{opts}->{retrycount};\n" 
	if ($self->{opts}->{retrycount}>=0);
    print JDL "MyProxyServer = \"$self->{opts}->{myproxyhost}\";\n" 
	if $self->{opts}->{myproxyhost};
    close JDL;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


sub createScript {
    my $dir = shift;

    my $file = "$dir/JobScript.sh";

    # Write out the job description.
    open SCRIPT, ">$file";
    print SCRIPT <<'END'
#!/bin/bash
echo Arg: $1
GSF=`edg-brokerinfo getSelectedFile $1 file`
echo GSF: $GSF
TFN=`echo $GSF | cut -d " " -f 2`
echo Printing local file $TFN:
cat $TFN
echo "+-+-+-+-+-+-+-+-"
/bin/uname -a
/bin/echo "Hello From Sgript"
/bin/echo "Goodbye. [OK]"
GSE=`edg-brokerinfo getSEs`
echo SEs: $GSE
CSE=`edg-brokerinfo getCloseSEs`
echo getCloseSEs: $CSE
CSE=`echo $CSE | cut -d " " -f 1`
SMP=`edg-brokerinfo getSEMountPoint $CSE`
echo getSEMountPoint: $SMP
END
    ;
    close SCRIPT;

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


## To extract the available SE list for a given VO 
sub getSEsOLD {
    my $self = shift;

    ## Set up some variables ...
#   my $edgl = `$ENV{EDG_LOCATION}/bin/edg-vo-env --shell=csh tstg`;
    my $edgl = `edg-vo-env --shell=csh $self->{opts}->{VO}`;
#p  print " EDGL: $edgl \n";
    my @ls = split /\n/, "$edgl";
    foreach my $l ( @ls ) {
	my @its = split /\s+/, $l;
	chop($its[2]);
	$ENV{$its[1]} = $its[2];
	print " ENV $its[1]: $ENV{$its[1]} \n";
    }
    my $gdmb = dirname( dirname( $ENV{RC_CONFIG_FILE}));
#   my $gdmb = basename( $ENV{RC_CONFIG_FILE});
    my $gdmp = `grep ^GDMP_LOCAL_HOST ${gdmb}/gdmp.shared.conf`;
#p  print " GDMP: $gdmp";
    chomp($gdmp);
    my @it = split /=/, $gdmp;
    $ENV{$it[0]}= $it[1];
    print " ENV $it[0]: $ENV{$it[0]} \n";

    # Collect the information from MDS.
    my $command = "ldapsearch -LLL -H ldap://" .
        $self->{opts}->{mds} .
        ":" .
	    $self->{opts}->{mdsport} .
        " -x -b 'mds-vo-name=local,o=grid' " .
        "'(|(objectclass=computingelement)(objectclass=storageelement))'" .
	    " SEId ";
    my $arrayref = timeout($command,90);
    my @output = @$arrayref;

    # Catch errors from the execution of the command.
    if ($output[0]==1) {
        $self->setErrorMessage('could not fork ldapsearch');
        return 1;
    } elsif ($output[0]==2) {
        $self->setErrorMessage('ldapsearch timed out on MDS');
        return 1;
    }

    # Pull out the list of CEs and SEs from the output.
    my %mdsres;
    my @lines = split("\n","$output[1]\n");
    foreach my $line (@lines) {
        $mdsres{$1} =1 if ($line =~ m/([CS]EId:\s*[^\s]+)\s*/);
    }

## On CertTB: MDS==0242:2135, BDII==0243:2170
## On  DevTB: MDS==0406:2135, BDII==0410:2170
## On CertTB: ldapsearch -LLL -H ldap://lxshare0243.cern.ch:2170 -x -b 'mds-vo-name=local,o=grid'
## ldapsearch -LLL -H ldap://lxshare0243.cern.ch:2170 -x -b 'mds-vo-name=local,o=grid' '(|(objectclass=GlueCESEBind)(objectclass=storageelement))' GlueCESEBindSEUniqueID
## ldapsearch -LLL -H ldap://lxshare0243.cern.ch:2170 -x -b 'mds-vo-name=local,o=grid' '(|(objectclass=GlueCESEBind)(objectclass=storageelement))' GlueCESEBindCEAccesspoint
## ldapsearch -LLL -H ldap://lxshare0243.cern.ch:2170 -x -b 'mds-vo-name=local,o=grid' '(|(objectclass=GlueCESEBindGroup)(objectclass=storageelement))' GlueCESEBindGroupSEUniqueID
## Use lxshare0410 for DevTB BDII

    # Collect the information from BDII.
    $command = "ldapsearch -LLL -H ldap://" .
        $self->{opts}->{bdii} .
        ":" .
	    $self->{opts}->{bdiiport} .
        " -x -b 'mds-vo-name=local,o=grid' " .
        "'(|(objectclass=computingelement)(objectclass=storageelement))'" .
	    " SEId ";
    $arrayref = timeout($command,90);
    @output = @$arrayref;

    # Catch errors from the execution of the command.
    if ($output[0]==1) {
        $self->setErrorMessage('could not fork ldapsearch');
        return 1;
    } elsif ($output[0]==2) {
        $self->setErrorMessage('ldapsearch timed out on BDII');
        return 1;
    }

    # Pull out the list of CEs and SEs from the output.
    my %bdiires;
    @lines = split("\n","$output[1]\n");
    foreach my $line (@lines) {
        $bdiires{$1} =1 if ($line =~ m/([CS]EId:\s*[^\s]+)\s*/);
    }

## For this, run it with:
## ./edg-testbed-test BaseTest::EDGLifecycle::RMSetupTest --mds=lxshare0233.cern.ch --bdii=lxshare0231.cern.ch

#   print map { "$_ => $mdsres{$_}\n" } keys %mdsres;
    print " BDIIres: @{[ %bdiires ]}\n";
    print " MDSres:  @{[ %mdsres ]}\n";

    my %seen;
    @seen{keys %mdsres} = ();
    my @unseen = grep { ! exists $seen{$_} } keys %bdiires;
    print " NOT in BDII: @unseen $#unseen\n";
    my $unSeenBdii = $#unseen;
    %seen = ();
    @unseen = ();
    @seen{keys %bdiires} = ();
    @unseen = grep { ! exists $seen{$_} } keys %mdsres;
    print " NOT in MDS:  @unseen $#unseen\n";
    my $unSeenMds = $#unseen;

    if ( ($unSeenMds+$unSeenBdii) != -2 ) {
	$self->setErrorMessage('MDS/BDII mismatch in SE resources');
	return 1;
    }
    @unseen = keys %mdsres;
    my @SEs = ();
#    my %SPs = ();
#    my %SPst= ();
    foreach my $se ( @unseen ) {
	$se =~ s/SEId: //;
	push @SEs, $se;
    }
    print " SEs: @SEs \n";

    if ( $#SEs <= 0 ) {
	$self->setErrorMessage('Less than 2 SEs: Test is Impossible. \n');
	return 1;
    }

    ## Temp ...
    ##if (1) { exit 0;}

    return (0, @SEs);

}


## To extract the SE Mount Points and all that ...
sub getSPsOLD {

    my $self = shift;
    my @SEs = @_;

    my %SPs = ();
    my %SPst= ();

    ## Check that "tstg" VO exists on all SEs ...
    foreach my $se ( @SEs ) {
        my $command = "ldapsearch -x -h $se -p 2135 -b \"mds-vo-name=local,o=grid\" \"objectclass=storageelement\" sevo | grep -i sevo:";
        my $arrayref = timeout($command, 90);
        my @output = @$arrayref;
#p      print " SE: $se \n @output \n";

    # Catch errors from the execution of the command.
	if ($output[0]==1) {
	    $self->setErrorMessage('could not fork ldapsearch');
	    return 1;
	} elsif ($output[0]==2) {
	    $self->setErrorMessage('ldapsearch timed out on SE $se');
	    return 1;
	}
        my @lines = split("\n","$output[1]\n");
        my @voex = grep { /$self->{opts}->{VO}/ } @lines;
        print " VOEX: $se @voex \n";
        if ( $#voex != 0 ) {
            $self->setErrorMessage("VO \"$self->{opts}->{VO}\" does not exist on this SE: $se");
            return 1;
        }
        my @p = split /\s+/, $voex[0];
        @p = split /:/, $p[1];
        if ( $p[0] ne "$self->{opts}->{VO}" ) {
            $self->setErrorMessage("Wrong VO \"$self->{opts}->{VO}\" exists on this SE: $se, $voex[0]");
            return 1;
        }
        $SPs{$se} = $p[1];


        ## Another check about these mount points ...
        my $gdmpSt = `gdmp_job_status -S $se -c gdmp.conf | grep GDMP_STORAGE_DIR`;
#p      print " GDMPST: $se $gdmpSt";
        @p = split /=/, $gdmpSt;
        $SPst{$se} = $p[1];
        chomp($SPst{$se});
        if ( $SPst{$se} ne $SPs{$se} ) {
            print STDERR " Mount point mismatch on SE $se: $SPst{$se} $SPs{$se} \n";
            $self->setErrorMessage("Mount point mismatch between GDMP_STORAGE_DIR and LDAPSEARCH on this SE: $se");
            return 1;
        }

    }
    print "SPs: \n", map { "$_ => $SPs{$_}\n" } keys %SPs;

    return (0, %SPs);

}


## Mutual subscription check for SPs
sub mutualSPsOLD {
    my ($subDir, $arrayref, $hashref) = @_;
    my @SEpair = @$arrayref;
    my %SPs = %$hashref;
    my $rc = 0;

    ## Mutual subscription test ...
    my $errSum = 0;
    for my $n ( 0..1 ) {
#       my $gdmpS = `gdmp_host_subscribe -c -S $SEpair[$n] -r $SEpair[1-$n]`;
#       print " gdmp_host_subscribe: \n $gdmpS";
        my $command = "gdmp_host_subscribe -c -S $SEpair[$n] -r $SEpair[1-$n]";
        my $arrayref = timeout ($command, 60);
        my @output = @$arrayref;
        print " gdmp_host_subscribe: $output[0]\n $output[1]";
        $errSum += $output[0];

        ## Creating the individual subdir whenever needed ... and checking the actual contents if already there !
        my $se = $SEpair[$n];
        my $resMk = `edg-gridftp-mkdir gsiftp://${se}$SPs{$se}/$subDir`;
        print " resMk: $se $SPs{$se} -$?- \n$resMk";
        chomp(my $resLs = `edg-gridftp-ls --verbose gsiftp://${se}$SPs{$se}/$subDir`);
        print " resLs1: $se $SPs{$se} -$?- \n$resLs";

        ## Cleaning up left over files ...
        chomp(my @resLs = `edg-gridftp-ls gsiftp://${se}$SPs{$se}/$subDir`);
        if ( $#resLs > -1 ) {
            foreach my $f ( @resLs ) {
                print " ==> Removing file: -$f- \n";
                if ( "$f" eq "" ) { last; }
                my $resDel = `edg-gridftp-rm gsiftp://${se}$SPs{$se}/$subDir/$f`;
            }
        }
        chomp(my $resLs = `edg-gridftp-ls --verbose gsiftp://${se}$SPs{$se}/$subDir`);
        print " resLs2: $se $SPs{$se} -$?- \n$resLs";
        if ( $resLs !~ "total 0" ) {
            $rc = 1;
            print STDERR " Dir: $subDir on: $se not created or not empty \n";
        }
    }

    if ( $errSum ) {
        $rc = 1;
        print STDERR " GDMPs are not mutually subscribed: @SEpair \n";
    }

    return $rc;

}


## Managing the GLOBUS locations if they both exist or do not exist ...
sub locationStuffOLD {
    my ($SE1, $SE2, $SP1, $SP2, $subDir, $file, $hostname) = @_;

    ## Initialization of the location stuff ...

        print "\n";
        open CNF, "$ENV{RC_CONFIG_FILE}";
        while (<CNF>) {
            if ( $_ =~ /(.*?)=(.*)/ ) {
                my $var = $1;
                chomp (my $val = "$2");
                print " CNF: $var => $val \n";
                $ENV{$var} = $val;
            }
        }
        close CNF;
        print "\n";
        my $echo = `echo $ENV{RC_REP_CAT_MANAGER_PWD} >> /tmp/$ENV{USER}-$$-pwd.conf`;

        foreach my $se ( $SE1, $SE2 ) {
            my @resD = `globus-replica-catalog -host "$ENV{RC_LOGICAL_COLLECTION}" -manager "$ENV{RC_REP_CAT_MANAGER_DN}" -password /tmp/$ENV{USER}-$$-pwd.conf  -location $se -delete 2>&1`;
            print " resD: on $se with $file -$?- =$#resD= \n@resD";
        }
        unlink "/tmp/$ENV{USER}-$$-pwd.conf";

        my @resRcCARF = `edg-replica-manager-copyAndRegisterFile -e -l ${subDir}/${file} -s ${hostname}//tmp/${file} -d $SE1/$SP1/$file 2>&1`;
#p      print "   resRcCARF0: $file -$?- \n@resRcCARF";
        print "   resRcCARF0: $file -$?- \n";
        my @resRcRF = `edg-replica-manager-replicateFile -e -l ${subDir}/${file} -s $SE1/$SP1/$file -d $SE2/$SP2/$file 2>&1`;
#p      print "   resRcRF0:   $file -$?- \n@resRcRF";
        print "   resRcRF0:   $file -$?- \n";
        my @resRcDF = grep { /deleting/ } `edg-replica-manager-deleteFile -e -a -l ${subDir}/${file} 2>&1`;
#p      print "   resRcDF0:   $file -$?- =$#resRcDF= \n@resRcDF";
        print "   resRcDF0:   $file -$?- =$#resRcDF= \n";
        my @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
#p      print "   resRcLR0:   $file -$?- =$#resRcLR= \n@resRcLR";
        print "   resRcLR0:   $file -$?- =$#resRcLR= \n";

	return 0;

}

sub testRMSetupTest {
    my $self = shift;
my ($a, $b, $c, $d) = @_;

## Here returnSE means SE2
print STDERR " testRMSetupTest: runTest ARGS: $a $b $c $d\n";
print STDOUT " testRMSetupTest: runTest ARGS: $a JobIndex: $b returnSE: $c SEtype: $d\n";
my $returnSE = $c;
my $closeSE  = $d;
my $SEtype   = "dummy";

    if ( 0 ) {  ##  Migration
my $res = `cp .BrokerInfo BrokerInfo 2>&1`;
my (@CSE) = `edg-brokerinfo getCloseSEs 2>&1`;
    chomp(my $theCE = `edg-brokerinfo getCE 2>&1 | cut -d":" -f 1`);
    chomp(my $theWN = `hostname --long 2>&1`);
chomp(my $closeSE = $CSE[0]);

my %cseType = ();
$closeSE = "";
foreach my $mycse ( @CSE ) {
    chomp $mycse;
    chomp(my $SEfs = `edg-brokerinfo getSEFreeSpace  $mycse 2>&1`);
    chomp(my $SEmp = `edg-brokerinfo getSEMountPoint $mycse 2>&1`);
    ## flatfile search is useless here ...
    my @ldap = `ldapsearch -x -H ldap://$mycse:2135 -b "mds-vo-name=local,o=grid" | egrep 'castor|pnfs|flatfile' 2>&1`;

my $pnfsO = grep { /pnfs/ } @ldap;
my $castorO = grep { /castor/ } @ldap;
print " MYCSE: $mycse $pnfsO $castorO \n";

    $cseType{$mycse} = "NADA";
#     if ( $SEfs =~ /\d+/ ) {
#         if ( $SEmp =~ /flatfiles/ || $SEmp !~ /dummy/ ) { $cseType{$mycse} = "Classic"; }
#         else { $cseType{$mycse} = "Castor"; }
#     } else { $cseType{$mycse} = "dCache"; }

    if ( $SEmp =~ /flatfiles/ || $SEmp !~ /dummy/ ) { $cseType{$mycse} = "Classic"; }
    else {
        if ( $pnfsO ) { $cseType{$mycse} = "dCache"; }
        if ( $castorO ) { $cseType{$mycse} = "Castor"; }
    }

    print STDOUT " CSEType: $mycse >$cseType{$mycse}< \n";
    if ( $cseType{$mycse} eq "NADA" ) { next; }

###    if ( $closeSE eq "" && $cseType{$mycse} ne "NADA" ) { $closeSE = $mycse; }
##    if ( $closeSE eq "" && $cseType{$mycse} ne "dCache" ) { $closeSE = $mycse; }
    if ( $closeSE eq "" && $cseType{$mycse} eq "dCache" ) { $closeSE = $mycse; }
#    if ( $closeSE eq "" && $cseType{$mycse} ne "Classic" ) { $closeSE = $mycse; }

}

    if (0) {
	my %cseF = (
		    'lxshare0235.cern.ch' => 'lxshare0282.cern.ch',
		    'lxshare0241.cern.ch' => 'lxshare0302.cern.ch',
		    'lxshare0277.cern.ch' => 'lxshare0247.cern.ch',
		    'lxshare0290.cern.ch' => 'lxshare0302.cern.ch'
		    );
	$closeSE = $cseF{$theCE};
    }

    if (0) {
	my %cseF = (
		    'lxshare0235.cern.ch' => 'lxshare0282.cern.ch',
		    'lxshare0241.cern.ch' => 'lxshare0303.cern.ch',
		    'lxshare0277.cern.ch' => 'lxshare0247.cern.ch',
		    'lxshare0290.cern.ch' => 'lxshare0229.cern.ch'
		    );
	$closeSE = $cseF{$theCE};
    }

my %SEofType = reverse %cseType;

if ( $SEtype eq 'All' ) {  ##  GOOD
    my $nCE = scalar(@CSE);
    my $rnd = int(rand($nCE));
    $closeSE = $CSE[$rnd];
} else {
    if ( $SEofType{$SEtype} ) { $closeSE = $SEofType{$SEtype}; }
    elsif ( $SEofType{'Classic'} ) { $closeSE = $SEofType{'Classic'}; }
}

if (1) {
    print STDOUT " CSE: >$closeSE< @CSE ( SEtype: $SEtype TypeOf: $cseType{$closeSE} ) \n";
##TEMP
    if ( scalar(@CSE) >= 2 && $closeSE =~ /lxshare0247|lxshare0282|lxshare0302/ ) {
##REMOVED??   chomp($closeSE = $CSE[1]);
    }
    if ( $closeSE eq "" ) { 
        print STDERR " FAILed: NO closeSE \n";
        exit 1; 
    }
}

## Look for default SE
my $defSE = `edg-rm --vo dteam pi | grep 'default SE' `;
if ( $defSE ) {
    my @m = split /\s+/, $defSE;
    $defSE = $m[3];
}

if ( 0 ) {
    $closeSE = $defSE;
    print " Warning: because of --defdest, the closeSE is now: $closeSE \n";
}
}  ##  Migration

##chomp(my $myVO = `edg-brokerinfo getVirtualOrganization 2>&1`);;;;;
    $self->{opts}->{vo} = $self->{opts}->{VO};
    my $myVO =   $self->{opts}->{vo};
    my $config = "";  ## Still empty

    my $edgRMopts;
if ( ! $self->{opts}->{lcgcr} ) {
    $edgRMopts = "$config --insecure --vo=$myVO";
} else {
    $edgRMopts = "$config -i --vo=$myVO";
    $edgRMopts = "$config --insecure --vo=$myVO";
}

#OLD    print STDOUT " DataStorm: edgRMopts: $edgRMopts WN: $theWN CE: $theCE closeSE: $closeSE returnSE: $c \n";
    print STDOUT " testRMSetupTest: edgRMopts: $edgRMopts closeSE: $closeSE returnSE: $c \n";

my ($cres, $file) = $self->createNewData($edgRMopts, $closeSE, $returnSE, $a, $b, "");
print " JobLFN: lfn:$file \n" unless $cres;

    return 0;

}

sub createNewData {
    my $self = shift;
    my ($edgRMopts, $SE1, $SE2, $serie, $index, $subDir) = @_;

#    $self->printLocation( (caller(0))[3] );

    ## Create another temporary data file ...
    my $file = "$ENV{USER}-$$-dstorm-$serie-$index.data";
    ## Some Pre-Cleaning
    my $rc = $self->removeGhostFile($edgRMopts, $file);
    return $rc if $rc;

    # Retrieve the temporary directory.
    my $tmpdir = $self->{tempdir};
#    my $tmpdir = "/tmp";

  # Create a local random file with the required size
    my $href = $self->runGommand("dd if=/dev/urandom of=$tmpdir/$file bs=1024 count=$self->{opts}->{filesize}",0);
    $rc = $href->{rc};
    return $rc if $rc;

#     my ($stat, $resRcCARF) = $self->fillRH2DRun($SE1, 
# 				       $SE2, 
# 				       "copyAndRegisterFile", 
# 				       $stepN++, 
# 				       "Create a copy of the local file", 
# 				       $command, 
# 				       0);

#     open TMP, ">$tmpdir/$file";
#     print TMP "Good Morning to all Everybody ! This is a DataStorm. Job# $index \n";
#     close TMP;

    print " JobLFN0: lfn:$file \n";
#p    print STDERR " JobLFN0: lfn:$file \n";

#     my $log = "$tmpdir/$ENV{USER}-$$-dstorm-$serie-$index.log";
#     $self->{log} = "$log";
#     my $resT = `touch $log`;

#     my @kSEpairs = keys %{$self->{SEpairs}};
#     my ($SE1, $SE2) = split /\s+/, $kSEpairs[int(rand(scalar(@kSEpairs)))];

#     my $SP1 = "$self->{SPs}->{$SE1}/$subDir";
#     my $SP2 = "$self->{SPs}->{$SE2}/$subDir";

    $rc = $self->createReplicas ($edgRMopts, $SE1, $SE2, $subDir, $file);
    return $rc if $rc;

    return (0, "$file");
}

## Dynamic creation of Replicas job by job ...
sub createReplicas {
    my $self = shift;

#     my ($self, $SE1, $SE2, $subDir, $file) = @_;
#     my $edgRMopts = $self->{edgRMopts};
    my ($edgRMopts, $SE1, $SE2, $subDir, $file) = @_;

    # Retrieve the temporary directory.
    my $tmpdir = $self->{tempdir};
#    my $tmpdir = "/tmp";

    my $stepN = 0;

#     my $command = "edg-replica-manager-copyAndRegisterFile -e -l ${subDir}/${file} -s $self->{hostname}//tmp/${file} -d $SE1/$SP1/$file";
#     print " CMD: $command\n";
#     my $resRcCARF = `$command 2>&1`;
#     my $stat = $?;
#     print "\n resRcCARF1: $file -$stat- \n$resRcCARF";
#     if ( $stat != 0 ) {
# 	print STDERR " File: ${subDir}/${file} on: $SE1 not correctly registered or created \n";
# 	return 1;
#     }

## =====================> Mettre un runCommand sur ttes les commandes edg-rm
## =====================> Creer un fichier avec dd comme Gonzalo
## =====================> Essayer de diviser en STEPs comme Gonzalo
## Pb bufferization ..............

my $command;
if ( ! $self->{opts}->{lcgcr} ) {
##EDG-RM case
    $command = "edg-replica-manager --verbose $edgRMopts copyAndRegisterFile file:$tmpdir/${file} -l lfn:${file} -d $SE1 ";
} else {
##LCG-CR case
    chomp(my $gguid = `uuidgen 2>&1`);
    print STDOUT " UUIDGEN: $gguid \n";
    $command = "lcg-cr --verbose $edgRMopts -l lfn:${file} -d $SE1 -g $gguid file:$tmpdir/${file}";
}

#p    print "\n CMD: $command\n";
    my $GUID;
##OLD    my $resRcCARF = `$command 2>&1`;
    my ($stat, $resRcCARF) = $self->fillRH2DRun($SE1, 
				       $SE2, 
				       "copyAndRegisterFile", 
				       $stepN++, 
				       "Create a copy of the local file", 
				       $command, 
				       0);
##OLD    my $stat = $?;
#p    print "\n resRcCARF1: $file -$stat- \n$resRcCARF";
    if ( $stat != 0 ) {
	print STDERR " File: ${file} on: $SE1 not correctly registered or created \n";
	return 1;
    } else {
	chomp($resRcCARF);
	if ( $resRcCARF !~ "guid:" ) {
	    print STDERR " File: ${file} on: $SE1 : Wrong GUID returned: $resRcCARF \n";
	    return 1;
	}
	my @l = split /^/, $resRcCARF;
	@l = grep { /guid:/ } @l;
	$GUID = $l[0];
	chomp($GUID);
#	$self->{reqs}->{$stepI}->{olout} = $resRcCARF;
    }

    my %PFNs = ();
    $PFNs{"$SE1/$file"} = "created";

##return 0;             ##   VERY TEMPORARY ?

#    my @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
    $command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
    if ( $self->{opts}->{lcgcr} ) { $command = "lcg-lr $edgRMopts lfn:${file}"; }
##OLD    my @resRcLR = `$command 2>&1`;
    my ($stat2, $resRcLR) = $self->fillRH2DRun($SE1, 
				       $SE2, 
				       "listReplicasFirst", 
				       $stepN++, 
				       "Check First stage replica", 
				       $command, 
				       0);
#p    print "\n resRcLR1: $file -$?- =$#resRcLR= \n@resRcLR";

    my $href = $self->parseTURLs($resRcLR);
#     foreach my $prot (@{$href->{protocols}}) {
# 	my $surl = shift @{$href->{turls}};
# 	if ($prot eq 'srm' || $prot eq 'sfn') {
# 	    $self->{SURL1} = $surl;
# 	    $ifound = 1;
# 	    last;
# 	}
#     }

#good    foreach my $l ( split /^ /, $resRcLR ) {
    foreach my $l ( @{$href->{turls}} ) {
        chomp $l;
        my @i = split /[:\/]/, $l;
#prev	if ( $i[0] ne "srm" ) {
	if ( $i[0] !~ /srm|sfn/ ) {
	    print STDERR " File: ${file} on: $SE1 : Wrong SRM returned: $l \n";
	    return 1;
	} 

	my $gsrm = "$i[3]/${file}";
	my $csrm = "$SE1/${file}";
	if ( $csrm ne $gsrm ) {
	    print STDERR " File: ${file} on: $SE1 : A ghost file still existed: $gsrm \n";
	    return 1;
	}

        if ( ! exists $PFNs{$gsrm} || $PFNs{$gsrm} !~ /created|existing/ ) {
            print STDERR " Step: $SE1 File: $file did not previously existed: $gsrm ! \n";
	    return 1;
        } else {
	    $PFNs{$gsrm} = "existing";
	    print " SRM keys SE1: $SE1 $file value: $l \n";
	}
    }
    if ( $PFNs{"$SE1/$file"} ne "existing" ) {
	print STDERR " Step: $SE1 File: $file not correctly copied/registered ! \n";
	return 1;
    }

if ( "$SE1" ne "$SE2" ) {
#    my $resRcRF = `edg-replica-manager-replicateFile -e -l ${subDir}/${file} -s $SE1 -d $SE2 2>&1`;
    $command = "edg-replica-manager --verbose $edgRMopts replicateFile lfn:${file} -d $SE2";
    if ( $self->{opts}->{lcgcr} ) { $command = "lcg-rep --verbose $edgRMopts -d $SE2 lfn:${file}"; }
##OLD    my $resRcRF = `$command 2>&1`;
    my ($stat, $resRcRF) = $self->fillRH2DRun($SE1, 
				       $SE2, 
				       "replicateFile", 
				       $stepN++, 
				       "Replicate First stage copy", 
				       $command, 
				       0);
##OLD    $stat = $?;
#p    print "\n resRcRF2: $file -$stat- \n$resRcRF";
    if ( $stat != 0 ) {
        print STDERR " File: ${file} on: $SE2 not correctly registered or created \n";
	return 1;
    }
    $PFNs{"$SE2/$file"} = "created";
}

##return 0;             ##   VERY TEMPORARY ?

#       @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
    $command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
    if ( $self->{opts}->{lcgcr} ) { $command = "lcg-lr $edgRMopts lfn:${file}"; }
##OLD    my @resRcLR = `$command 2>&1`;
    my ($stat3, $resRcLR) = $self->fillRH2DRun($SE1, 
				       $SE2, 
				       "listReplicasSecond", 
				       $stepN++, 
				       "Check Second stage replica", 
				       $command, 
				       0);
#p    print "\n resRcLR2: $file -$?- =$#resRcLR= \n@resRcLR";
    $href = $self->parseTURLs($resRcLR);
    my $SURL2;
    my @resRcLR = split /^/, $resRcLR;
#good    foreach my $l ( @resRcLR ) {
    foreach my $l ( @{$href->{turls}} ) {
##NEW    foreach my $l ( split /^ /, $resRcLR ) {
        chomp $l;
#p	    print STDOUT " File: ${file} on: $SE2 : SRM returned: $l \n";
        my @i = split /[:\/]/, $l;
#	if ( $i[0] ne "srm" ) {
	if ( $i[0] !~ /srm|sfn/ ) {
	    print STDERR " File: ${file} on: $SE2 : Wrong SRM returned: $l \n";
	    return 1;
	} 

	my $gsrm = "$i[3]/${file}";
	my $csrm = "$SE2/${file}";
	if ( $csrm ne $gsrm ) {
	    next;
# 	    print STDERR " File: ${file} on: $SE2 : A ghost file still existed: $gsrm \n";
# 	    return 1;
	}

        if ( ! exists $PFNs{$gsrm} || $PFNs{$gsrm} !~ /created|existing/ ) {
            print STDERR " Step: $SE2 File: $file did not previously existed: $gsrm ! \n";
	    return 1;
        } else {
	    $PFNs{$gsrm} = "existing";
	    print " SRM keys SE2: $SE2 $file value: $l \n";
	    $SURL2 = $l;
	}
    }

    print "\nPFN0: \n", map { "$_ => $PFNs{$_}\n" } keys %PFNs;
    print "-----------------------------------\n";
    if ( $#resRcLR < 1 || scalar(keys(%PFNs)) != 2 ) {
	print STDERR " Replicates not correctly created on: $SE1 $SE2 \n";
	return 1;
    }

    ## Copy the file back to local ...
    $command = "edg-replica-manager $edgRMopts --verbose copyFile $SURL2 file://$tmpdir/${file}.2";
    if ( $self->{opts}->{lcgcr} ) { $command = "lcg-cp $edgRMopts --verbose $SURL2 file://$tmpdir/${file}.2"; }
    my ($stat3, $resRcLR) = $self->fillRH2DRun($SE1, 
				       $SE2, 
				       "copyFile", 
				       $stepN++, 
				       "From Second stage replica, Copy the file back to local", 
				       $command, 
				       0);
    if ( $stat3 ) {
	print STDERR " copyFile command failed with ret val $stat3 \n";
	return 1;
    }

    ## Compare both local files with simple diff ...
    $command = "/usr/bin/diff $tmpdir/${file} $tmpdir/${file}.2";
    my ($stat4, $resRcLR) = $self->fillRH2DRun($SE1, 
				       $SE2, 
				       "compare", 
				       $stepN++, 
				       "Compare both local files with simple diff", 
				       $command, 
				       0);
    if ( $stat4 ) {
	print STDERR " Files are different! Ret val $stat4 \n";
	return 1;
    }

    ## Delete everything about the files (lfn, sfn, guid ...)
    $command = "edg-replica-manager $edgRMopts --verbose deleteFile $GUID -a";
    if ( $self->{opts}->{lcgcr} ) { $command = "lcg-del -a $edgRMopts --verbose $GUID"; }
    my ($stat5, $resRcLR) = $self->fillRH2DRun($SE1, 
				       $SE2, 
				       "deleteFile", 
				       $stepN++, 
				       "Delete everything about the files (lfn, sfn, guid)", 
				       $command, 
				       0);
    if ( $stat5 ) {
	print STDERR " deleteFile command failed with ret val $stat5 \n";
	return 1;
    }

    my $del = `rm $tmpdir/${file} $tmpdir/${file}.2`;

    ## Check that everything is deleted indeed ...
#     $command = "edg-replica-manager $edgRMopts --verbose deleteFile $GUID -a";
#     if ( $self->{opts}->{lcgcr} ) { $command = "lcg-del -a $edgRMopts --verbose $GUID"; }
#     my ($stat6, $resRcLR) = $self->fillRH2DRun($SE1, 
# 				       $SE2, 
# 				       "deleteFile", 
# 				       $stepN++, 
# 				       "Delete everything about the files (lfn, sfn, guid)", 
# 				       $command, 
# 				       0);
#     if ( $stat6 ) {
# 	print STDERR " deleteFile command failed with ret val $stat5 \n";
# 	return 1;
#     }
    $command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
    if ( $self->{opts}->{lcgcr} ) { $command = "lcg-lr $edgRMopts lfn:${file}"; }
    my ($stat6, $resRcLR) = $self->fillRH2DRun($SE1, 
				       $SE2, 
				       "listReplicasEmpty", 
				       $stepN++, 
				       "Check everything is deleted indeed", 
				       $command, 
				       0);
    if ( $stat6 ) {
	print STDOUT " listReplicas command failed with ret val $stat6. This should mean the replicas have been deleted ok... \n";
#p	print STDERR " listReplicas command failed with ret val $stat6. This should mean the replicas have been deleted ok... \n";
	return 0;
    }

    return 0;

}

sub removeGhostFile {
    my $self = shift;

#     my ($self, $file) = @_;
#     my $edgRMopts = $self->{edgRMopts};
    my ($edgRMopts, $file) = @_;

#    $self->printLocation( (caller(0))[3] );

##	    foreach my $loop ( 0..2 ) {
##		my $file = "$ENV{USER}-$upid-$loop-rmstorm.txt";

		my $command = "edg-replica-manager $edgRMopts listGUID lfn:${file}";
                if ( $self->{opts}->{lcgcr} ) { $command = "lcg-lg $edgRMopts lfn:${file}"; }
		chomp(my $resRcLR = `$command 2>&1`);
		my $stat = $?;
		print "$stat\t                                   $resRcLR\n";
#		if ( $stat ) { next; }
    return 0 if $stat;

		$command = "edg-replica-manager --verbose $edgRMopts deleteFile --all-available --all-lfn $resRcLR";
                if ( $self->{opts}->{lcgcr} ) { $command = "lcg-del --verbose $edgRMopts -a $resRcLR"; }
		my $resRcDF = `$command 2>&1`;
		$stat = $?;
		print "\n resRcDF: $file -$stat- \n$resRcDF\n";
    return $stat if $stat;

		$command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
                if ( $self->{opts}->{lcgcr} ) { $command = "lcg-lr $edgRMopts lfn:${file}"; }
		my $resRcLRafter = `$command 2>&1`;
		$stat = $?;
		print "\n resRcLRafter: $file -$stat- \n$resRcLRafter";
#    return $stat if $stat;

		$command = "edg-replica-manager $edgRMopts listReplicas $resRcLR";
                if ( $self->{opts}->{lcgcr} ) { $command = "lcg-lr $edgRMopts $resRcLR"; }
		my $resRcLGafter = `$command 2>&1`;
		$stat = $?;
		print "\n resRcLGafter: $file -$stat- \n$resRcLGafter";
#    return $stat if $stat;
##	    }

    return 0;

}

# Run job submissions in parallel.  This takes all of the parameters
# from the state of the object. 
## ***************************************************************************************
sub parallelSubmit {

    my $self = shift;
    $self->printStack( 3 );

    my @pids = ();
    my @phs  = ();

	      STDOUT->autoflush(1);
	      STDERR->autoflush(1);

    foreach my $i (1 .. $self->{opts}->{njobs}) {

	if ( scalar(@{$self->{pairh}}) ) {
	    my ($pidr, $phr) = $self->forkSubmit();
	    push @pids, @$pidr;
	    push @phs , @$phr;
	}

    }

    print " Pids: @pids \n";
    print " Stack: @phs \n";

    return (\@pids, \@phs);

}

# Run job submissions in parallel.  This takes all of the parameters
# from the state of the object. 
## ***************************************************************************************
sub parallelSubmitOLD {

#    my ($self, @pairs) = @_;
    my $self = shift;
   $self->printStack( 3 );

    my @pids = ();
    my @phs  = ();

	      STDOUT->autoflush(1);
	      STDERR->autoflush(1);

#    $self->{debug} = 1;
#    $self->printLocation( (caller(0))[3] );

    ## VERY SPECIAL
##    $self->{opts}->{njobs} = scalar(@pairs);
#    $self->{opts}->{njobs} = 2;
    ## VERY SPECIAL

    foreach my $i (1 .. $self->{opts}->{njobs}) {
##forhosts	foreach my $host (@hosts) {
#	my $aref = shift @pairs;
	my $aref = shift @{$self->{pairh}};
	my ($row, $col, $ind) = @$aref;
	print STDERR "\n Showing  runStepCycle with $row, $col (job# $ind/$self->{opts}->{njobs}) \n";

	## VERY SPECIAL
# 	my $rc = $self->testRMSetupTest("serie", $i, $row, $col);
# 	next;
	## VERY SPECIAL

	    local *PH;
      FORK: { 
	    my $pid;
  die "cannot fork: $!" unless defined ( $pid = open ( PH, "-|" ) );
#          if ($pid = fork) {
          if ($pid) {
              # parent here
              # child process pid is available in $pid
              push @pids, $pid;
	      push @phs, *PH;
          } elsif (defined $pid) {
              # $pid is zero here if defined
              # child here
              # parent process available with getppid

	      STDOUT->autoflush(1);
	      STDERR->autoflush(1);

	      # Do the actual submission.  The error return is ignored
	      # because there is no way to feed back this information
	      # easily.  
# 	      my $rc = $self->submit(["$self->{prefix}.$i",
# 				      "$self->{opts}->{njobs}"]);
	      print STDOUT " Starting runStepCycle with $row, $col (job# $ind) \n";
	      my $tStart = time();
##	      my $rc = $self->testGatekeeper($host);
	      my $rc = $self->testRMSetupTest("serie", $ind, $row, $col);
	      my $tAlmost = time() - $tStart;
	      print STDOUT " TimeRMC: $tAlmost \n";
              # This is important otherwise child may execute parent code!
              exit;
          } elsif ($! =~ /No more process/) {
              # Recoverable error (EAGAIN); try again. 
              sleep 5;
              redo FORK;
          } else {
              # Strange error! Print error message. 
              print STDERR "Fork failure.\n";
          }
      }
#	$self->{job}->{submit}->{file} = "$self->{prefix}.$i" if ( $self->{opts}->{nstreams} == 1 );
##forhosts    }
    }

    print " Pids: @pids \n";
    print " Stack: @phs \n";

    return (\@pids, \@phs);
}

sub forkSubmit {

    my $self = shift;
   $self->printStack( 3 );

    my @pids = ();
    my @phs  = ();

	      STDOUT->autoflush(1);
	      STDERR->autoflush(1);

	my $aref = shift @{$self->{pairh}};
	my ($row, $col, $ind) = @$aref;
	print STDERR "\n Showing  runStepCycle with $row, $col (job# $ind/$self->{opts}->{njobs}) \n";

	## VERY SPECIAL
# 	my $rc = $self->testRMSetupTest("serie", $i, $row, $col);
# 	next;
	## VERY SPECIAL

	    local *PH;
      FORK: { 
	    my $pid;
  die "cannot fork: $!" unless defined ( $pid = open ( PH, "-|" ) );
#          if ($pid = fork) {
          if ($pid) {
              # parent here
              # child process pid is available in $pid
              push @pids, $pid;
	      push @phs, *PH;
          } elsif (defined $pid) {
              # $pid is zero here if defined
              # child here
              # parent process available with getppid

	      STDOUT->autoflush(1);
	      STDERR->autoflush(1);

	      # Do the actual submission.  The error return is ignored
	      # because there is no way to feed back this information
	      # easily.  
# 	      my $rc = $self->submit(["$self->{prefix}.$i",
# 				      "$self->{opts}->{njobs}"]);
	      print STDOUT " Starting runStepCycle with $row, $col (job# $ind) \n";
	      my $tStart = time();
##	      my $rc = $self->testGatekeeper($host);
	      my $rc = $self->testRMSetupTest("serie", $ind, $row, $col);
	      my $tAlmost = time() - $tStart;
	      print STDOUT " TimeRMC: $tAlmost $rc $$ \n";
#p	      print STDERR " TimeRMC: $tAlmost $rc $$ \n";
              # This is important otherwise child may execute parent code!
              exit;
          } elsif ($! =~ /No more process/) {
              # Recoverable error (EAGAIN); try again. 
              sleep 5;
              redo FORK;
          } else {
              # Strange error! Print error message. 
              print STDERR "Fork failure.\n";
          }
      }

    return (\@pids, \@phs);

}

sub parallelMonit {

    my ( $self, $pidref, $phref ) = @_;

    $self->printLocation( (caller(0))[3] );

    my @pids = @$pidref;
    my @phs = @$phref;

#p    print " Pids: @pids \n";
#p    print " Stack: @phs \n";

    my %pha = {};
    foreach my $pid ( @pids ) {
	my $ph = shift @phs;
	$pha{$pid} = $ph;
    }

##OLDPERL56    my @{$self->{logs}} = ();
    @{$self->{logs}} = ();

    my @stack = ();
    my $stpid;
    my $proc = 0;
#    while ( ($stpid = waitpid(-1, &WNOHANG)) > 0 ) {
    while ( ($stpid = waitpid(-1, 0)) > 0 ) {
	# Put $stpid on a stack ...
	push @stack, $stpid;
	my $ph = $pha{$stpid};
	my @lines = ();
	print "\n Proc: $proc $stpid >$ph< \n";
	while ( my $line = readline($ph) ) {
	     print " P: $line";
	    push @lines, $line;
	     if ( $line =~ /CONTROL:/ ) {
		 my ($void, $cont, $case, $mace, $stepn, $filen, $not) = split /\s+/, $line;
		 $self->{reqs}->{$case}->{$mace}->{$stepn}->{olog} = "$filen";
		 push @{$self->{logs}}, "$filen";
	     }
	}
	close ($ph);
	print " P: ====++++==== \n\n ";
	$proc++;
	## Voir ici comment stocker l'output
	## Attention aux pb de buffer plein
#OLD	push @{$self->{logs}}, \@lines;
	
#GOOD	if ( scalar(@{$self->{pairh}}) && $proc < 10 ) {

#p      print STDOUT " Signal Value: $BaseTest::sigC \n";
	if ( $BaseTest::sigC ) { 
	    print STDOUT " Signal Value: $BaseTest::sigC \n";
	} elsif ( scalar(@{$self->{pairh}}) ) {
	    my ($pidr, $phr) = $self->forkSubmit();
	    push @pids, @$pidr;
	    push @phs , @$phr;
	    	$pha{@$pidr[0]} = @$phr[0];
	    my $pp = $pha{@$pidr[0]};
#p	    print STDERR " New PID: @$pidr @$phr $pp $pha{@$pidr[0]}\n";
	}
#     print " PidsL: @pids \n";
#     print " PhrL : @phs \n";
    }

    print " Pids: @pids \n";
    print " Stack: @stack \n";
#p    print " Logs: @{$self->{logs}} \n";

    return 0;

}

# Override the BaseTest::evaluateTest method
sub evaluateTest {
  my $self = shift;
    $self->printStack(6);

##  TO BE FILLED UP
##    FROM THE LOG(s), FILL UP the $self->{XMLOUT}->{$row}->{$col}->{$step}->{rc}
##      AND $self->{XMLOUT}->{$row}->{$col}->{$step}->{out}
##	TRYING TO REUSE AT LEAST parseTURLs and other facilities ...

#OLD    print "GlobusGatekeeper: evaluateTest\n";

#    my $stepn = 0;

#    $self->{steps}->{$stepn}->{sname} = "GlobusGatekeeper";
#    $self->{steps}->{$stepn}->{sdesc} = "11 steps";
##    $self->{steps}->{$stepn}->{scomm} = "Several commands";

    my @logs = @{$self->{logs}};
#    my @ls = grep { s/ STEPSTEP:// } @{$logs[0]};
#    $self->{steps}->{$stepn}->{scomm} = "@ls";

    foreach my $filen ( @logs ) {
	open LOG, "<$filen";
	my @lines = <LOG>;
	close LOG;
#	my $stat1;
	my @linec = grep { /CONTROL:/ } @lines;
		 my ($void, $cont, $case, $mace, $stepn, $filenn, $not) = split /\s+/, $linec[0];
#p	print STDOUT " Test: $filenn $filen \n";
#	next;
# # 	my $mach = \$mace;
# # 	my $case = \$case;

	my $c = 0;
	my $l = $#lines;
#	foreach my $line ( @lines ) {
	foreach my $line ( 0 .. $l ) {
	    $line = shift @lines;
	    chomp($line);
	    my ( $void, $key, @rest ) = split / /, $line;
#	    chomp($key);
	    chop($key);
#p	    print " Kontrol: $key @rest \n";
	    $c++;
	    if ( $key eq "CONTROL" ) { next; }
	    if ( $key =~ /sname|sdesc|scomm/ ) {
		$self->{steps}->{$stepn}->{$key} = "@rest";
		next;
	    }
	    if ( $key =~ /stat1|stat2/ ) {
		$self->{reqs}->{$case}->{$mace}->{$stepn}->{$key} = "@rest";
		if ( $key eq "stat1" ) { 
		    $self->{reqs}->{$case}->{$mace}->{$stepn}->{stat1} = ( "@rest" eq "0" ) ? "[OK]" : "[FAIL]";
#prev		    $self->{globalresult} .= " $self->{reqs}->{$case}->{$mace}->{$stepn}->{stat1}" if $self->{globalresult};
		}
		next;
	    }
	    if ( $key eq "olout" ) { 
#		my $slurp = join "", (@lines)[$c .. -2];
		my @slurp = @lines;  ##  (@lines)[9 .. -2];
		$self->{reqs}->{$case}->{$mace}->{$stepn}->{$key} = join"", @slurp;
#p		print " SLURP: $c\n$lines[1]\n$lines[$c]\n$lines[-2]\n @slurp \n";
		last; 
	    } else {
#		next;  ##  Temporary ?

		chop($key);
		my $restt = $rest[0];
#bad		$self->{$key}->{${"$key"}} = ( $self->{$key}->{${"$key"}} < $rest ) ? $rest : $self->{$key}->{${"$key"}};
#bad		my $kkey = $$key;
		my $kkey;
		if ( $key eq "mach" ) { $kkey = $mace };
		if ( $key eq "case" ) { $kkey = $case };
		$self->{$key}->{$kkey} = ( $self->{$key}->{$kkey} < $restt ) ? $restt : $self->{$key}->{$kkey};
#p		print " $key: $kkey $self->{$key}->{$kkey} $restt\n";
	    }
	}

#p	print STDERR " EVAL0: $self->{steps}->{$stepn}->{sname} $self->{reqs}->{$case}->{$mace}->{$stepn}->{stat1} \n";
	if ( "$self->{steps}->{$stepn}->{sname}" eq 'listReplicasEmpty' ) {
	    if ( "$self->{reqs}->{$case}->{$mace}->{$stepn}->{stat1}" eq '[FAIL]' ) { 
		$self->{reqs}->{$case}->{$mace}->{$stepn}->{stat1} = '[OK]';
	    }
	}
#p	print STDERR " EVAL1: $self->{steps}->{$stepn}->{sname} $self->{reqs}->{$case}->{$mace}->{$stepn}->{stat1} \n";
	$self->{globalresult} .= " $self->{reqs}->{$case}->{$mace}->{$stepn}->{stat1}" if $self->{globalresult};

#	next;

# 	my @m = split /\s+/, $lines[0];
# 	my $failed = grep { /\[FAIL\]/ } @lines;
# 	my $oked   = grep { /\[OK\]/ }   @lines;
# 	chop $m[6];
# 	print " Eval: $m[4] $m[6] $failed $oked \n";
# 	my $case = "Job_$m[6]";
# 	my $mace = $m[4];
# 	$mace =~ s/\./<br> @ /;
# 	$self->{reqs}->{$case}->{$mace}->{$stepn}->{stat1} = ( $oked == 11 && $failed == 0 ) ? "[OK]" : "[FAIL]";
# 	$self->{reqs}->{$case}->{$mace}->{$stepn}->{olout} = "@lines";
# 	@m = split /\s+/, $lines[-1];
# 	$self->{reqs}->{$case}->{$mace}->{$stepn}->{stat2} = "$m[2]"."sec";
# 	$self->{mach}->{$mace}++;
# 	$self->{case}->{$case}++;
# 	$self->{globalresult} .= " $self->{reqs}->{$case}->{$mace}->{$stepn}->{stat1}" if $self->{globalresult};

    }

    return 0;
}

sub buildXMLFile {
    my $self = shift;
    $self->printStack(6);

    $self->{NARGS} = 2;

  my @aLogs = $self->{opts}->{log};

##  $self->{OPTS}->{njobs} = $self->{opts}->{njobs};

#   # If selected, the xml log file will have the same name as the ordinary log file plus .xml appended
#   if ($self->{opts}->{xml}) {
# #    $self->{XMLLOG} = $self->{opts}->{log} . ".xml";
#     $self->{XMLLOG} = $self->{xmlfile};  ##  GG
#     push (@aLogs,$self->{XMLLOG});
#   }

##????
#   foreach my $logs (@aLogs) {
#     # Touch the needed logfiles for the whole test.
#     # Later, every single command will append its log to these main logfiles.
#     system ("rm -rf $logs ; touch  $logs");
#     print "Logfile $logs created\n";
#   }

  # Check that the number of args arrives to the minimum set by the NARGS var:
#    if ($#{$self->{args}} + 1 < $self->{NARGS}) {
#       print "TestCycle: ERROR - Wrong number of arguments found, expect at least $self->{NARGS}\n";
#       print "<@{$self->{args}}>\n";
#       return 1;
#     }

  @{$self->{ARGSLIST}} = @{$self->{args}};
  my @aColsList = ();
  my @aRowsList = ();

  if ($self->{NARGS} == 2) {
    # 2D test
    @aColsList = @{$self->{ARGSLIST}};
    @aRowsList = @{$self->{ARGSLIST}};
  } else {
    # 1D test
    @aColsList = @{$self->{ARGSLIST}};
    @aRowsList = ("dummyOneElementRow");
  }

#     $handle->shlock();
#     my $bugger = $buffer;
#     $handle->shunlock();

#     $handle->shlock();
# 	my $buffer = \%buhher;

  # Launch the test cycle series here to build the table:
  foreach my $row (@aRowsList) {
#p      print STDOUT " Signal Value: $BaseTest::sigC \n";
##NO      if ( $BaseTest::sigC ) { last; }  ##  GG
    foreach my $col (@aColsList) {

	## GG for CTRL-C
#p	print STDOUT " Signal Value: $BaseTest::sigC \n";
##NO	if ( $BaseTest::sigC ) { last; }  ##  GG

      if ($row eq $col) {
	# For the diagonal elements, insert some dummy test results
##OLD	foreach my $step (@{$self->{STEPLIST}->{$self->{opts}->{cycle}}}) {
##HUM	    my @stepL = sort keys %{$buffer->{reqs}->{$row}->{$col}};
    my @stepL = (0..0);
	foreach my $step ( @stepL ) {
##??	  $self->{XMLOUT}->{$row}->{$col}->{$step}->{rc} = 0;  ##  GG  5;
#	  $buffer->{reqs}->{$row}->{$col}->{$step}->{rc} = "";  ##  GG  5;
	  $self->{reqs}->{$row}->{$col}->{$step}->{olout} = "Step not run. Need 2 different SEs";
#	  $buffer->{reqs}->{$row}->{$col}->{$step}->{olout} = "Step not run. Need 2 different SEs";
	}
      } else {
	# Run the cycle test
##replaced	$self->runStepCycle($row,$col);
# 	  my @pair = ($row, $col);
# 	  push @{$self->{pairh}}, \@pair;
#p	  print STDOUT " Adding: @pair \n";
      }
    }
  }
#    $handle->shunlock();


  my $global_retval=0;
  my $rowM = "";
    my @stepL = ();
#hum    $handle->shlock();
  foreach my $row (@aRowsList) {
    my $colM = "";
    foreach my $col (@aColsList) {
      my $stepM = "";
##OLD      foreach my $step (@{$self->{STEPLIST}->{$self->{opts}->{cycle}}}) {
      if ( 0 ) {
#	     @stepL = sort keys %{$buffer->{reqs}->{$row}->{$col}};
	} else {
     @stepL = (0..0);
}
	foreach my $step ( @stepL ) {
#BUF	    my @stepL = sort keys %{$self->{reqs}->{$row}->{$col}};
#	    my @stepL = sort keys %{$buffer->{reqs}->{$row}->{$col}};
	my $retv = $self->{reqs}->{$row}->{$col}->{$step}->{stat1};
	my $logf = $self->{reqs}->{$row}->{$col}->{$step}->{olout};
#     $handle->shlock();
# 	my $retv = $buffer->{reqs}->{$row}->{$col}->{$step}->{stat1};
# 	my $logf = $buffer->{reqs}->{$row}->{$col}->{$step}->{olout};
#     $handle->shunlock();
	if ( $row ne $col && $retv != 0) {  ##  GG
#p	    print STDOUT " global_retval: $retv $row $col \n";
	    $global_retval=1; 
	}
	$stepM .= makeX(makeX("$logf", "output").makeX("$retv", "retval"), "step");
	    print STDOUT " global_retval: $retv $row $col >@stepL< \n";
      }
      $colM .= makeX($stepM, "col");
    }
    $rowM .= makeX($colM, "row");
  }
#hum    $handle->shunlock();

  # Build now the info part of the XML output
  my $stepI = "";
##OLD  foreach my $step (@{$self->{STEPLIST}->{$self->{opts}->{cycle}}}) {
#    $handle->shunlock();
##HUM	    my @stepL = sort keys %{$buffer->{reqs}->{$row}->{$col}};
#    $handle->shunlock();
     @stepL = (0..0);
	foreach my $step ( @stepL ) {
    my $brf_info = $self->{STEPINFO}->{$step}->{brf_info};
    my $det_info = $self->{STEPINFO}->{$step}->{det_info};
    $stepI .= makeX(makeX("$brf_info", "brf_info").makeX("$det_info", "det_info"), "step");
  }

  my $body = $rowM;

  my $info = makeX($self->{SHORTINFO}, "title");
  $info .= makeX(`date`, "date");
  if ($self->{NARGS} == 2) {
    # Add the rows_list field only for 2D tests
      @aRowsList = map  { makeX($_, "row"); } @aRowsList;
      @aRowsList = grep { s/\./<br>\@\ / }    @aRowsList;
    $info .= makeX("@aRowsList", "rows_list");
  }
  @aColsList = map  { makeX($_, "col"); } @aColsList;
  @aColsList = grep { s/\./<br>\@\ /; }   @aColsList;
  $info .= makeX("@aColsList", "cols_list");
  $info .= $stepI;
  $info = makeX($info, "info");

  $info =~ s/lxshare/h/g;  ##  GG

  ## The whole log is made of INFO+BODY
  open XML, "> $self->{xmlfile}";
  print XML makeX($info." \n".$body, "log");
  close XML;

##GG
  my $global_result = ' [OK]';
  $global_result = ' [FAIL]' if $global_retval;
  $self->{globalresult} .= $global_result if $self->{globalresult};

  return $global_retval;
##    return 0;
}

## ***************************************************************************************
sub fillRH2DRun {
#prevGOOD    return unless $ENV{MAINSCRIPT};
    my $self = shift;
  my ($row, $col, $sname, $stepn, $sdesc, $cmd, $logflag, $timeout) = @_;

    ## Running step
	      my $tStart  = time();
    my $href = $self->runGommand($cmd, $logflag, $timeout);
	      my $tAlmost = time() - $tStart;


    ## Filling step
    return unless $self->{opts}->{xml};

    ## case = row, mace = col, scomm = cmd, stat1 = rc, olout = out
##    my ($case, $stat1, $olout, $mace, $sname, $stepn, $sdesc, $scomm) = @_;
    my ($case, $stat1, $olout, $mace, $sname, $stepn, $sdesc, $scomm) = ($row, 
									 $href->{rc}, 
									 $href->{out}, 
									 $col, 
									 $sname, 
									 $stepn,
									 $sdesc,
									 $cmd);

#p    print STDOUT " Control: $case, $stat1, $mace, $sname, $stepn, $sdesc \n";

#     $self->{reqs}->{$case}->{$mace}->{$stepn}->{stat1} = $stat1;
#     $self->{reqs}->{$case}->{$mace}->{$stepn}->{stat2} = $tAlmost;
#     $self->{reqs}->{$case}->{$mace}->{$stepn}->{olout} = $olout;
#     $self->{steps}->{$stepn}->{sname} = $sname;
#     $self->{steps}->{$stepn}->{sdesc} = $sdesc;
#     $self->{steps}->{$stepn}->{scomm} = $scomm;
     $self->{mach}->{$mace}++;
     $self->{case}->{$case}++;

#NO    my $file = "$self->{log}";
    my $file = "$self->{tempdir}/SLOG-$case-$mace-$stepn.log";
    print STDOUT " CONTROL: $case $mace $stepn $file \n";
    open LOG, ">$file";
    print LOG " CONTROL: $case $mace $stepn $file \n";
    print LOG " stat1: $stat1 \n";
    print LOG " stat2: $tAlmost sec \n";
    print LOG " sname: $sname \n";
    print LOG " sdesc: $sdesc \n";
    print LOG " scomm: $scomm \n";
    print LOG " machc: $self->{mach}->{$mace} \n";
    print LOG " casec: $self->{case}->{$case} \n";
    print LOG " olout:\n$olout \n";
    close LOG;

#     $handle->shlock();
#     my $buffer = \%buhher;
#     $buffer->{reqs}->{$case}->{$mace}->{$stepn}->{stat1} = $stat1;
#     $buffer->{reqs}->{$case}->{$mace}->{$stepn}->{stat2} = $tAlmost;
#     $buffer->{reqs}->{$case}->{$mace}->{$stepn}->{olout} = $olout;
#     $buffer->{steps}->{$stepn}->{sname} = $sname;
#     $buffer->{steps}->{$stepn}->{sdesc} = $sdesc;
#     $buffer->{steps}->{$stepn}->{scomm} = $scomm;
#     $buffer->{mach}->{$mace}++;
#     $buffer->{case}->{$case}++;
#     $handle->shunlock();

    return ($href->{rc}, $href->{out}) ;

}
# 	      my $tStart = time();
# ##	      my $rc = $self->testGatekeeper($host);
# 	      my $rc = $self->testRMSetupTest("serie", $i, $row, $col);
# 	      my $tAlmost = time() - $tStart;

# Utility function to issue a command
# What it does:
#   1-Writes the issued command to a tmp logfile
#   2-Issues the command with a timeout
#   3-Appends the output of this command to the main test logfile $self->{opts}->{log}
# Arguments:
#   $cmd: String with the command to be issued
#   $logflag: Flag to indicate wether to append the result to the main log (logflag=1, default) or not (logflag=0)
# Returns a hash ref with keys:
#   rc: Return code of the issued command
#   out: stdout+stderr of the issued command
sub runGommand {
  my $self = shift;
  my ($cmd, $logflag, $timeout) = @_;

  $timeout ||= 240;
  if (!defined $logflag) {$logflag = 1};

#p  print STDERR "runGommand(): Now running: \"$cmd\" \n";

  return 1 if ( ! defined $cmd );

#p  print STDERR "runGommand(): Now running: \"$cmd\" \n";

  print "runGommand(): Now running: \"$cmd\" \n";

  my $head = "\n";
  $head .= "*************************************** \n";
  $head .= "* Log for: $cmd \n";
  $head .= "*************************************** \n";

#  my $href = mytimeout($cmd,$timeout);
  ## The one with NO fork only works ...
  my $href = mytimeoutOLD($cmd,$timeout);

#p  print STDERR "runGommand(): Now running: \"$cmd\" \n";
  my $dateST = `date '+%y%m%d-%H%M%S'`;

  $head = $head . "++++ $$ $dateST";
  $head = $head . $href->{out};
  $head .= "*************************************** \n";

  # Finally, if temporary log file name was given, append it to the main test logfile
##NOTANYMORE  if ( $logflag != 0 ) {$self->logIt($head)}

#p  print STDOUT "\n$head\n";  ##  GG
#p  print STDERR "\n STDERR $head\n";  ##  GG

  $href->{out} = "$head";

  return $href;
}

# Building the HTML report from the XML report (gg)
# Subclasses should not need to override this method.
sub runXMLParserScript {
    my $self = shift;

#    print "BaseTest: runXMLParserScript\n";
#    $self->printLocation( (caller(0))[3] );
   $self->printStack( 3 );

    my $ptrbsf = $self->can('buildXMLFile2D');
    return 0 if ( ! $ptrbsf );
    my $errbsf = $self->buildXMLFile2D();
    return 1 if ( $errbsf );

##    my $parser = "log2html-script.sh";
#    my $html_out = "index.html";
    my $html_out = "$self->{htmlout}";
#    my $parser = "log2html-script.sh --styledir $EDG_TESTROOT/etc/parser_conf/html_style ";
#    my $parser = "log2html-script.pl --outdir $self->{htmldir} --outfile $html_out --styledir $ENV{EDG_TEST_HTML_STYLE} ";
    my $parser = "log2html-script.pl -j -f $self->{htmldir}/$html_out ";
    print STDERR " Running XMLParsing: $parser $self->{xmlfile} \n";
## Call this with "bash -x" or "bash -v" when needed ...
    my $resPars = `$parser $self->{xmlfile}`;
    print STDERR " After XMLParsing: \n$resPars \n" if ( $resPars ne "" ) ;

    return 0;

}

## Type with several rows, several steps and several columns 
## ***************************************************************************************
sub buildXMLFile2D {
 
    my $self = shift;

#    print "Stack: buildXMLFile\n";
    $self->{debug} = 1;
#    $self->printLocation( (caller(0))[3] );
   $self->printStack( 3 );

    my @lof_cols;
    my @lof_steps;
    my @lof_rows;
#notbad    my $Title = (caller(0))[3];
    my $Title = __PACKAGE__;
    chomp(my $date = `date`);
    my %ret = ( 
	     '[OK]' => 0,
	     '[FAIL]' => 1,
	     '[WARNING]' => 2,
	     '[XFAIL]' => 3,
	     '[UNRESOLVED]' => 4,
	     '[UNTESTED]' => 5,
	     '[UNSUPPORTED]' => 6
	    );

    print STDERR " Called buildXMLFile2D() to write $self->{xmlfile} \n";

#    my $cols = {};
    ## Col names
    foreach my $mace ( sort keys %{$self->{mach}} ) {
	push @lof_cols, $mace;
    }
#   print " COLS: @lof_cols \n";

#    my $cols = {};
    ## Row names
    foreach my $case ( sort keys %{$self->{case}} ) {
	push @lof_rows, $case;
    }
#   print " ROWS: @lof_rows \n";

    ## List of steps ... (steps are numbers)
    foreach my $stepn ( sort { $a <=> $b } keys %{$self->{steps}} ) {
	push @lof_steps, "Step_$stepn";
    }

    my $nsteps = scalar(@lof_steps)-1;
    print STDERR " Called buildXMLFile2D() to cols @lof_cols \n";
    print STDERR " Called buildXMLFile2D() to rows @lof_rows \n";
    my $rowM = "";
    foreach my $row ( @lof_rows ) {
	my $colM = "";
	foreach my $mace ( @lof_cols ) {
	    my $stepM = "";
	    for my $stepn (0..$nsteps) {
		my $retv = $ret{$self->{reqs}->{$row}->{$mace}->{$stepn}->{stat1}};
		my $subs = $self->{reqs}->{$row}->{$mace}->{$stepn}->{stat2};
		my $logf = $self->{reqs}->{$row}->{$mace}->{$stepn}->{olout};
		$logf =~ s/&/&amp;/g; # This must be changed first.
		$logf =~ s/\"/&quot;/g;
		$logf =~ s/</&lt;/g;
		$logf =~ s/>/&gt;/g;
#good		$stepM .= makeX(makeX("$logf", "output").makeX("$retv", "retval"), "step");
		$stepM .= makeX(makeX("$logf", "output").makeX("$retv", "retval").makeX("$subs", "substatus"), "step");
	    }
	    $colM .= makeX($stepM, "col");
	}
	$rowM .= makeX($colM, "row");
    }

    ## building the step info list ...
    my $stepI = "";
    foreach my $stepn ( sort { $a <=> $b } keys %{$self->{steps}} ) {
	my $brf_info = "Step_$stepn";
#old	my $brf_info = $self->{steps}->{$stepn}->{sname};
	$brf_info =~ s/ /_/g;
#old	my $det_info = $self->{steps}->{$stepn}->{sdesc}." : \n".$self->{steps}->{$stepn}->{scomm};
	my $det_info = $self->{steps}->{$stepn}->{sname}." : \n".$self->{steps}->{$stepn}->{sdesc}." : \n".$self->{steps}->{$stepn}->{scomm};
	$stepI .= makeX(makeX("$brf_info", "brf_info").makeX("$det_info", "det_info"), "step");
    }

    my $body = $rowM;

#old my $info = makeX($Title, "title");
my $info = makeX($self->{type}, "title");
$info .= makeX($date, "date");
@lof_rows = map { makeX($_, "row"); } @lof_rows;
@lof_rows = grep { s/\./<br>\@\ / }    @lof_rows;
$info .= makeX("@lof_rows", "rows_list");
@lof_cols = map { makeX($_, "col"); } @lof_cols;
@lof_cols = grep { s/\./<br>\@\ / }    @lof_cols;
$info .= makeX("@lof_cols", "cols_list");
$info .= $stepI;
$info = makeX($info, "info");

  $info =~ s/lxshare/h/g;  ##  GG

## The whole log is made of INFO+BODY
open XML, "> $self->{xmlfile}";
print XML makeX($info." \n".$body, "log");
close XML;
#p print STDERR makeX($info." \n ".$body, "log");

    return 0;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
