#!/bin/bash
# 
# 19 SEP 2002
# Author:       Fede Eric fede@cppm.in2p3.fr
#######################################################################
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
#######################################################################
#
# Requires :
#       - The EDG_TESTROOT environment variable being defined
#
#
########################################
# Main script to the the CERTIFICAT STATUS
# 
# RETURN VALUE: 0 PASSED
#	 	1 FAIL
#		2 WARNING
#		3 XFAIL
#		4 UNRESOLVED
#		5 UNTESTED
#		6 UNSUPPORTED
#               7 ENVIRONMENT / OPTIONS FAIL
#	
########################################
# The default value

VERBOSE_LEVEL=0	
#0 ->output on log file , 1 -> output on logfile and screen 
EDG_TSTG_LOGS_PATH=Testsecurity.log
# path to the logfile 
EDG_TSTG_CERTIF_PATH=/etc/grid-security/certificates/
# Path to the CA and CRL
EDG_TSTG_HOST_CERTIFICAT=/etc/grid-security/hostcert.pem

#Path to the Host certificat
#
# load the function
#
source $EDG_TESTROOT/sbin/common.sh
source $EDG_TESTROOT/sbin/fileExist.sh
source $EDG_TESTROOT/sbin/echoFunc.sh
source $EDG_TESTROOT/sbin/checkService.sh
source writeOutput.sh
source $EDG_TESTROOT/sbin/findOpenssl.sh
source verifyHostCertValidity-func.sh
source verifyCRLversusCA-func.sh
source verifyCRLTimeValidity-func.sh
source verifyNbCAeqNbCRL-func.sh

# 
# test the script option

if test "$#" -gt "0" ; then	
case $1 in
	--help|-help|-help)
		echo ' Command syntax';
		echo ' --help : the command inline help';
		echo ' --v : the verbose mode'
		exit 0;
	;;
	--version|-version|-V)
		echo ' Version 0.3';
		exit 0;
	;;
	
	--v|-v|-verbose)
	VERBOSE_LEVEL=1
	;;
	*)
	echo ' Unknow option please refere to the help';
		exit 0;
	;;
esac
fi

# Initialization of the status variable
STATUS=0; 

#try to find openssl binarie
$ECHO "Find Openssl";
findOpenssl;
print_status;


#test if host certificat is valid
$ECHO "Host Certificat Validity";
verifyHostCertValidity_func $EDG_TSTG_HOST_CERTIFICAT $TSTG_OPENSSL_LOCATION;
print_status;

#test if deamon is running
$ECHO "EDG-crl-upgraded deamon";
checkservice edg-crl-upgraded;
print_status;

#test if ntpd is running
$ECHO "ntpd deamon";
checkservice ntpd;
print_status;

#test the number of Ca versus Number of CRL
$ECHO "Nb CRL equal nb CA";
verifyNbCAeqNbCRL_func $EDG_TSTG_CERTIF_PATH $EDG_TSTG_LOGS_PATH;
print_status;

# test if for each CA the corresponding CRL exist
$ECHO "Check CA versus CRL";
verifyCRLversusCA_func $EDG_TSTG_CERTIF_PATH $TSTG_OPENSSL_LOCATION;
print_status;


# test if each CRL is valid
$ECHO "Check CRL validity";
verifyCRLTimeValidity_func  $EDG_TSTG_CERTIF_PATH $TSTG_OPENSSL_LOCATION;
print_status;
