#!/bin/bash

######################################################################
#
# Authors: Alessandro Cavalli  - cavalli@cnaf.infn.it
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
######################################################################

######################################################################
# FUNCTION: ceinfoprovcheck
#
# ARGUMENTS:
#
# DESCRIPTION: Parses the local GRIS configuration file, produces and
#              examines the info providers' output to check that it's
#              schema-compliant
#
# RETURN VALUE: 0 - if everything is OK
#               1 - if the gris configuration file isn't as expected or
#                   if required data is missing from the I.P. output
#               4 - if cannot find the gris configuration file
######################################################################

ceinfoprovcheck()
{
    echo

    CEIDOK=no
    CLOSESEOK=no
    MISSING=no
    MISSING_MAY=no

    CE_ATTR_LIST=`echo "CEId GlobusResourceContactString GRAMVersion Architecture OpSys \
   MinPhysicalMemory MinLocalDiskSpace TotalCPUs FreeCPUs NumSMPs MinSPUProcessors \
   MaxSPUProcessors TotalJobs RunningJobs IdleJobs MaxTotalJobs MaxRunningJobs \
   WorstTraversalTime EstimatedTraversalTime Active Priority MaxCPUTime \
   MaxWallClockTime AverageSI00 MinSI00 MaxSI00 AuthorizedUser \
   RunTimeEnvironment AFSAvailable OutboundIP InboundIP" | tr 'A-Z' 'a-z'`

    CE_ATTR_LIST_MAY=`echo "QueueName LRMSType LRMSVersion" | tr 'A-Z' 'a-z'`

    CLOSESE_ATTR_LIST=`echo "CloseSE CEId" | tr 'A-Z' 'a-z'`

    CLOSESE_ATTR_LIST_MAY=`echo "MountPoint" | tr 'A-Z' 'a-z'`

    cat /dev/null > /tmp/$$_all

    if [ -e /opt/edg/info/mds/etc/testbed1-site-globus.ldif ] ; then
	while read attribute value
	do
	    if [ "$attribute" = "path:" ] ; then
		TMP_PATH=$value
		read attribute value
		
		if [ "$attribute" = "base:" ] ; then
		    TMP_BASE=$value
		else
		    echo "ERROR: unexpected attribute sequence in gris configuration file"
                    echo
		    return 4
		fi
		read attribute value
		
		if [ "$attribute" = "args:" ] ; then
		    TMP_ARGS=$value
		else
		    echo "ERROR: unexpected attribute sequence in gris configuration file"
                    echo
		    return 4
		fi
		
		if [ "$TMP_BASE" = "ce-pbs" -o "$TMP_BASE" = "ce-lsf" ] ; then
		    $TMP_PATH/$TMP_BASE $TMP_ARGS | tr 'A-Z' 'a-z' >> /tmp/$$_all
		fi
	    fi
	done < /opt/edg/info/mds/etc/testbed1-site-globus.ldif 
    else
	echo "ERROR: gris configuration file not found"
        echo
	return 1
    fi

    while read attribute value
      do
      cat /dev/null > /tmp/$$_entry

      if [ "$attribute" = "dn:" ] ; then
      	
	  if [ "`echo $value | cut -c1-5`" = "ceid=" ] ; then
	      echo "Analizyng ceId: " `echo ${value} |sed -e "s/,.*//g" `
	      CEIDOK=yes

	      while read attribute value
		do
		echo ${attribute} ${value} >> /tmp/$$_entry

		if [ -z "${attribute}" -a -z "${value}" ] ; then
               
		    for attr in ${CE_ATTR_LIST} ; do
			if [ -z  "`grep -i ${attr}: /tmp/$$_entry | sed -n -e 1p | sed -e \"s/.*: //g\"`" ] ; then
			    echo "Attribute '${attr}' not defined"
			    MISSING=yes
			fi
		    done

		    for attr in ${CE_ATTR_LIST_MAY} ; do
			if [ -z  "`grep -i ${attr}: /tmp/$$_entry | sed -n -e 1p | sed -e \"s/.*: //g\"`" ] ; then
			    echo "Optional attribute '${attr}' not defined"
			    MISSING_MAY=yes
			fi
		    done
               
		    echo
		    continue 2
		fi
	      done
	  fi

	  if [ "`echo $value | cut -c1-5`" = "close" ] ; then
	      echo "Analizyng closeSE: " `echo ${value} |sed -e "s/,.*//g" `
	      CLOSESEOK=yes

	      while read attribute value
		do
		echo ${attribute} ${value} >> /tmp/$$_entry

		if [ -z "${attribute}" -a -z "${value}" ] ; then

		    for attr in ${CLOSESE_ATTR_LIST} ; do
			if [ -z  "`grep -i ${attr}: /tmp/$$_entry | sed -n -e 1p | sed -e \"s/.*: //g\"`" ] ; then
			    echo "Attribute '${attr}' not defined"
			    MISSING=yes
			fi
		    done

		    for attr in ${CLOSESE_ATTR_LIST_MAY} ; do
			if [ -z  "`grep -i ${attr}: /tmp/$$_entry | sed -n -e 1p | sed -e \"s/.*: //g\"`" ] ; then
			    echo "Optional attribute '${attr}' not defined"
			    MISSING_MAY=yes
			fi
		    done
               
		    echo
		    continue 2
		fi
	      done
	  fi

      fi
    done < /tmp/$$_all

    rm -f /tmp/$$_entry
    rm -f /tmp/$$_all

    if [ "${CEIDOK}${CLOSESEOK}${MISSING}${MISSING_MAY}" = "yesyesnono" ] ; then
	echo "The GRIS information providers for the Computing Element"
        echo "are correctly configured"
	echo
	return 0
    fi

    if [ "${MISSING}${MISSING_MAY}" = "noyes" ] ; then
	echo "The GRIS information providers for the Computing Element"
        echo "are correctly configured"
	echo "But one or more OPTIONAL attributes are not defined"
	echo
	return 0
    fi

    if [ "${CEIDOK}" = "no" ] ; then
	echo "ERROR: no CEID data returned from GRIS."
    fi

    if [ "${CLOSESEOK}" = "no" ] ; then
	echo "ERROR: no CloseSE data returned from GRIS."
    fi

    if [ "${MISSING}" = "yes" ] ; then
	echo "ERROR: missing attribute(s)"
    fi

    echo
    return 1
}
