#!/bin/bash
# 
# 08 OCTOBER 2002
# Author:       Fede Eric fede@cppm.in2p3.fr
#
#
########################################################################
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
########################################################################
#######################################################################
#
# Requires :
#       - The EDG_TESTROOT environment variable being defined
#
#
########################################
# Main script to Test a CE
# 
# RETURN VALUE: 0 PASSED
#	 	1 FAIL
#		2 WARNING
#		3 XFAIL
#		4 UNRESOLVED
#		5 UNTESTED
#		6 UNSUPPORTED
#               7 ENVIRONMENT / OPTIONS FAIL
#	
########################################

# The default value
VERBOSE_LEVEL=0	
#0 ->output on log file , 1 -> output on logfile and screen 

EDG_TSTG_LOGS_PATH=TestCE.log
# path to the logfile 


# load the function
#
source $EDG_TESTROOT/sbin/common.sh
source $EDG_TESTROOT/sbin/echoFunc.sh
source $EDG_TESTROOT/sbin/checkService.sh
source $EDG_TESTROOT/sbin/fileExist.sh
source writeOutput.sh
source $EDG_TESTROOT/sbin/randomfilecreate-func.sh
source testgridproxyexist-func.sh
source $EDG_TESTROOT/sbin/checkMD5FiletoFile-func.sh


if test "$#" -gt "0" ; then	
case $1 in
	--help|-help|-help)
		echo ' Command syntax';
		echo ' --help : the command inline help';
		echo ' --h Remote host : Optional option to define a remote host name where some realistic test of copy can be done'; 
		echo ' --v : the verbose mode';
		exit 0;
	;;
	--version|-version|-V)
		echo ' Version 0.1';
		exit 0;
	;;
	
	--v|-v|-verbose)
	VERBOSE_LEVEL=1;
	;;
	--h|-h|-host)
	REMOTE_HOST=$2;
	shift;
	;;
	*)
	echo ' Unknow option please refere to the help';
		exit 0;
	;;
esac
fi


# test the availability of a gridproxy
$ECHO "Grid proxy availability";
testgridproxyexist_func;
print_status;

# test the availability of a gatekeeper deamon
$ECHO "globus-gatekeeper daemon";
checkservice globus-gatekeeper;
print_status;

# test the availability of a gsiftp deamon
$ECHO "globus-gsi_wuftpd daemon";
checkservice globus-gsi_wuftpd;
print_status;

#creation of a file
Filename="/tmp/file.init"
randomfilecreate_func $Filename 1024


IPADRESS=`hostname`;
Filecopy="/tmp/file.copy"

#Test of local copy with  globus-url-copy from file to file

$EDG_TESTROOT/sbin/timeout-script.exp 10 "globus-url-copy file:$Filename file:$Filecopy 2>> $EDG_TSTG_LOGS_PATH"

$ECHO "Check globus-url-copy from file to file";
checkMD5filetofile_func $Filename $Filecopy;
print_status;

$RM -f $Filecopy;

#Test of local copy with  globus-url-copy from file to gsiftp

$EDG_TESTROOT/sbin/timeout-script.exp 10 "globus-url-copy file:$Filename gsiftp://$IPADRESS$Filecopy 2>> $EDG_TSTG_LOGS_PATH";
$ECHO "Check globus-url-copy from file to gsiftp on local host";
checkMD5filetofile_func $Filename $Filecopy;
print_status;

$RM -f $Filecopy;
#Test of local copy with  globus-url-copy from gsiftp to gsiftp

$EDG_TESTROOT/sbin/timeout-script.exp 10 "globus-url-copy gsiftp://$IPADRESS$Filename gsiftp://$IPADRESS$Filecopy 2>> $EDG_TSTG_LOGS_PATH";
$ECHO "Check globus-url-copy from gsiftp to gsiftp on local host";
checkMD5filetofile_func $Filename $Filecopy;
print_status;

$RM -f $Filecopy;

#Test if remote host parameter

if [ $REMOTE_HOST ]; then

#Test copy with  globus-url-copy from file to gsiftp on remote host

$ECHO "Check globus-url-copy from file to gsiftp remote host";
$EDG_TESTROOT/sbin/timeout-script.exp 10 "globus-url-copy file:$Filename gsiftp://$REMOTE_HOST$Filecopy 2>> $EDG_TSTG_LOGS_PATH";
print_status;

#Test copy with  globus-url-copy from local host by gsiftp to gsiftp on remote host
$ECHO "Check globus-url-copy from gsiftp on local host to gsiftp remote host";
$EDG_TESTROOT/sbin/timeout-script.exp 10 "globus-url-copy gsiftp://$IPADRESS$Filename gsiftp://$REMOTE_HOST$Filecopy 2>> $EDG_TSTG_LOGS_PATH";
print_status;

fi
