#!/bin/bash

######################################################################
#
# Authors: Alessandro Cavalli  - cavalli@cnaf.infn.it
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
######################################################################

######################################################################
# FUNCTION: seinfoprovcheck
#
# ARGUMENTS:
#
# DESCRIPTION: Parses the local GRIS configuration file, produces and
#              examines the info providers' output to check that it's
#              schema-compliant
#
# RETURN VALUE: 0 - if everything is OK
#               1 - if the gris configuration file isn't as expected or
#                   if required data is missing from the I.P. output
#               4 - if cannot find the gris configuration file
######################################################################

seinfoprovcheck()
{
    echo

    SEIDOK=no
    SEPROTOK=no
    SESTATOK=no
    MISSING=no
    MISSING_MAY=no

    SE_ATTR_LIST=`echo "SEId CloseCE SEtypearchitecture SEsize SEResourceContactString SEvo" | tr 'A-Z' 'a-z'`

    SE_ATTR_LIST_MAY=`echo "SESubjectName SEmaxfilesize SEmaxdata SEmaxnumfile" | tr 'A-Z' 'a-z'`

    SE_PROT_ATTR_LIST=`echo "SEProtocol SEId" | tr 'A-Z' 'a-z'`

    SE_PROT_ATTR_LIST_MAY=`echo "Port" | tr 'A-Z' 'a-z'`

    SE_STAT_ATTR_LIST=`echo "SEfreespace" | tr 'A-Z' 'a-z'`

    SE_STAT_ATTR_LIST_MAY=`echo "SEId SEaccesstime SEfilequeuelen" | tr 'A-Z' 'a-z'`


    cat /dev/null > /tmp/$$_all

    if [ -e /opt/edg/info/mds/etc/testbed1-site-globus.ldif ] ; then
	while read attribute value
	do
	    if [ "$attribute" = "path:" ] ; then
		TMP_PATH=$value
		read attribute value
		
		if [ "$attribute" = "base:" ] ; then
		    TMP_BASE=$value
		else
		    echo "ERROR: unexpected attribute sequence in gris configuration file"
                    echo
		    return 4
		fi
		read attribute value
		
		if [ "$attribute" = "args:" ] ; then
		    TMP_ARGS=$value
		else
		    echo "ERROR: unexpected attribute sequence in gris configuration file"
                    echo
		    return 4
		fi
		
		if [ "$TMP_BASE" = "se-disk" ] ; then
		    $TMP_PATH/$TMP_BASE $TMP_ARGS | tr 'A-Z' 'a-z' >> /tmp/$$_all
		fi
	    fi
	done < /opt/edg/info/mds/etc/testbed1-site-globus.ldif 
    else
	echo "ERROR: gris configuration file not found"
        echo
	return 1
    fi

    while read attribute value
      do
      cat /dev/null > /tmp/$$_entry

      if [ "$attribute" = "dn:" ] ; then
      	
	  if [ "`echo $value | cut -c1-5`" = "seid=" ] ; then
	      echo "Analizyng seId: " `echo ${value} |sed -e "s/,.*//g" `
	      SEIDOK=yes

	      while read attribute value
		do
		echo ${attribute} ${value} >> /tmp/$$_entry

		if [ -z "${attribute}" -a -z "${value}" ] ; then
               
		    for attr in ${SE_ATTR_LIST} ; do
			if [ -z  "`grep -i ${attr}: /tmp/$$_entry | sed -n -e 1p | sed -e \"s/.*: //g\"`" ] ; then
			    echo "Attribute '${attr}' not defined"
			    MISSING=yes
			fi
		    done

		    for attr in ${SE_ATTR_LIST_MAY} ; do
			if [ -z  "`grep -i ${attr}: /tmp/$$_entry | sed -n -e 1p | sed -e \"s/.*: //g\"`" ] ; then
			    echo "Optional attribute '${attr}' not defined"
			    MISSING_MAY=yes
			fi
		    done
               
		    echo
		    continue 2
		fi
	      done
	  fi

	  if [ "`echo $value | cut -c1-5`" = "sepro" ] ; then
	      echo "Analizyng SeProtocol: ${value}"
	      SEPROTOK=yes
	      PROT_FILE=no

	      while read attribute value
		do
		echo ${attribute} ${value} >> /tmp/$$_entry

		if [ "${attribute}" = "seprotocol:" -a "${value}" = "file" ] ; then
		    PROT_FILE=yes
		fi

		if [ -z "${attribute}" -a -z "${value}" ] ; then

		    for attr in ${SE_PROT_ATTR_LIST} ; do
			if [ -z  "`grep -i ${attr}: /tmp/$$_entry | sed -n -e 1p | sed -e \"s/.*: //g\"`" ] ; then
			    echo "Attribute '${attr}' not defined"
			    MISSING=yes
			fi
		    done

		    for attr in ${SE_PROT_ATTR_LIST_MAY} ; do
			if [ -z  "`grep -i ${attr}: /tmp/$$_entry | sed -n -e 1p | sed -e \"s/.*: //g\"`" ] ; then
			    if [ "${PROT_FILE}" != "yes" ] ; then
                                echo $PROT_FILE
				echo "Optional attribute '${attr}' not defined"
				MISSING_MAY=yes
			    fi
			fi
		    done
               
		    echo
		    continue 2
		fi
	      done
	  fi

	  if [ "`echo $value | cut -c1-5`" = "in=st" ] ; then
	      echo "Analizyng SeStatus: ${value}"
	      SESTATOK=yes
 
	      while read attribute value
		do
		echo ${attribute} ${value} >> /tmp/$$_entry
 
		if [ -z "${attribute}" -a -z "${value}" ] ; then
 
		    for attr in ${SE_STAT_ATTR_LIST} ; do
			if [ -z  "`grep -i ${attr}: /tmp/$$_entry | sed -n -e 1p | sed -e \"s/.*: //g\"`" ] ; then
			    echo "Attribute '${attr}' not defined"
			    MISSING=yes
			fi
		    done
 
		    for attr in ${SE_STAT_ATTR_LIST_MAY} ; do
			if [ -z  "`grep -i ${attr}: /tmp/$$_entry | sed -n -e 1p | sed -e \"s/.*: //g\"`" ] ; then
			    echo "Optional attribute '${attr}' not defined"
			    MISSING_MAY=yes
			fi
		    done
 
		    echo
		    continue 2
		fi
	      done
	  fi
      fi
    done < /tmp/$$_all

    rm -f /tmp/$$_all
    rm -f /tmp/$$_entry

    if [ "${SEIDOK}${SEPROTOK}${SESTATOK}${MISSING}${MISSING_MAY}" = "yesyesyesnono" ] ; then
        echo "The GRIS information providers for the Storage Element"
        echo "are correctly configured"
	echo
	return 0
    fi

    if [ "${MISSING}${MISSING_MAY}" = "noyes" ] ; then
        echo "The GRIS information providers for the Storage Element"
        echo "are correctly configured"
	echo "But one or more OPTIONAL attributes are not defined"
	echo
	return 0
    fi

    if [ "${SEIDOK}" = "no" ] ; then
	echo "ERROR: no SEID data returned from GRIS."
    fi

    if [ "${SEPROTOK}" = "no" ] ; then
	echo "ERROR: no SeProtocol data returned from GRIS."
    fi

    if [ "${SESTATOK}" = "no" ] ; then
	echo "ERROR: no SeStatus data returned from GRIS."
    fi

    if [ "${MISSING}" = "yes" ] ; then
	echo "ERROR: missing attribute(s)"
    fi

    echo
    return 1
}
