#!/bin/sh
#
######################################################################
#
# Authors: Andrea Formica   - Andrea.Formica@cea.fr
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
# Based on INFN Testgroup scripts
#
######################################################################

##################################################################
# FUNCTION: add-list <flag> <input file> <list> 
#
# ARGUMENTS: <flag>    {string} (mandatory)
#            <input file> {string} (mandatory)
#            <list>    {string} 
#
# Input:  <list>              Write out a list of EDG packages marked by <flag>
#                             and write the list to the output file indicated     
#                             by <output file>. <list> parameter can be empty
#
#         <flag>              INSTALL,UPGRADE,ORIGINAL,DISTRIBUTION....
#
# RETURN VALUE: 0 - list 
#               1 - file not found
#	        4 - no mandatory parameter is passed
#
##################################################################
#
add_list()
{

    PRESTRING="${PRESTRING}+"
    local THIS_SCRIPT="add-list.sh"
    local HOUR=`date | awk '{print $4}'`
    local PRETXT="[${THIS_SCRIPT} ${HOUR}]:"
    
    if [ $LEVEL -gt 1 ]
    then
        PRETXT="$PRESTRING [${THIS_SCRIPT} ${HOUR}]:"
    fi

    if [ -z $1 ]
    then
        [ $DEBUG ] && writeOutput "add-list(): no mandatory parameter is passed"
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: input  $1={flag}"
        PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
	return 4
    fi
    if [ -z $2 ]
    then
        [ $DEBUG ] && writeOutput "add-list(): no mandatory parameter is passed"
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: input  $2={output file}"
        PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
	return 4
    fi

    
    local FLAG="$1"
    local INST_INFO_FILE=$2
    local mylist="$3"
    local FILEHEAD=$2.head
    local FILETAIL=$2.tail
    local status=0
    
    [ $DEBUG ] && writeOutput "${PRETXT} fileexist $INST_INFO_FILE" 
    fileexist $INST_INFO_FILE
#    print_status
    [ $? != 0 ] && (PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`; return 1)

    a=0
    nlines=`wc -l $INST_INFO_FILE |  awk '{print $1}'`
    [ $nlines -gt 0 ] && a=$[nlines-1]
    
    head -$a $INST_INFO_FILE > $FILEHEAD
    tail -1 $INST_INFO_FILE > $FILETAIL

    [ $LEVEL -gt 0 ] && writeOutput "${PRETXT} write-list add $FLAG field"
    write-list "$FLAG" $FILEHEAD "$mylist"; [ $? != 0 ] && status=$[ status + 1 ] 

    cat $FILETAIL >> $FILEHEAD
    rm $FILETAIL
    mv $FILEHEAD $INST_INFO_FILE

# Check the status
    if [ $status -ge 1 ]
    then
        PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
        return 1        
    fi

    [ $DEBUG ] && writeOutput "${PRETXT} ### END" 
    PRESTRING=`echo ${PRESTRING} | cut -c 2-`
    
    return 0
}
