#!/bin/bash
#$Id: checkBrokerInfo.sh,v 1.1.1.2 2005/02/08 16:09:43 mbarroso Exp $
######################################################################
#
# Authors: Livio Salconi - Livio.Salconi@pi.infn.it
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
# 
######################################################################
# Modified : 29/01/03 Frederique Chollet chollet@lapp.in2p3.fr
##################################################################
# FUNCTION: checkbrokerinfo
#
# ARGUMENTS:
#
# DESCRIPTION: check if edg-brokerinfo is correctly installed
#
# RETURN VALUE: 0 - if all is Ok
#               1 - if it is not OK
#               4 - if some variables or files are missing
#
##################################################################
checkbrokerinfo() 
{
#
# Check environment variables
#
checkenvvariable EDG_LOCATION
if [ $? != 0 ] 
then
    return 4
fi
checkenvvariable EDG_WL_PATH
if [ $? != 0 ] 
  then
    return 4
fi
checkenvvariable EDG_WL_LOCATION
if [ $? != 0 ] 
  then
    return 4
fi
checkenvvariable EDG_WL_LIBRARY_PATH
if [ $? != 0 ] 
  then
    return 4
fi
#
# Check for needed files
#
fileexist $EDG_LOCATION/bin/edg-brokerinfo
if [ $? != 0 ] 
  then
    return 4
fi 
fileexist $EDG_LOCATION/lib/libBrokerInfo.a
if [ $? != 0 ] 
  then
    return 4
fi
fileexist $EDG_LOCATION/lib/libBrokerInfo.la
if [ $? != 0  ] 
  then
    return 4
fi
fileexist $EDG_LOCATION/lib/libBrokerInfo.so
if [ $? != 0 ] 
  then
    return 4
fi
fileexist $EDG_LOCATION/lib/libBrokerInfo.so.0.1.0
if [ $? != 0 ] 
  then
    return 4
fi
#
# Create a BrokerInfo file on the fly 
# Check if it is parsed correctly by the edg-brokerinfo command
#
$ECHO "[" >/tmp/test_$$.BrokerInfo
$ECHO "CE=\"lxde01.pd.infn.it:2119/jobmanager-lsf-grid01\";" >>/tmp/test_$$.BrokerInfo
$ECHO "DataAccessProtocol={\"gridftp\",\"file\"};" >>/tmp/test_$$.BrokerInfo
$ECHO "InputPFNs={\"se1.pd.infn.it/data01/file762.dat\"};" >>/tmp/test_$$.BrokerInfo
$ECHO "LFNs={\"file67.dat\",\"filex\",\"mndb\"};" >>/tmp/test_$$.BrokerInfo
$ECHO "PFNs={">>/tmp/test_$$.BrokerInfo
$ECHO "          {\"se1.pd.infn.it/data00/file67.dat\",\"se.cern.ch/cms/mn/file67.dat\"}," >>/tmp/test_$$.BrokerInfo
$ECHO "          {\"se1.pd.infn.it/data00/filex\",\"se.in2p3.fr/data/00/filex\"}," >>/tmp/test_$$.BrokerInfo
$ECHO "          {\"se.cern.ch/cms/mn/cal1.DB\"}" >>/tmp/test_$$.BrokerInfo
$ECHO "};" >>/tmp/test_$$.BrokerInfo
$ECHO "SEs={\"se1.pd.infn.it\",\"se.cern.ch\",\"se.in2p3.fr\"};" >>/tmp/test_$$.BrokerInfo
$ECHO "SEProtocols={" >>/tmp/test_$$.BrokerInfo
$ECHO "              {\"gridftp\",\"file\"}," >>/tmp/test_$$.BrokerInfo
$ECHO "              {\"gridftp\",\"rfio\"}," >>/tmp/test_$$.BrokerInfo
$ECHO "              {\"gridftp\"}" >>/tmp/test_$$.BrokerInfo
$ECHO "};" >>/tmp/test_$$.BrokerInfo
$ECHO "SEPorts={" >>/tmp/test_$$.BrokerInfo
$ECHO "           {\"4444\",undefined}," >>/tmp/test_$$.BrokerInfo
$ECHO "           {\"4444\",\"5555\"}," >>/tmp/test_$$.BrokerInfo
$ECHO "           {\"4433\"}" >>/tmp/test_$$.BrokerInfo
$ECHO "};" >>/tmp/test_$$.BrokerInfo
$ECHO "CloseSEs={\"se1.pd.infn.it\",\"se2.pd.infn.it\"};" >>/tmp/test_$$.BrokerInfo
$ECHO "SEMountPoint={\"/disk1\",undefined};" >>/tmp/test_$$.BrokerInfo
$ECHO "]" >>/tmp/test_$$.BrokerInfo

export EDG_WL_RB_BROKERINFO=/tmp/test_$$.BrokerInfo
retval=`$EDG_LOCATION/bin/edg-brokerinfo getCE | $GREP "lxde01.pd.infn.it:2119/jobmanager-lsf-grid01" | $WC -l`
#$RM /tmp/test_$$.BrokerInfo

if [ $retval = 0 ]
  then
    return 1
fi

return 0
}


