#!/bin/bash
#$Id: checkMultipleServices.sh,v 1.1.1.2 2005/02/08 16:09:43 mbarroso Exp $
######################################################################
#
# Authors: Gennaro Tortone   - Gennaro.Tortone@na.infn.it
#          Mario Reale       - Mario.Reale@cnaf.infn.it
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
######################################################################

##################################################################
# FUNCTION: checkmultipleservices <service> <selection-string>
#
# ARGUMENTS: <service> {string} (mandatory)  
#            <selection-string> (mandatory)
#
# DESCRIPTION: check if <service> is running, uses a second string to
#              select specific deamon among many returned ones.
# 
# RETURN VALUE: 0 - if <service> is running 
#               1 - if <service> is not running 
#               4 - no mandatory parameter is passed
##################################################################

checkmultipleservices()
{
   if [ -z $1 ]
   then
        [ $DEBUG ] && $ECHO "checkmultipleservices(): no mandatory parameter is passed"
	return 4
   fi

  if [ -z $2 ]
   then
        [ $DEBUG ] && $ECHO "checkmultipleservices(): no mandatory parameter is passed"
        return 4
   fi

   local status

   # first check in /etc/rc.d/init.d

   if [ -e /etc/rc.d/init.d/$1 ]
   then
   	status=`/etc/rc.d/init.d/$1 status | $GREP running | $GREP $2  > /dev/null ; echo $?`

	if [ $status = 0 ]
	then
   		[ $DEBUG ] && $ECHO "checkm.s.(): serv $1 $2 is running"
		return 0
	else
		[ $DEBUG ] && $ECHO "checkm.s.(): serv $1 $2 is NOT running"
		return 1
	fi

   else
   
   # second check with 'netstat -a'

   	status=`$NETSTAT -a | $GREP $1 | $GREP LISTEN | $GREP $2 > /dev/null ; echo $?` 

   	if [ $status = 0 ]
   	then
		[ $DEBUG ] && $ECHO "checkm.serv.(): service $1 $2 is running"
		return 0
   	else
		[ $DEBUG ] && $ECHO "checkm.serv.(): service $1 $2 is NOT running"
		return 1
	fi
   fi
}
