#!/bin/bash
######################################################################
#
# Authors: Mario Reale   - Mario.Reale@cnaf.infn.it
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
# Modified : Apr.07 03 TSTG
#            Frederique Chollet  chollet@lapp.in2p3.fr
#             
#            This test makes use of sudo if EDG_TEST_USER is not set to root
#	     In this case, EDG_TEST_USER must be run by a permitted 
#            user specified in the sudoers file
######################################################################
#
#FUNCTION: checkuserenv <user> <env-variable>
#
# ARGUMENTS: <user> {string} (mandatory)
#            <env-variable> (string)  (mandatory)
#
# DESCRIPTION: check if a user has a given env variable defined in his
#                       default environment -
#
# RETURN VALUE: 0 - if <env-var> exists
#               1 - if <env-var> does not exist
#               4 - no mandatory parameter is passed = Unresolved
#               5 - untested if EDG_TEST_USER not root and $SUDO uninstalled
##################################################################

checkuserenv()
{
 
 if ! test "$EDG_TEST_USER" = "root" 
 then
 # see if we can use $SUDO
    which $SUDO > /dev/null 2>&1
    if [ $? = 1 ]
    then
	[ $DEBUG ] && $ECHO "checkuserenv(): $SUDO is not installed in this computer" 1>&2
	return 5
    fi
 else
    # test if I am really running as root
    if ! test `whoami` = "root"
    then
	 [ $DEBUG ] && $ECHO "checkuserenv(): log as root or change EDG_TEST_USER="$EDG_TEST_USER
    fi
 fi

 if [ -z $1 ]
   then
        [ $DEBUG ] && $ECHO "checkuserenv(): no mandatory parameter is passed" 
	return 4
 fi
 if [ -z $2 ]
   then
        [ $DEBUG ] && $ECHO "checkuserenv(): no mandatory parameter is passed"  
        return 4 
   fi

 checkuser $1 > /dev/null  2>&1
 if [ $? = 1 ] 
    then  $ECHO "checkuserenv(): user" $1 "is NOT defined"  1>&2
	  return 4

 fi

 if ! test "$EDG_TEST_USER" = "root" 
 then
	if [ $2 = "PATH" ]
	then
	    a=`$SUDO -p "you are not a permitted user for this test" -S /bin/su - $1 -c /usr/bin/env </dev/null | $GREP /usr/bin: | $SED "s/PATH=//"`
	else
	    a=`$SUDO -p "checkuserenv(): you are not a permitted user for this test" -S /bin/su - $1 -c /usr/bin/env </dev/null | $GREP $2 `  
	fi
 else
	if [ $2 = "PATH" ]
	then
	    a=`/bin/su - $1 -c /usr/bin/env </dev/null | $GREP /usr/bin: | $SED "s/PATH=//"`
	else
	    a=`/bin/su - $1 -c /usr/bin/env </dev/null | $GREP $2 `
	fi    
 fi 
 if [ "$a" ]
   then
	if [ $2 = "PATH" ]
	then
	    $ECHO "checkuserenv(): variable defined for user" $1 "PATH="$a 1>&2
	else
	    $ECHO "checkuserenv(): variable defined for user" $1 $a 1>&2
	fi
	$ECHO "$a" | $SED "s/$2=//" 
        return 0
   else 
        $ECHO "checkuserenv(): variable" "$2" "is NOT defined for user" "$1" 1>&2
        return 1 

   fi

}

