#!/bin/sh
#
######################################################################
#
# Authors: Andrea Formica   - Andrea.Formica@cea.fr
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
# Based on INFN Testgroup scripts
#
######################################################################

##################################################################
# FUNCTION: clean-list <flag> <input file>
#
# ARGUMENTS: <list>    {string} (mandatory)
#            <flag>    {string} (mandatory)
#            <input file> {string} (mandatory)
#
# Input:  <flag>              Get a list of EDG packages marked by <flag>
#                             from <input file>.  
#                             
#
#
# RETURN VALUE: 0 - list 
#               1 - file not found
#	        4 - no mandatory parameter is passed
#
##################################################################
#
clean_list()
{

    PRESTRING="${PRESTRING}+"
    local THIS_SCRIPT="clean-list.sh"
    local HOUR=`date | awk '{print $4}'`
    local PRETXT="[${THIS_SCRIPT} ${HOUR}]:"
    
    if [ $LEVEL -gt 1 ]
    then
        PRETXT="$PRESTRING [${THIS_SCRIPT} ${HOUR}]:"
    fi

    if [ -z $1 ]
    then
        [ $DEBUG ] && writeOutput "clean-list(): no mandatory parameter is passed"
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: input  $1={flag}"
        PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
	return 4
    fi
    if [ -z $2 ]
    then
        [ $DEBUG ] && writeOutput "clean-list(): no mandatory parameter is passed"
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: input  $2={output file}"
        PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
	return 4
    fi

    
    local FLAG="$1"
    local INST_INFO_FILE=$2
    local TMP_FILE=$INST_INFO_FILE.tmp

    [ $DEBUG ] && writeOutput "${PRETXT} fileexist $INST_INFO_FILE" 
    fileexist $INST_INFO_FILE
#    print_status
    [ $? != 0 ] && (PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`; return 1)

    nlines=`wc -l $INST_INFO_FILE |  awk '{print $1}'`
    fline=`cat $INST_INFO_FILE | egrep -n $FLAG | awk -F":" '{print $1}' | head -1`
    lline=`cat $INST_INFO_FILE | egrep -n $FLAG | awk -F":" '{print $1}' | tail -1`
    a=$fline
    b=$lline
    
    [ "$a" = "" ] && a=0
    [ "$b" = "" ] && b=0
    cmd="${a},${b}d"
    [ $DEBUG ] && writeOutput "${PRETXT} command $cmd\n"
    res=`sed -e $cmd $INST_INFO_FILE > $TMP_FILE`
    mv $TMP_FILE $INST_INFO_FILE

    [ $DEBUG ] && writeOutput "${PRETXT} ### END" 
    PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
    
    return 0
}
