#!/bin/sh
#
######################################################################
#
# Authors: Andrea Formica   - Andrea.Formica@cea.fr
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
# Based on INFN Testgroup scripts
#
######################################################################

##################################################################
# FUNCTION: create-list <input file>
#
# ARGUMENTS: <input file>    {string} (mandatory)
#
# Input:  <input file>              String matching the classify-rpm output files
#                                   This function will check the existing packages
#                                   and write out a list of original and EDG packages
#
#
# RETURN VALUE: 0 - list found and analysed
#		1 - list not found
#	        4 - no mandatory parameter is passed
##################################################################
#
create_list()
{

    PRESTRING="${PRESTRING}+"
    local THIS_SCRIPT="create-list.sh"
    local HOUR=`date | awk '{print $4}'`
    local PRETXT="[${THIS_SCRIPT} ${HOUR}]:"
    
    if [ $LEVEL -gt 1 ]
    then
        PRETXT="$PRESTRING [${THIS_SCRIPT} ${HOUR}]:"
    fi

    if [ -z $1 ]
    then
        [ $DEBUG ] && writeOutput "create-list(): no mandatory parameter is passed"
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: input  $1 = {input file string}"
        echo
	return 4
    fi

    local PACKAGE=$1
    local INST_INFO_FILE=./info${PACKAGE}
    local status=0

    [ $DEBUG ] && writeOutput "${PRETXT} fileexist $INST_INFO_FILE" 
    fileexist $INST_INFO_FILE
    if [ $? = 0 ]
    then
        writeOutput "${PRETXT} WARNING: Removing old $INST_INFO_FILE"
        rm $INST_INFO_FILE
    fi

    INSTALLED=$PACKAGE.installed
    TOUPGRADE=$PACKAGE.canbeupgraded
    TOINSTALL=$PACKAGE.canbeinstalled
    FAILED=$PACKAGE.failed
    NOUPGRADE=$PACKAGE.failedupgrade
    NOINSTALL=$PACKAGE.failedinstall

    [ $DEBUG ] && writeOutput "${PRETXT} fileexist $INSTALLED" 
    fileexist $INSTALLED
#    print_status
    [ $? != 0 ] && (PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`; return 1)

    echo "[$PACKAGE]" >> $INST_INFO_FILE
    ORIGRPMLIST=`cat $INSTALLED | egrep -v "^[\*\# ]|^$"`

    [ $LEVEL -gt 0 ] && writeOutput "${PRETXT} writing ORIGINAL list"
    write-list "ORIGINAL" $INST_INFO_FILE "$ORIGRPMLIST" ; [ $? != 0 ] && status=$[ status + 1 ]
    get-list "ORIGINAL" $INST_INFO_FILE ; [ $? != 0 ] && status=$[ status + 1 ]

    exclude=" "
    for excl in $RPMLIST; do
        res=`echo $excl | egrep -v none`
        [ $? = 0 ] && exclude="$exclude|$excl"
    done 
    [ "$exclude" = "" ] && exclude=" "
    EDGRPMLIST=`cat $PACKAGE | egrep -v "[\*|\# ]|^$" | egrep -v "($exclude)"`
    [ $LEVEL -gt 0 ] && writeOutput "${PRETXT} writing DISTRIBUTION list"
    write-list "DISTRIBUTION" $INST_INFO_FILE "$EDGRPMLIST" ; [ $? != 0 ] && status=$[ status + 1 ]
# Check for rpm to upgrade [canbeupgraded]
#
    UPGRADELIST=`cat $TOUPGRADE | egrep -v "^[\*\# ]|^$"`
    [ $LEVEL -gt 0 ] && writeOutput "${PRETXT} writing UPGRADE list"
    write-list "UPGRADE" $INST_INFO_FILE "$UPGRADELIST" ; [ $? != 0 ] && status=$[ status + 1 ]

    INSTALLIST=`cat $TOINSTALL | egrep -v "^[\*\# ]|^$"`
    [ $LEVEL -gt 0 ] && writeOutput "${PRETXT} writing INSTALL list"
    write-list "INSTALL" $INST_INFO_FILE "$INSTALLIST" ; [ $? != 0 ] && status=$[ status + 1 ]
    echo "[END-${PACKAGE}]" >> $INST_INFO_FILE

# Check the status
    if [ $status -ge 1 ]
    then
        PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
        return 1        
    fi

    [ $DEBUG ] && writeOutput "${PRETXT} ### END" 
    PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
    
    return 0
}
