#!/bin/sh
#
######################################################################
#
# Authors: Andrea Formica   - Andrea.Formica@cea.fr
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
# Based on INFN Testgroup scripts
#
######################################################################

##################################################################
# FUNCTION: get-list <flag> <input file>
#
# ARGUMENTS: <list>    {string} (mandatory)
#            <flag>    {string} (mandatory)
#            <input file> {string} (mandatory)
#
# Input:  <flag>              Get a list of EDG packages marked by <flag>
#                             from <input file>.  
#                             
#
#
# RETURN VALUE: 0 - list 
#               1 - file not found
#	        4 - no mandatory parameter is passed
#
##################################################################
#
get_list()
{

    PRESTRING="${PRESTRING}+"
    local THIS_SCRIPT="get-list.sh"
    local HOUR=`date | awk '{print $4}'`
    local PRETXT="[${THIS_SCRIPT} ${HOUR}]:"
    
    if [ $LEVEL -gt 1 ]
    then
        PRETXT="$PRESTRING [${THIS_SCRIPT} ${HOUR}]:"
    fi

    if [ -z $1 ]
    then
        [ $DEBUG ] && writeOutput "get-list(): no mandatory parameter is passed"
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: input  $1={flag}"
        echo
        PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
	return 4
    fi
    if [ -z $2 ]
    then
        [ $DEBUG ] && writeOutput "get-list(): no mandatory parameter is passed"
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: input  $2={output file}"
        echo
        PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
	return 4
    fi

    
    RPMLIST=" "
    local FLAG="$1"
    INST_INFO_FILE=$2

#    commonenv
    [ $DEBUG ] && writeOutput "${PRETXT} fileexist $INST_INFO_FILE" 
    fileexist $INST_INFO_FILE
    [ $? != 0 ] && (PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`; return 1)

    RPMLIST=`cat $INST_INFO_FILE | \\
             awk -v m=$FLAG 'BEGIN {p=0;f=m}; $0 ~ f {p=1}; $0 ~ /END/ {p=0} ; ( p==1 ){print $1};' | \\
             egrep -v $FLAG`

    [ $DEBUG ] && writeOutput "${PRETXT} ### END" 
    PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
    
    return 0
}
