#!/bin/bash

######################################################################
#
# Authors: Mario Reale    -  Mario.Reale@cnaf.infn.it
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
######################################################################


##################################################################
# FUNCTION: getportnumber_netstat <variable>
#
# ARGUMENTS: <variable> {string} (mandatory)
#
# DESCRIPTION: gets <service> port number from netstat
#	       and returns the value of the corresponding port number
#
# RETURN VALUE: 0 - if <service> is defined (value in output)
#		1 - if <service> is not defined
#               4 - if no mandatory parameter is passed  
####################################################################################


function getportnumber_netstat() {

 if [ -z $1 ]
   then
        [ $DEBUG ] && $ECHO "getportnumber_netstat(): no mandatory parameter is passed"
	return 4
   fi

 a=`netstat -l |  grep $1  | awk 'BEGIN { FS = " " } $0 ~ // { print $2} '  | awk 'BEGIN { FS = "/" } $0 ~ // {print $1} '`

   if [ "$a" ]
   then
        [ $DEBUG ] && $ECHO -e "getportnumber_netstat(): according to /etc/services  $1 should run on port $a"   
        $ECHO $a
        return 0
   else
		[ $DEBUG ] && $ECHO "getportnumber_netstat(): service $1 not found in /etc/services "
        $ECHO ""
        return 1
   fi


}




