#!/bin/sh
#
######################################################################
#
# Authors: Andrea Formica   - Andrea.Formica@cea.fr
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
# Based on INFN Testgroup scripts
#
######################################################################

##################################################################
# FUNCTION: getpkg-list <pkg> <dir> <list> 
#
# ARGUMENTS: <list>    {string} (mandatory)
#            <dir>    {string} (mandatory)
#            <pkg>    {string} (mandatory)
#
# Input:  <pkg>              Get the pkg <pkg> from list of EDG packages
#         <dir>              <list>, and search for it in directory <dir>  
#         <list>               
#
#
# RETURN VALUE: 0 - PKG found in DIR
#               1 - PKG not found in DIR
#	        4 - no mandatory parameter is passed
#
##################################################################
#
getpkg_list()
{

    PRESTRING="${PRESTRING}+"
    local THIS_SCRIPT="getpkg-list.sh"
    local HOUR=`date | awk '{print $4}'`
    local PRETXT="[${THIS_SCRIPT} ${HOUR}]:"
    
    if [ $LEVEL -gt 1 ]
    then
        PRETXT="$PRESTRING [${THIS_SCRIPT} ${HOUR}]:"
    fi

    if [ -z $1 ]
    then
        [ $DEBUG ] && writeOutput "getpkg-list(): no mandatory parameter is passed"
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: input  $1={pkg}"
        echo
	return 4
    fi
    if [ -z $2 ]
    then
        [ $DEBUG ] && writeOutput "getpkg-list(): no mandatory parameter is passed"
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: input  $2={dir}"
        echo
        PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
	return 4
    fi
    if [ "$3" = "" ]
    then
        [ $DEBUG ] && writeOutput "getpkg-list(): no mandatory parameter is passed"
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: input  $2={list}"
        echo
        PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
	return 4
    fi
    

    local PKG=$1
    local PDIR=$2
    local mylist="$3"

# local variables
    local nfield
    local nf
    local pkgorig
    local isold
    local mtpm

    RPMNAME=""

    nfield=`echo $PDIR | awk -F'/' '{print NF}'`    
    nf=$[nfield+1]
    mtmp=`ls -1 $PDIR/* | egrep "$PKG" | $CUT -d '/' -f $nf`
    [ $DEBUG ] && writeOutput "${PRETXT} variables {mtmp, nf} $mtmp, $nf"    
    if [ $? = 0 ]; then
        pkgorig=`echo $mtmp | awk '{print $1}'`
        [ $DEBUG ] &&  writeOutput "${PRETXT} found $pkgorig"
        if [ -z $pkgorig ]
        then
            isold=`echo $mylist | egrep "$pkgorig"`
            [ $? != 0 ] && pkgorig=""
            RPMNAME=$pkgorig
        fi
    else
        [ $DEBUG ] &&  writeOutput "${PRETXT} cannot find $PKG"
        PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
        return 1
    fi
    

    [ $DEBUG ] && writeOutput "${PRETXT} ### END" 
    PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
    
    return 0
}
