#!/bin/sh
#
######################################################################
#
# Authors: Andrea Formica   - Andrea.Formica@cea.fr
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
# Based on INFN Testgroup scripts
#
######################################################################

##################################################################
# FUNCTION: install-rpm <input file> <action>
#
# ARGUMENTS: <input file>    {string} (mandatory)
#            <action>        {string}
#
# Input:  info<input file>          String matching the create-list output files
#                                   This function will check the packages listed
#                                   and try to install them.
#
#         <action>                  {install | restore}
#
# Output: info<input file>          Output file, added information on packages to install 
#                                   or to upgrade
#
#
#
# RETURN VALUE: 0 - input file found and upgraded
#		1 - input file not found, or done list existing
#	        4 - no mandatory parameter is passed
##################################################################
#
install_rpm()
{

    PRESTRING="${PRESTRING}+"
    local THIS_SCRIPT="install_rpm.sh"
    local HOUR=`date | awk '{print $4}'`
    local PRETXT="[${THIS_SCRIPT} ${HOUR}]:"
    
    if [ $LEVEL -gt 1 ]
    then
        PRETXT="$PRESTRING [${THIS_SCRIPT} ${HOUR}]:"
    fi

    local ACTION=$2
    local status=0
#    commonenv
    
    if [ -z $1 ]
    then
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: no mandatory parameter is passed"
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: input  $1={input file string}"
        echo
        PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
	return 4
    fi
    if [ -z $2 ]
    then
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: no mandatory parameter is passed"
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: input  $2={action string = install}"
        echo
        ACTION=install
	return 4
    fi

    local PACKAGE=$1
    local INST_INFO_FILE=./info${PACKAGE}

    [ $DEBUG ] && writeOutput "${PRETXT} fileexist  $INST_INFO_FILE" 
    fileexist $INST_INFO_FILE
#    print_status
    [ $? != 0 ] && (PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`; return 1)
    
    original=""
    distribution=""
    upgrade=""
    install=""
    donelist=""
    local restored=""
    
    [ $DEBUG ] && writeOutput "${PRETXT} get_list"
    get_list "ORIGINAL" $INST_INFO_FILE
#    print_status
    [ $LEVEL -gt 2 ] && writeOutput "ORIG $DEBUG $RPMLIST"
    original=`echo $RPMLIST | egrep -v none`
    get_list "DISTRIBUTION" $INST_INFO_FILE
#    print_status
    [ $LEVEL -gt 2 ] && writeOutput "DISTRIBUTION $DEBUG $RPMLIST"
    distribution=`echo $RPMLIST | egrep -v none`
    get_list "UPGRADE" $INST_INFO_FILE
#    print_status
    [ $LEVEL -gt 2 ] && writeOutput "UPGRADE $DEBUG $RPMLIST"
    upgrade=`echo $RPMLIST | egrep -v none`
    get_list "INSTALL" $INST_INFO_FILE
#    print_status
    [ $LEVEL -gt 2 ] && writeOutput "INSTALL $DEBUG $RPMLIST"
    install=`echo $RPMLIST | egrep -v none`
    get_list "DONE" $INST_INFO_FILE
#    print_status

    donelist=`echo $RPMLIST | egrep -v none`
    restored="$donelist"
    
    if [ ${ACTION} = "install" ]; then
        [ "$donelist" != "" ] && (writeOutput "${PRETXT} DONE not empty, cannot install" ) \
        && (PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`; return 1)
    else
        [ "$donelist" = "" ] && (writeOutput "${PRETXT} DONE is empty, cannot restore" ) \
        && (PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`; return 1)
    fi
    
    
# Start with installation or upgrading
    ulist=""; ilist=""
    [ $LEVEL -gt 0 ] && writeOutput "${PRETXT} dolist $ACTION Uvh"
    dolist_$ACTION "Uvh" "$upgrade"
    print_status
    ulist=`echo $RPMLIST | egrep -v none`
    
    [ $LEVEL -gt 0 ] && writeOutput "${PRETXT} dolist $ACTION ivh"
    dolist_$ACTION "ivh" "$install"
    print_status
    ilist=`echo $RPMLIST | egrep -v none`

    donelist="$ulist $ilist"
    if [ ${ACTION} = "install" ]; then
        [ $LEVEL -gt 0 ] && writeOutput "${PRETXT} add-list DONE"
        add_list "DONE" $INST_INFO_FILE "$donelist"; [ $? != 0 ] && status=$[ status + 1 ]
#        print_status
    else
        [ $LEVEL -gt 0 ] && writeOutput "${PRETXT} restore-list INSTALL"
        restore_list "$install" "$donelist"  
        if [ $? = 0 ]
        then
            install="$RPMLIST"
            clean_list "INSTALL" $INST_INFO_FILE ; [ $? != 0 ] && status=$[ status + 1 ]
#            print_status
            add_list "INSTALL" $INST_INFO_FILE "$install" ; [ $? != 0 ] && status=$[ status + 1 ]
#            print_status
        fi
        
        [ $LEVEL -gt 0 ] && writeOutput "${PRETXT} restore-list UPGRADE"
        restore_list "$upgrade" "$donelist"  
        if [ $? = 0 ]
        then
            upgrade="$RPMLIST"
            clean_list "UPGRADE" $INST_INFO_FILE 
#            print_status
            add_list "UPGRADE" $INST_INFO_FILE "$upgrade"
#            print_status
        else
            status=$[ status + 1 ]  
        fi
        
        [ $LEVEL -gt 0 ] && writeOutput "${PRETXT} restore-list DONE"
        restore_list "$restored" "$donelist"  
        if [ $? = 0 ]
        then
            restored="$RPMLIST"
            clean_list "DONE" $INST_INFO_FILE ; [ $? != 0 ] && status=$[ status + 1 ]
#            print_status
            add_list "DONE" $INST_INFO_FILE "$restored" ; [ $? != 0 ] && status=$[ status + 1 ]
#            print_status
        else
            status=$[ status + 1 ]  
        fi
    fi
# Check the status
    if [ $status -ge 1 ]
    then
        PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
        return 1        
    fi

    [ $DEBUG ] && writeOutput "${PRETXT} ### END" 
    PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
    
    return 0    
}
