#!/bin/bash
#$Id: rpmChecker.sh,v 1.1.1.2 2005/02/08 16:09:43 mbarroso Exp $
######################################################################
#
# Authors: Mario Reale   - Mario.Reale@cnaf.infn.it
#          Flavia Donno  - Flavia.Donno@pi.infn.it
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
######################################################################

##################################################################
# FUNCTION: rpmchecker <filename>
#
# ARGUMENTS: <filename> {string} (mandatory)
#
# DESCRIPTION: It checks if the list of rpms specified in <filename>
#              are installed on the machine 
#
# RETURN VALUE: 0 - if all rpms are installed
#               1 - if some of the specified rpms is not installed
#               2 - no mandatory parameter is passed
#		3 - if the input file specified is empty
##################################################################

rpmchecker()
{
   if [ -z $1 ]
   then
        $ECHO "rpmchecker(): no mandatory parameter is passed"
        return 2 
   fi

   if [ -f $1 ]
   then
      nlines=`wc -l $1 |  awk '{print $1}'`
      errflg=0
      k=1
      until [ $k -gt $nlines ] ; do
          f=`sed -n ${k}p $1`
          rpm -q $f > /dev/null 2>&1
          if [ $? -eq 0 ]; then
             [ $DEBUG ] && $ECHO "rpmchecker(): $f is installed"
          else
             $ECHO "rpmchecker(): $f NOT INSTALLED !"
             errflg=1
          fi
          k=$[ k + 1 ]
      done
   else
      [ $DEBUG ] && $ECHO "rpmchecker(): the input file specified is empty"
      return 3 
   fi
   return $errflg
}
