#!/bin/bash

##################################################################
# FUNCTION: runCommand_GDMP <command> <host> <port> <vo> <logfile>
#
# ARGUMENTS: <command> {string} (mandatory)
#            <host>    {string} (mandatory)
#            <port>    {string} (mandatory)
#            <vo>      {string} (mandatory)
#            <logfile> {string} (optional)
#
# DESCRIPTION: runs GDMP command, waits until it ends and check
#              result 
#
# RETURN VALUE: 0 - if <command> successful
#		1 - if <command> fails 
#	        4 - no mandatory parameter is passed
##################################################################

runCommand_GDMP()
{
   if [ -z "$1" ] || [ -z "$2" ] || [ -z "$3" ] || [ -z "$4" ]
   then
        message 1 "runCommand_GDMP(): no mandatory parameter is passed"
	return 4
   fi    

   if [ -z "$5" ]
   then
       local TMPLOGFILE=`echo "$1" | awk '{print $1}'`.log
   else
       local TMPLOGFILE=$5
   fi

   local GDMPMAXRETRY=20
   local GDMPSLEEP=5

   local LID

   runCommand "$1" $TMPLOGFILE
   if [ $? -ne 0 ]
   then 
	return 1
   fi

   getLogID_GDMP $TMPLOGFILE LID

   if [ $? -ne 0 ]
   then
        return 1
   fi

   local key=1
   local count=$GDMPMAXRETRY
   local TMPLOGFILEREMOTE=$TMPLOGFILE.remote
	
   while [ $key -ne 0 ] && [ $count -ne 0 ]
   do
	sleep $GDMPSLEEP
	getLogEnd_GDMP $LID $2 $3 $4 $TMPLOGFILEREMOTE
	key=$?
	count=`expr $count - 1`
   done

   if [ -z "$5" ]
   then
       rm $TMPLOGFILE
   else
       cat $TMPLOGFILEREMOTE >> $TMPLOGFILE
       if [ $key -ne 0 ]
       then
	    echo "Timeout" >> $TMPLOGFILE
       fi
   fi
   rm $TMPLOGFILEREMOTE
   if [ $key -eq 0 ]
   then
        message 2 "runCommand_GDMP(): command $1 ran succesfully"
	return 0
   else
        message 1 "runCommand_GDMP(): command $1 timeout"
	return 1
   fi
}
