#! /usr/bin/env bash
###############################################################
###############################################################
##  RUN TEST
##############################################################
###############################################################
## 
## Author: TSTG - Andrey Kiryanov - Frederique Chollet
## Date: September 2003
## Last modification :  1/12/2003 by F.Chollet
## Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
##     http://www.edg.org/license.html or see LICENSE file
##
###############################################################
###############################################################
##
## ARGUMENTS: name of a directory where are some tests
##
## DESCRIPTION: run tests located in a sub directory passed in argument.
##              
## EXIT CODE: 0 - test runs and html output is produced
##            3 - wrong configuration
##            4 - wrong argument passed
##            
###############################################################
###############################################################

usage()
{
 $ECHO "Usage:      $prog_name"
 $ECHO "            $prog_name    [-t|--test <path>]"
 $ECHO "            $prog_name    [-h|--help]"
 $ECHO ""
 $ECHO "Options:"
 $ECHO "            -h|--help"
 $ECHO "            -a|--list    List names of all available tests"
 $ECHO "            -t|--test    <Path to the test definition file> "
 $ECHO "            -x|--xml      Produce an xml like log file and generate an HTML page from it."
 $ECHO "            -s|--step     Run only the specified step, not the entire test."
 $ECHO "            -n|--numstep  Prints the number of selectable steps on the test."
 $ECHO "            -i|--info     Prints brief info about the selected step using ""-s""."
 $ECHO "                          Prints detailed info about the entire test if no"
 $ECHO "                          step selected."
 $ECHO ""
}
list() {
 find $EDG_TESTROOT/tests -name "*.def" 2>/dev/null|sed -e "s/^.*\/\(\w\+\)\.def$/\1/" 2>/dev/null
}
#################################################################
DIRNAME=`dirname $0`
export XML=""
export SEL_STEP=""
export EDG_TESTROOT="`dirname $DIRNAME`/tests"
export EDG_FLAVOR="edg2"

export PATH="$DIRNAME:$PATH"
export PERLLIB="$DIRNAME:$DIRNAME/../lib/perl:$PERLLIB"

################ Function initialization ######################
. $EDG_TESTROOT/sbin/sourcer.sh || exit 3
################ Variable initialization ######################
. $EDG_TESTROOT/etc/edg-test.conf || exit 3
################# Parse the command line ######################

if ! test -e $EDG_TESTROOT/etc/test_site-LocalTB.conf 
then
    $ECHO "A Local TestBed configuration file :"
    $ECHO $EDG_TESTROOT"/etc/test_site-LocalTB.conf"
    $ECHO "must be created from the available template file :"
    ls $EDG_TESTROOT/etc/test_site-LocalTB.template 
    exit 3
fi

ARGS_SHORT="haxnit:s:"
ARGS_LONG="help,list,xml,numstep,info,test:,step:"

PROG_NAME=`$BASENAME $0`
getopt_out=`$GETOPT -o $ARGS_SHORT --long $ARGS_LONG -n $PROG_NAME -- "$@"`

if ! test "$?" = 0
then
   $ECHO "Try \`$prog_name -h' for more informations."
   exit 3
fi      

eval set -- "$getopt_out"
while true;
do
  case "$1" in
    -h|--help)     usage           ; exit 3 ;;   
    -a|--list)     list            ; exit   ;;
    -x|--xml)      XML=1                    ;;
    -n|--numstep)  NB_STEP=1                ;;
    -i|--info)     INFO=1                   ;;
    -t|--test)     TEST_DEFFILE=$2 ; shift  ;;
    -s|--step)     SEL_STEP=$2     ; shift  ;;  
    --)            shift           ; break  ;;
    *)             $ECHO "Internal error!"  > /dev/stderr ; exit 4 ;; 
  esac
  shift
done

if test -z "$TEST_DEFFILE"
then
    usage
    exit 3
fi

if ! test -e "$TEST_DEFFILE"
then
    SUB_TEST=$($ECHO $TEST_DEFFILE | $CUT -d. -f1)
    TEST_DEFFILE=`find $EDG_TESTROOT/tests -name "*$TEST_DEFFILE*.def"`
    $ECHO $TEST_DEFFILE" selected..."
    ! test -e "$TEST_DEFFILE"  && $ECHO "ERROR: No such test defined." && exit 4
fi
  
TIME_STAMP=`$DATE '+%y%m%d-%H%M%S'`

### Show the total number of steps of the selected test
if test -n "$NB_STEP"
then   
    def2log-script.sh -d $TEST_DEFFILE -n 
    exit 0
fi

### Show infos about steps of the selected test
if test -n "$INFO"
then  
   if test -z "$SEL_STEP" 
   then  
       def2log-script.sh  -d  $TEST_DEFFILE -i 
   else
       def2log-script.sh  -d  $TEST_DEFFILE -i -s $SEL_STEP
   fi
   exit 0
fi    

$ECHO "$(echo $SUB_TEST | tr '_' ' ') running.........."

### launching test ###
if test "$XML" = "1"
then
    mkdir -p $EDG_TEST_HTML_OUTPUT/${TIME_STAMP}
    export EDG_TEST_HTML_DIR=$EDG_TEST_HTML_OUTPUT/${TIME_STAMP}
    export HTML_DIR=$EDG_TEST_HTML_OUTPUT/${TIME_STAMP}
    export MAINSCRIPT="$0"; export EDG_BASEUTILS=$PERLLIB
    if test -z "$SEL_STEP" 
    then  
	def2log-script.sh -x -d $TEST_DEFFILE -l $EDG_TEST_HTML_DIR/$SUB_TEST.xml
    else
	def2log-script.sh -x -d $TEST_DEFFILE -l $EDG_TEST_HTML_DIR/$SUB_TEST.xml -s $SEL_STEP
    fi
else
    if test -z "$SEL_STEP" 
    then  
	def2log-script.sh  -d  $TEST_DEFFILE -l /tmp/$SUB_TEST.log
    else
	def2log-script.sh  -d  $TEST_DEFFILE -l /tmp/$SUB_TEST.log -s $SEL_STEP
    fi
fi

if test "$?" = "0"
then
      $ECHO "$(echo $SUB_TEST | tr '_' ' ') has run successfully ! "      
else
      $ECHO "$(echo $SUB_TEST | tr '_' ' ') over. Overall status is failed."
fi

if test "$XML" = "1" 
then
    $ECHO -n "Formatting output to html.........."

#    log2html-script.sh -f $EDG_TEST_HTML_DIR/$SUB_TEST.html -s $EDG_TEST_HTML_STYLE $EDG_TEST_HTML_DIR/$SUB_TEST.xml
    log2html-script.pl -l $EDG_TEST_HTML_DIR/$SUB_TEST.log.html -f $EDG_TEST_HTML_DIR/$SUB_TEST.html $EDG_TEST_HTML_DIR/$SUB_TEST.xml

    if test "$?" = "0"
    then
      $ECHO "done"   
      ### cleaning ###
      $RM $EDG_TEST_HTML_DIR/$SUB_TEST.xml
      $LN -s $EDG_TEST_HTML_DIR/$SUB_TEST.html $EDG_TEST_HTML_DIR/index.html
      $ECHO "Html file saved: "$EDG_TEST_HTML_DIR/$SUB_TEST.html  
      exit 0
    else
      $ECHO "failed"
      ### cleaning ###
      $ECHO "Xml log file saved: "$EDG_TEST_HTML_DIR/$SUB_TEST.xml  
#      $RM $EDG_TEST_HTML_DIR/$SUB_TEST.xml
      exit 1
    fi
else
    $ECHO "Log file saved: /tmp/"$SUB_TEST.log
fi
exit 0
