#!/bin/sh

# Define some variables
HOST=`hostname`
WDIR=`pwd`
echo "insert Host $HOST"                                      # Host

# Source the card file
. ./*.cards

# Name arguments
RUN=$1
DATA="${DATA_NAME}${RUN}.dat"

# Run the fake event generator and calculate the wall clock time
chmod +x Test.exe
starttime=`date +%s`
./Test.exe $EVENTS $DATA_SIZE
mv dummy_data $DATA
endtime=`date +%s`
difftime=`expr $endtime - $starttime`
echo "insert WallClockTime $difftime"                         # WallClockTime

# Copy locally the input data, if required
if [ -n "$INPUT_DATA" -a "$INPUT_DATA" != 'NO' ] ; then
    INPUT="${INPUT_DATA}${RUN}.dat"
    edg-rm --vo $VO cp lfn:$INPUT file:$WDIR/input.dat
    ERR=$?
    echo "copyFile exit status: $ERR"
    if [ $ERR -ne 0 ] ; then
	echo "ERROR: NO_REPLICA_FOR_INPUT_DATA" >&2
	exit 1
    else
	echo "Input copy succeeded"
    fi
fi

# Store data in a SE
if [ "$COPY2SE" = YES ] ; then

# Check that the RM commands are there
    which edg-rm > /dev/null 2> /dev/null
    if [ ! $? ] ; then
	echo "ERROR: RM_MISSING" >&2
	exit 1
    fi

# Use the Replica manager
    DEST=
    if [ $OUTPUT_SE != "CLOSE" ] ; then
	DEST="-d $OUTPUT_SE"
    fi
    edg-rm --vo=$VO -v cr file:${WDIR}/${DATA} -l lfn:${LFN_DIR}/${DATA} $DEST
    ERR=$?
    echo "copyAndRegisterFile exit status: $ERR"
    if [ $ERR -ne 0 ] ; then
	echo "ERROR: COPY_AND_REG_FAILED" >&2
	echo "insert LFN !COPY_AND_REG_FAILED"
	exit 1
    else
	echo "insert LFN lfn:${LFN_DIR}/${DATA}"
    fi

# Replicate the file to another SE
    if [ "$REPLICATION" = 'YES' -a -n "$REP_SE" ] ; then
	edg-rm --vo=$VO rep lfn:${LFN_DIR}/${DATA} -d ${REP_SE}
	ERR=$?
	echo "replicateFile exit status: $ERR"
	if [ $ERR -ne 0 ] ; then
	    echo "ERROR: REPLICATE_FAILED"
	else
	    echo "insert RepSE $REP_SE"
	fi
    fi
fi

