#!/bin/bash
# 
# 31/01/03
# Author:       Frederique Chollet chollet@lapp.in2p3.fr
# 
########################################################################
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
#######################################################################
# Verify CE Globus Configuration - Compliant with EDG v1.4.x Installation Guide
# Document identifier : DataGrid-01-TED-0105-1-25
# 
# RETURN VALUE: 0 PASSED
#	 	1 FAIL
#		2 WARNING
#		3 XFAIL
#		4 UNRESOLVED
#		5 UNTESTED
#		6 UNSUPPORTED
#               7 ENVIRONMENT / OPTIONS FAIL
#	
########################################
# 
#
# INPUT none
#	
#	
########################################
#

function verifyCEGlobusConf()
{
	$ECHO -n "GLOBUS_LOCATION in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
	globus_location=`getglobusconfparam GLOBUS_LOCATION` && test "$globus_location" = "$GLOBUS_LOCATION"
   	trace_status
	$ECHO -n "GLOBUS_HOST_DN defined in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
	globus_host_dn=`getglobusconfparam GLOBUS_HOST_DN`
 	trace_status 
	$ECHO -n "GLOBUS_ORG_DN defined "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
        globus_org_dn=`getglobusconfparam GLOBUS_ORG_DN` && globus_org_dn="\""$globus_org_dn"\"" && test "$globus_org_dn" = "$SITE_DN"
	trace_status
#
	$ECHO -n "GLOBUS_GATEKEEPER_SUBJECT in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
        globus_gatekeeper_subject=`getglobusconfparam GLOBUS_GATEKEEPER_SUBJECT`
 	trace_status 
	$ECHO -n "GLOBUS_GATEKEEPER_HOST in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
        globus_gatekeeper_host=`getglobusconfparam GLOBUS_GATEKEEPER_HOST` && test "$globus_gatekeeper_host" = `hostname`
	trace_status
	$ECHO -n "GLOBUS_GATEKEEPER_PORT in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
	gate_port=`getglobusconfparam GATEKEEPER_PORT` && test $gate_port = 2119
	trace_status
	$ECHO -n "GLOBUS_GATEKEEPER_LOG in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
	gate_log=`getglobusconfparam GATEKEEPER_LOG` && fileexist $gate_log
	trace_status
#
	$ECHO -n "X509_CERT_DIR in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
	cert_dir=`getglobusconfparam X509_CERT_DIR`
	trace_status
   	$ECHO -n "X509_GATEKEEPER_CERT in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
	x509_gatekeeper_cert=`getglobusconfparam X509_GATEKEEPER_CERT` && test "$x509_gatekeeper_cert" = "$SITE_DEF_HOST_CERT"    
 	trace_status
	$ECHO -n "X509_GATEKEEPER_KEY in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
	x509_gatekeeper_key=`getglobusconfparam X509_GATEKEEPER_KEY`  && test "$x509_gatekeeper_key" = "$SITE_DEF_HOST_KEY"    
 	trace_status 
#
	$ECHO -n "GRIDMAP in "$GLOBUS_CONFIG >> $TMP_LOG_FILE 2>&1
	gridmap=`getglobusconfparam GRIDMAP` && test "$gridmap" = "$SITE_DEF_GRIDMAP" && fileexist $gridmap
	trace_status
#
	$ECHO -n "GLOBUS_JOBMANAGERS in "$GLOBUS_CONFIG >> $TMP_LOG_FILE 2>&1
	globus_jobmanagers=`getglobusconfparam GLOBUS_JOBMANAGERS`
	trace_status
#
	$ECHO -n "GSIWUFTPPORT in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
	gsiwuftpport=`getglobusconfparam GSIWUFTPPORT` && test $gsiwuftpport = 2811
 	trace_status
	$ECHO -n "GSIWUFTPDLOG in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
	gsiwuftpdlog=`getglobusconfparam GSIWUFTPDLOG` 
 	trace_status
#
	$ECHO -n "GLOBUS_FLAVOR_NAME in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
	globus_flavor_name=`getglobusconfparam GLOBUS_FLAVOR_NAME`
	trace_status 
#
	$ECHO -n "GRID_INFO_EDG in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
	grid_info_edg=`getglobusconfparam GRID_INFO_EDG` && test "$grid_info_edg" = "yes"
 	trace_status 
	$ECHO -n "GRIDMAPDIR in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
 	gridmapdir=`getglobusconfparam GRIDMAPDIR`   
 	trace_status
	$ECHO -n "GRID_INFO_GRIS_REG_GIIS in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
	grid_info_gris_reg_giss=`getglobusconfparam GRID_INFO_GRIS_REG_GIIS`   
 	trace_status
	$ECHO -n "GRID_INFO_GRIS_REG_HOST in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
	grid_info_gris_reg_host=`getglobusconfparam GRID_INFO_GRIS_REG_HOST`  &&  test "$grid_info_gris_reg_host" = `hostname`
 	trace_status
#
	$ECHO -n "GLOBUS_GATEKEEPER in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
        globus_gatekeeper=`getglobusconfparam GLOBUS_GATEKEEPER` && fileexist $globus_gatekeeper
	trace_status
	$ECHO -n "GLOBUS_GATEKEEPER_OPTIONS in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
	globus_gatekeeper_options=`getglobusconfparam GLOBUS_GATEKEEPER_OPTIONS`
	trace_status
	$ECHO -n "GLOBUS_GSIWUFTPD_UMASK in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
	globus_gsiwuftpd_umask=`getglobusconfparam GLOBUS_GSIWUFTPD_UMASK` && test "$globus_gsiwuftpd_umask" = "002"
	trace_status
#
	$ECHO -n "GRID_INFO_GRIS in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
	grid_info_gris=`getglobusconfparam GRID_INFO_GRIS` && test "$grid_info_gris" = "yes"
        trace_status
	$ECHO -n "GRID_INFO_USER in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
	grid_info_user=`getglobusconfparam GRID_INFO_USER` && checkuser $grid_info_user
	trace_status
#
	$ECHO -n "X509_GSIWUFTPD_CERT in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
	x509_gsiwuftpd_cert=`getglobusconfparam X509_GSIWUFTPD_CERT`  && test "$x509_gsiwuftpd_cert" = "$SITE_DEF_HOST_CERT"    
 	trace_status
	$ECHO -n "X509_GSIWUFTPD_KEY in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
	x509_gsiwuftpd_key=`getglobusconfparam X509_GSIWUFTPD_KEY`  && test "$x509_gsiwuftpd_key" = "$SITE_DEF_HOST_KEY"    
 	trace_status 
#
	$ECHO -n "GLOBUS_GRAM_JOB_MANAGER_QSUB in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
        gglm_qsub=`getglobusconfparam GLOBUS_GRAM_JOB_MANAGER_QSUB`
 	trace_status
	$ECHO -n "GLOBUS_GRAM_JOB_MANAGER_QDEL in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
        gglm_qdel=`getglobusconfparam GLOBUS_GRAM_JOB_MANAGER_QDEL`
 	trace_status 
	$ECHO -n "GLOBUS_GRAM_JOB_MANAGER_QSTAT in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
        gglm_qstat=`getglobusconfparam GLOBUS_GRAM_JOB_MANAGER_QSTAT`
 	trace_status 
	$ECHO -n "GLOBUS_GRAM_JOB_MANAGER_MPIRUN in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
        gglm_mpirun=`getglobusconfparam GLOBUS_GRAM_JOB_MANAGER_MPIRUN`
 	trace_status 
	$ECHO -n "GLOBUS_GRAM_JOB_MANAGER_QSELECT in "$GLOBUS_CONFIG >>$TMP_LOG_FILE 2>&1
        gglm_qselect=`getglobusconfparam GLOBUS_GRAM_JOB_MANAGER_QSELECT`
 	trace_status 
#
	return 

}
