#!/usr/bin/perl -w

package BaseTest::EDGLifecycle::Stack::MultiDStorm;

use FindBin qw($Bin);
use lib $Bin;

use File::Basename;
use EDGUtils;

use strict;
use vars qw(@ISA);
@ISA=qw(BaseTest::EDGLifecycle::Stack);

=head1 NAME

MultiDStorm.pm - Check that globusrun authentication works on the hosts
# $Id: MultiDStorm.pm,v 1.1.1.2 2005/02/08 16:09:43 mbarroso Exp $

=head1 SYNOPSIS

Check that globusrun authentication works on the hosts

=head1 DESCRIPTION 

B<MultiDStorm.pm> checks that the Globusrun authentication works
for all hosts of the list for the testing user. 
 This test requires that a valid GSI proxy is available for the latter.

=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
    my $self = shift;

    $self->SUPER::setContext();

#    print "MultiDStorm: setContext\n";
    $self->printLocation( (caller(0))[3] );

    $self->addOption('maxSubs=i',
                     'Number of jobs.',
                     '6');
    $self->addOption('maxStack=i',
                     'Number of simultaneous jobs in the stack.',
                     '2');
    $self->addOption('reqLapse=i',
                     'Delay between 2 submissions (in sec).',
                     '15');
    $self->addOption('serie=i',
                     'Serial number of the storm.',
                     '135');
#     $self->addOption ("cksumSize=i", 
# 		      "Size of the file for checksum test (Mb)", 
# 		      2000);
#     $self->addOption('circular',
#                      'To scan circularly the CEList built with useCEList');

  # Add the filesize option
    $self->addOption('filesize=i',
		     'Size in kB of the file used for replication. (Default: 256)',
		     1);  ##  256);
    $self->addOption('ofile=i',
		     'Number of files to return via the Data Management S/W',
		     10);  ##  10);


    # Additionnal Commands necessary to run this test.
    my @commands = (
#			  "edg-vo-env",
#			  "edg-gridftp-rm",
#			  "edg-gridftp-ls",
#			  "edg-gridftp-mkdir",
#			  "gdmp_host_subscribe",
#			  "gdmp_job_status",
#			  "globus-replica-catalog",
#			  "ldapsearch",
#			  "edg-replica-manager-replicateFile",
#			  "edg-replica-manager-copyAndRegisterFile",
#			  "edg-replica-manager-deleteFile",
#			  "edg-replica-manager-listReplicas"
		   );
    push @{$self->{commands}}, @commands;

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::Stack::MultiDStorm [options]

END
    ;

    $self->{DescText} = <<'END'

CHANGE THIS TEXT ...
This submits a simple job which simply echos the [OK] string.  It does
this from multiple streams of multiple jobs, i.e. a job "storm".

END
    ;

	$self->{type} = "MultiDStorm";
	$self->{brfinfo} = "Multiple DM files";
#	 $self->{subDir} = "";
#	 $self->{location} = "";
#	 $self->{VO} = "";
#	 $self->{dataFL} = [];      ## List
#	 $self->{SPs} = {};         ## Hash
	 $self->{SEs} = [];         ## List
#	 $self->{SEpairs} = {};     ## Hash
	$self->{CEs} = [];         ## List
#    $self->{bigFile} = "";

    return 0;

}

# Build any necessary setup for the storm.
sub setupEnvironment {
    my $self = shift;

#    print "MultiDStorm: setupEnvironment\n";
    $self->printLocation( (caller(0))[3] );

## /opt/edg/etc/edg-replica-manager/edg-replica-manager.conf:defaultCE=adc0015.cern.ch

    my @m;
    if ( -e "/opt/edg/var/etc/edg-replica-manager/edg-replica-manager.conf" ) {
	chomp(my $res = `grep defaultSE /opt/edg/var/etc/edg-replica-manager/edg-replica-manager.conf`);
	@m = split /=/, $res;
    } elsif ( -e "/opt/edg/etc/edg-replica-manager/edg-replica-manager.conf.lcfg" ) {
	chomp(my $res = `grep 'DEFAULT.SE' /opt/edg/etc/edg-replica-manager/edg-replica-manager.conf.lcfg`);
	@m = split /\|/, $res;
    }

## Look up for overall SE declared in the local BDII (assumed to be the first one)...
    my ($bdii, @rest) = split/\s+/, $ENV{IIhostname__};
#    my ($bdii, @rest) = $ENV{IIhostname__};
    my @SEs = `ldapsearch -x -H ldap://$bdii:2170 -b mds-vo-name=local,o=grid | grep 'GlueSEUniqueID:' `;
    my $nSE = scalar(@SEs);
    my $rnd = int(rand($nSE));
    my @s = split/\s+/, $SEs[$rnd];
    print " bdiiSE: $bdii $s[1] \n";

    if ( "@SEs" =~ /$m[1]/ ) {
	$self->{targetSE} = "$m[1]";
    } else {
	$self->{targetSE} = "$s[1]"; ## || "AHAHAH";
    }

    print " TargetSE: =$self->{targetSE}= \n";
    if ( ! $self->{targetSE} ) {  ##  || "=$self->{targetSE}=" eq "==" ) {
	print STDERR " TargetSE: =$self->{targetSE}= \n";
	$self->setErrorMessage('setupEnvironment failed while setting the targetSE which returned empty');
	return 1;
    }	

    return 0;
}

sub dgjobUSELESS {
    my $cmdn = shift;
    my $root = "dg-job-";
    if ( defined $ENV{EDG_FLAVOR} && $ENV{EDG_FLAVOR} ne 'edg1' ) {
	$root = "e".$root;
    }
    return "$root"."$cmdn";
}

# Add any necessary arguments to the existing local JDL file before submission.
sub addArguments {
    my ($self, $jdlTemp, $index) = @_;

#    print "MultiDStorm: addArguments\n";
    $self->printLocation( (caller(0))[3] );

## Create the checksum file ...
#     $self->{reqs}->{$index}->{bigo} = $self->{tempdir}."/BigOrig-$index.crap";
#     my $checksum1 = $self->createBig($self->{reqs}->{$index}->{bigo});
#     $checksum1 =~ s/ /-/;
#     $checksum1 = "=$checksum1=";
    my $findex = sprintf("%04d", $index);

    my $bid = `echo 'Arguments = "S$ENV{STACK_SERIE} I${findex} $self->{targetSE} $self->{opts}->{ofile}"; ' >> $jdlTemp`;
##    my $bid = `echo 'Arguments = "S$ENV{STACK_SERIE} I${findex} $self->{opts}->{ofile}"; ' >> $jdlTemp`;
#     $bid = `echo '#BigFileName = "mumbojombo"; ' >> $jdlTemp`;
#     $bid = `echo 'InputSandbox = {"$self->{shFile}","$self->{reqs}->{$index}->{bigo}"}; ' >> $jdlTemp`;

if (0) {
    my $nCE = scalar(@{$self->{CEs}});
    my $rnd = int(rand($nCE));
    my $ce;
    if ( $self->{opts}->{circular} ) {
	$ce = ${$self->{CEs}}[($index%$nCE)];
    } else {
	$ce = ${$self->{CEs}}[$rnd];
    }
##    my $ce = "";

    if ( $self->{opts}->{useCEList} ) {
#glue    my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
#norm    my $reqs = 'Requirements = other.CEId=="'.$ce.'"';
	my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
	print STDERR " Requirements: $reqs \n";
	$bid = `echo '$reqs; ' >> $jdlTemp`;
	my @m = split /:/, $ce;
	$self->{reqs}->{$index}->{mace} = "CE:$m[0]";
    } else {
	$self->{reqs}->{$index}->{mace} = "free";
    }
} else {
    $self->chooseCE($index, $jdlTemp);
}
    
    return 0;
}

# Create a BigFile for checksuming ...
sub createBigOLD {
    my $self = shift;
    my $bigOrig = shift;

    my $size = $self->{opts}->{cksumSize};  ## in Mb
##    $size = 2;  ## temp
##    my $time = $size*70000;
    open BGF, "> $bigOrig";
    my $bgf = "";
    for ( my $t=0; $t<$size; $t++ ) {
	my $r = rand($t);
	print BGF "$r";
	$bgf .= "$r";
    }
    print BGF "\n";
    $bgf .= "\n";
    my $cumul = 70000;
    for ( my $t=0; $t<$cumul; $t++ ) {
	print BGF "$bgf\n";
    }
    close BGF;
    #system("ls -l $bigOrig; cksum $bigOrig");
    my $checksum = `cksum $bigOrig | cut -d" " -f1-2`;
    chomp($checksum);

    return $checksum;

}

# Prepare everything for running the test itself ...
sub setupTest {
    my $self = shift;

#    print "MultiDStorm: setupTest\n";
    $self->printLocation( (caller(0))[3] );

    @{$self->{SEs}} = @{$self->{args}};

#    print " Defaults: @{$self->{optionDefaults}} \n";
#    print " Linkage:  @{$self->{optionLinkage}}  \n";
#    exit 0 if (1);

    my $rc = $self->SUPER::setupTest();
    return $rc if $rc;

    ## Possible only when jdlFile and shFile get created
    $rc = $self->buildCEList();
    return $rc if $rc;

#     if ( $self->{opts}->{useCEList} ) {
# 	if ( ! scalar(@{$self->{CEs}}) ) {
# 	    $self->setErrorMessage('Setup failed while building the CEList which returned empty');
# 	    return 1;
# 	}	
#     }

    $self->{zombSlep} = ( $self->{zombSlep} > $self->{reqLapse} ) ? $self->{reqLapse} : $self->{zombSlep};
    print "\nOPTIONS: \n", map { "$_ => $self->{opts}->{$_}\n" } keys %{$self->{opts}};
    print "\n";

    return $rc;
}


sub createJDL {
    my ($self, $file, $script) = @_;

#    print "MultiDStorm: createJDL\n";
    $self->printLocation( (caller(0))[3] );
    print " file script: $file $script \n";

    my ($sname, $spath, $suffix) = fileparse($script, []);

#    my $file = "$dir/JobDescription.jdl";

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
Executable = "$sname";
InputSandbox = {"$script"};
StdOutput = "result.out";
StdError = "result.err";
OutputSandbox = {"result.out","result.err", "BrokerInfo"};
VirtualOrganisation = "$self->{opts}->{vo}";
#    Requirements = other.GlueCEPolicyMaxCPUTime<600000;
#    Rank = -other.GlueCEPolicyMaxCPUTime;
END
    ;

## Remark: use the Requirements and Rank clause ABOVE to select the queues
## Like it is now: one selects the shortest one (with - sign in Rank expression)
## When - sign REMOVED in Rank: one selects the longest queue

    print JDL "RetryCount = 0;\n";   ##  Special ?

    if (0) {
    my $sandout = "OutputSandbox = {\"result.out\",\"result.err\", \"BrokerInfo\"";
    foreach my $oFile ( 0 .. $self->{opts}->{ofile} ) {
	$oFile = sprintf("oFile%04d", $oFile);
	push @{$self->{lofile}}, $oFile;
	$sandout .= ",\"$oFile\"";
    }
    $sandout .= "};\n";
    print JDL $sandout;
}

    close JDL;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


sub createScript {
#   my $dir = shift;
#   my $file = "$dir/JobScript.sh";
    my ($self, $file) = @_;

#    print "MultiDStorm: createScript\n";
    $self->printLocation( (caller(0))[3] );

    # Write out the job description.
    open SCRIPT, ">$file";
    print SCRIPT <<'END'
#!/usr/bin/perl -w

my ($a, $b, $c, $d) = @ARGV;
print STDERR "MultiDStorm: runTest ARGS: $a $b $c $d\n";
print STDOUT "MultiDStorm: runTest ARGS: $a JobIndex: $b returnSE: $c nFiles: $d\n";
my $returnSE = $c;
my $nFiles = $d;

my $res = `cp .BrokerInfo BrokerInfo 2>&1`;
my (@CSE) = `edg-brokerinfo getCloseSEs 2>&1`;
    chomp(my $theCE = `edg-brokerinfo getCE 2>&1 | cut -d":" -f 1`);
    my @t = split /\./, $theCE;
    $theCE = $t[0];
    chomp(my $theWN = `hostname --short 2>&1`);
chomp(my $closeSE = $CSE[0]);
chomp(my $myVO = `edg-brokerinfo getVirtualOrganization 2>&1`);
    my $config = "";  ## Still empty
    my $edgRMopts = "$config --insecure --vo=$myVO";
    print STDOUT " MultiDStorm: edgRMopts: $edgRMopts WN: $theWN CE: $theCE closeSE: $closeSE returnSE: $c nfiles: $d \n";


    foreach my $oFile ( 0 .. $nFiles ) {
	$oFile = sprintf("oF%02d", $oFile);
	push @{$self->{lofile}}, $oFile;
#dhs my ($cres, $file) = createNewData($edgRMopts, $closeSE, $returnSE, $a, $b, "");
##my ($cres, $file) = createNewData($edgRMopts, $closeSE, $closeSE, $a, $b, "CE$theCE-WN$theWN");
my ($cres, $file) = createNewData($edgRMopts, $closeSE, $returnSE, $a, $b, "CE$theCE-WN$theWN-$oFile");
print " JobLFN: $file \n" unless $cres;
print " JobLFN: Failed $file $closeSE $returnSE \n" if $cres;
        $res = $cres if $cres;
    }

my $service = `echo; date; pwd; host $ENV{HOSTNAME}; host $ENV{HOST}; echo EDG_WL_JOBID: $ENV{EDG_WL_JOBID}; uname -a; echo; ls -l`;
print STDOUT $service;

exit $res;

sub createNewData {
    my ($edgRMopts, $SE1, $SE2, $serie, $index, $label) = @_;

#    $self->printLocation( (caller(0))[3] );

    ## Create another temporary data file ...
    chomp (my $date = `date '+%y%m%d-%H%M%S'`);
    my $file = "$ENV{USER}-mdstorm-$serie-$index-$label-$$-$date.mds";
##    my $file = "$label";

    # Retrieve the temporary directory.
#    my $tmpdir = $self->{tempdir};
    my $tmpdir = "/tmp";

##    my $rc = `dd if=/dev/urandom of=$file bs=1024 count=$self->{opts}->{filesize}`;
    my $rc = `dd if=/dev/urandom of=$tmpdir/$file bs=1024 count=1`;
    return $rc if $rc;

    ## Some Pre-Cleaning
#     my $rc = removeGhostFile($edgRMopts, $file);
#     return $rc if $rc;

#     open TMP, ">$tmpdir/$file";
#     print TMP "Good Morning to all Everybody ! This is a MultiDStorm. Job# $index \n";
#     close TMP;

    print " JobLFN0: lfn:$file \n";

#     my @kSEpairs = keys %{$self->{SEpairs}};
#     my ($SE1, $SE2) = split /\s+/, $kSEpairs[int(rand(scalar(@kSEpairs)))];

#     my $SP1 = "$self->{SPs}->{$SE1}/$subDir";
#     my $SP2 = "$self->{SPs}->{$SE2}/$subDir";

    $rc = createReplicas ($edgRMopts, $SE1, $SE2, $serie, $subDir, $file);
    return $rc if $rc;

    return (0, "$file");
}

## Dynamic creation of Replicas job by job ...
sub createReplicas {

#     my ($self, $SE1, $SE2, $subDir, $file) = @_;
#     my $edgRMopts = $self->{edgRMopts};
    my ($edgRMopts, $SE1, $SE2, $serie, $subDir, $file) = @_;

    # Retrieve the temporary directory.
#    my $tmpdir = $self->{tempdir};
    my $tmpdir = "/tmp";

#     my $command = "edg-replica-manager-copyAndRegisterFile -e -l ${subDir}/${file} -s $self->{hostname}//tmp/${file} -d $SE1/$SP1/$file";
#     print " CMD: $command\n";
#     my $resRcCARF = `$command 2>&1`;
#     my $stat = $?;
#     print "\n resRcCARF1: $file -$stat- \n$resRcCARF";
#     if ( $stat != 0 ) {
# 	print STDERR " File: ${subDir}/${file} on: $SE1 not correctly registered or created \n";
# 	return 1;
#     }

my $command;
my $gguid;
if ( 0 ) {
##EDG-RM case
    $command = "edg-replica-manager --verbose $edgRMopts copyAndRegisterFile file:$tmpdir/${file} -l lfn:${file} -d $SE1 ";
} else {
##LCG-CR case
    chomp($gguid = `uuidgen 2>&1`);
    print STDOUT " UUIDGEN: $gguid \n";
###    $command = "lcg-cr --verbose $edgRMopts -l lfn:${file} -d $SE1 -g $gguid file:$tmpdir/${file}";
    $command = "lcg-cr --verbose $edgRMopts -l lfn:${file} -d $SE1 -g $gguid -P tests/$serie/$gguid file:$tmpdir/${file}";
    $command = "lcg-cr --verbose $edgRMopts -l lfn:${file} -d $SE1 -g $gguid -P tests/$file file:$tmpdir/${file}";
}

    print "\n CMD: $command\n";
    my $resRcCARF = `$command 2>&1`;
    my $stat = $?;
    print "\n resRcCARF1: $file -$stat- \n$resRcCARF";
    if ( $stat != 0 ) {
	print STDERR " File: ${file} on: $SE1 not correctly registered or created \n";
	return 1;
    } else {
	chomp($resRcCARF);
	if ( $resRcCARF !~ "guid:" ) {
	    print STDERR " File: ${file} on: $SE1 : Wrong GUID returned: $resRcCARF \n";
	    return 1;
	}
#	$self->{reqs}->{$stepI}->{olout} = $resRcCARF;
    }

    my %PFNs = ();
    $PFNs{"$SE1/$file"} = "created";

#    my @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
    $command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
    if ( 1 ) { $command = "lcg-lr $edgRMopts lfn:${file}"; }
    my @resRcLR = `$command 2>&1`;
    print "\n resRcLR1: $file -$?- =$#resRcLR= \n@resRcLR";
    foreach my $l ( @resRcLR ) {
        chomp $l;
        my @i = split /[:\/]/, $l;
#prev	if ( $i[0] ne "srm" ) {
	if ( $i[0] !~ /srm|sfn/ ) {
	    print STDERR " File: ${file} on: $SE1 : Wrong SRM returned: $l \n";
	    return 1;
	} 

	my $gsrm = "$i[3]/${file}";
	my $csrm = "$SE1/${file}";
	if ( $csrm ne $gsrm ) {
	    print STDERR " File: ${file} on: $SE1 : A ghost file still existed: $gsrm \n";
	    return 1;
	}

        if ( ! exists $PFNs{$gsrm} || $PFNs{$gsrm} !~ /created|existing/ ) {
            print STDERR " Step: $SE1 File: $file did not previously existed: $gsrm ! \n";
	    return 1;
        } else {
	    $PFNs{$gsrm} = "existing";
	    print " SRM keys SE1: $SE1 $file value: $l \n";
	}
    }
    if ( $PFNs{"$SE1/$file"} ne "existing" ) {
	print STDERR " Step: $SE1 File: $file not correctly copied/registered ! \n";
	return 1;
    }

if ( "$SE1" ne "$SE2" ) {
#    my $resRcRF = `edg-replica-manager-replicateFile -e -l ${subDir}/${file} -s $SE1 -d $SE2 2>&1`;
    $command = "edg-replica-manager $edgRMopts replicateFile lfn:${file} -d $SE2";
###    if ( 1 ) { $command = "lcg-rep $edgRMopts -d $SE2 lfn:${file}"; }
    if ( 1 ) { $command = "lcg-rep $edgRMopts -d $SE2 -P tests/grodid/$serie/$gguid lfn:${file}"; }
    if ( 1 ) { $command = "lcg-rep $edgRMopts -d $SE2 -P tests/$file lfn:${file}"; }
    my $resRcRF = `$command 2>&1`;
    $stat = $?;
    print "\n resRcRF2: $file -$stat- \n$resRcRF";
    if ( $stat != 0 ) {
        print STDERR " File: ${file} on: $SE2 not correctly registered or created \n";
	return 1;
    }
    $PFNs{"$SE2/$file"} = "created";

#       @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
    $command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
    if ( 1 ) { $command = "lcg-lr $edgRMopts lfn:${file}"; }
    @resRcLR = `$command 2>&1`;
    print "\n resRcLR2: $file -$?- =$#resRcLR= \n@resRcLR";
    foreach my $l ( @resRcLR ) {
        chomp $l;
        my @i = split /[:\/]/, $l;
#	if ( $i[0] ne "srm" ) {
	if ( $i[0] !~ /srm|sfn/ ) {
	    print STDERR " File: ${file} on: $SE2 : Wrong SRM returned: $l \n";
	    return 1;
	} 

	my $gsrm = "$i[3]/${file}";
	my $csrm = "$SE2/${file}";
	if ( $csrm ne $gsrm ) {
	    next;
# 	    print STDERR " File: ${file} on: $SE2 : A ghost file still existed: $gsrm \n";
# 	    return 1;
	}

        if ( ! exists $PFNs{$gsrm} || $PFNs{$gsrm} !~ /created|existing/ ) {
            print STDERR " Step: $SE2 File: $file did not previously existed: $gsrm ! \n";
	    return 1;
        } else {
	    $PFNs{$gsrm} = "existing";
	    print " SRM keys SE2: $SE2 $file value: $l \n";
	}
    }
}

    print "\nPFN0: \n", map { "$_ => $PFNs{$_}\n" } keys %PFNs;
    print "-----------------------------------\n";
    if ( "$SE1" eq "$SE2" ) {
        if ( scalar(keys(%PFNs)) != 1 ) {
	    print STDERR " Replicates1 not correctly created on: $SE1 $SE2 \n";
	    return 1;
        }
    }
    elsif ( $#resRcLR < 1 || scalar(keys(%PFNs)) != 2 ) {
	print STDERR " Replicates2 not correctly created on: $SE1 $SE2 \n";
	return 1;
    }

    return 0;

}

sub removeGhostFile {

#     my ($self, $file) = @_;
#     my $edgRMopts = $self->{edgRMopts};
    my ($edgRMopts, $file) = @_;

#    $self->printLocation( (caller(0))[3] );

##	    foreach my $loop ( 0..2 ) {
##		my $file = "$ENV{USER}-$upid-$loop-rmstorm.txt";

		my $command = "edg-replica-manager $edgRMopts listGUID lfn:${file}";
                if ( 1 ) { $command = "lcg-lg $edgRMopts lfn:${file}"; }
		chomp(my $resRcLR = `$command 2>&1`);
		my $stat = $?;
		print "$stat\t                                   $resRcLR\n";
#		if ( $stat ) { next; }
    return 0 if $stat;

		$command = "edg-replica-manager $edgRMopts deleteFile --all-available --all-lfn $resRcLR";
                if ( 1 ) { $command = "lcg-del --verbose $edgRMopts -a $resRcLR"; }
		my $resRcDF = `$command 2>&1`;
		$stat = $?;
		print "\n resRcDF: $file -$stat- \n$resRcDF\n";
    return $stat if $stat;

		$command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
                if ( 1 ) { $command = "lcg-lr $edgRMopts lfn:${file}"; }
		my $resRcLRafter = `$command 2>&1`;
		$stat = $?;
		print "\n resRcLRafter: $file -$stat- \n$resRcLRafter";
#    return $stat if $stat;

		$command = "edg-replica-manager $edgRMopts listReplicas $resRcLR";
                if ( 1 ) { $command = "lcg-lr $edgRMopts $resRcLR"; }
		my $resRcLGafter = `$command 2>&1`;
		$stat = $?;
		print "\n resRcLGafter: $file -$stat- \n$resRcLGafter";
#    return $stat if $stat;
##	    }

    return 0;

}

__DATA__

#additional storage here

END
    ;
    close SCRIPT;

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}

# This will run the evaluateOutput method on all of the jobs.  The
# evaluateOutput method may be overridden to achieve a different
# evaluation. 
sub evaluateTest {
    my $self = shift;
    my %cseLab = (
		   'castor'    => 'Castor',
		   'pnfs'      => 'dCache',
		   'flatfiles' => 'Classic',
		   'storage' => 'Classic'
		  );

#    print "MultiDStorm: evaluateTest\n";
    $self->printLocation( (caller(0))[3] );

    my $config = "";  ## Still empty
    my $edgRMopts = "$config --insecure --vo=$self->{opts}->{vo}";

    my $gommand = "edg-lrc --insecure mappingsByPfn \'*$self->{targetSE}*-mdstorm-S$ENV{STACK_SERIE}-*.mds\' --length 20400 -h rlscert02.cern.ch -p 7777 --endpoint http://rlscert02.cern.ch:7777/dteam/v2.2/edg-local-replica-catalog/services/edg-local-replica-catalog";
    print " EvalCmd: $gommand \n";
    my @resRgLR = `$gommand 2>&1`;
    my $lrstat = 0;
    $lrstat = $? if $?;

    ## Implement proper parsing of output files, and subsequent evaluation here.
#     foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
# 	print " JOBL\t$index :\n$self->{reqs}->{$index}->{olout} \n";
#     }

    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
	print " JOBL\t$index :\n$self->{reqs}->{$index}->{olout} \n";
#p	print " JOUT\t$index :\n".`cat $self->{reqs}->{$index}->{olout}`." \n" if $self->{reqs}->{$index}->{olout};
	next unless $self->{reqs}->{$index}->{olout};
	my @res = `grep "JobLFN0: " $self->{reqs}->{$index}->{olout}`;

###GOOD	my $match = scalar(@res);
	my $match = 0;
	my $file;

	foreach (@res) {
	    my @m = split;
	    print " JLFN: $m[1] \n";
	    $file = $m[1];
	    last;  ##  bizarre, mais c'est comme ca ...
	    my $command = "lcg-lr $edgRMopts ${file}";
	    my @resRcLR = `$command 2>&1`;
#p	    print "\n resRcLR3: $file -$?- =$#resRcLR= \n@resRcLR";
	if ( ! $? ) {
	foreach my $l ( @resRcLR ) {
	    chomp($l);
	    if ( $l =~ "$self->{targetSE}" ) {
		print " Found: $l \n";
		$match++;
	    }
	}
	}
	}

	my $omatch = $match;
	$match = 0;
# 	my @m = split, $res[0];
# 	print " JLFN2: $m[1] \n";
# 	my $file = $m[1];
	my @m = split /-/, $file;

##LRC
##edg-lrc --insecure --verbose mappingsByPfn '*lxb1759*grodid*' --length 1400 -h rlscert02.cern.ch -p 7777 --endpoint http://rlscert02.cern.ch:7777/dteam/v2.2/edg-local-replica-catalog/services/edg-local-replica-catalog
##  my $gommand = "edg-rmc --insecure mappingsByAlias \'*-hstorm-S$ENV{STACK_SERIE}-*.dhs\' --length 20400 -h rlscert02.cern.ch -p 7777 --endpoint http://rlscert02.cern.ch:7777/dteam/v2.2/edg-replica-metadata-catalog/services/edg-replica-metadata-catalog";
	if (0) {
    my $gommand = "edg-lrc --insecure mappingsByPfn \'*$self->{targetSE}*-mdstorm-S$ENV{STACK_SERIE}-*.mds\' --length 20400 -h rlscert02.cern.ch -p 7777 --endpoint http://rlscert02.cern.ch:7777/dteam/v2.2/edg-local-replica-catalog/services/edg-local-replica-catalog";
    print " EvalCmd: $gommand \n";
    my @resRgLR = `$gommand 2>&1`;
}

	## Filtering these job files only ...
	my @resRcLR = grep { /.*?-$m[1]-$m[2]-$m[3]-$m[4]-$m[5]-.*\.mds/ } @resRgLR;
#pGOOD	print "\n resRcLR4: $file -$lrstat- =$#resRcLR= \n@resRcLR";
	print "\n resRcLR4: $file -$lrstat- =$#resRcLR= \n";
	if ( ! $lrstat ) {
	foreach my $l ( @resRcLR ) {
	    chomp($l);
	    ## The UI_CLOSE_SE name was already filtered thru the edg-lrc request ...
##	    if ( $l =~ "$self->{targetSE}" ) {
		$match++;
#p		print " Found: $l $match \n";
##	    }
	}
    }

# 	if (0) {
# 	foreach my $oFile ( @{$self->{lofile}} ) {
# #	    my @ostat = stat "$self->{reqs}->{$index}->{dirn}/$oFile";
# 	    $match-- if -s "$self->{reqs}->{$index}->{dirn}/$oFile" != 1024;
# 	    print " JOUT\t$index : $self->{reqs}->{$index}->{dirn}/$oFile $match \n";
# 	}
#     }
	
# 	next;

# 	chomp(my $res = `grep "JobLFN0: " $self->{reqs}->{$index}->{olout}`);
# 	my @m = split /\s+/, $res;
# 	print " JLFN1:\t$m[2] \n";
# 	my $file = $m[2];
# 	my $command = "lcg-lr $edgRMopts ${file}";
# 	my @resRcLR = `$command 2>&1`;
# 	print "\n resRcLR3: $file -$?- =$#resRcLR= \n@resRcLR";

# 	next;

# 	@m = split /-/, $file;
# 	my $match = 0;
# ##edg-rmc -i mappingsByAlias '*-hstorm-S1002-I0029-*-040915-*.dhs' --vo dteam --length 400 -h rlscert02.cern.ch -p 7777 --endpoint http://rlscert02.cern.ch:7777/dteam/v2.2/edg-replica-metadata-catalog/services/edg-replica-metadata-catalog
# ##lfn:dteam001-hstorm-S1002-I0029-CElxb1905-WNlxb1907-20144-040915-105207.dhs
# ##	my $command = "edg-replica-manager $edgRMopts listReplicas ${file}";
# 	if (0) {
# 	my $command = "edg-rmc --insecure mappingsByAlias \'*-$m[1]-$m[2]-$m[3]-*-$m[7]-*.dhs\' --length 10400 -h rlscert02.cern.ch -p 7777 --endpoint http://rlscert02.cern.ch:7777/dteam/v2.2/edg-replica-metadata-catalog/services/edg-replica-metadata-catalog";
# 	my @resRcLR = `$command 2>&1`;
# 	print " EvalCmd: $command \n";
#     }
# 	my @resRcLR = grep { /.*?-$m[1]-$m[2]-$m[3]-.*?-$m[7]-.*\.dhs/ } @resRcLR;
# 	print "\n resRcLR0: $file -$?- =$#resRcLR= \n@resRcLR";
# 	if ( ! $? ) {
# 	foreach my $l ( @resRcLR ) {
# 	    chomp($l);
# ##	    if ( $l =~ "$self->{targetSE}" ) {
# 		print " Found: $l \n";
# 		$match++;
# ##	    }
# 	}
#     }

##test	if ( $index == 1 ) { $match = 10; }  ##  TEMP TEST
	if ( $match != $self->{opts}->{ofile}+1 ) { print " Matching: [FAILED] \n"; }
	else { print " Matching: [OK] \n"; }
#old	$self->{reqs}->{$index}->{match} = $match;
	if ( $self->{reqs}->{$index}->{stat1} eq '[OK]' ) {
	    if ( $match != $self->{opts}->{ofile}+1 ) {
		$self->{reqs}->{$index}->{stat1} = '[FAIL]';
		$self->{reqs}->{$index}->{stat2} = 'MATCHING';
		$self->{reqs}->{$index}->{olog} = $self->{reqs}->{$index}->{olout};
	    } else {
		delete $self->{reqs}->{$index}->{olout} unless ($index < 10);
	    }
	}
#p	print " SUMM\t$index :\n", map { "\t$_\t=>\t$self->{reqs}->{$index}->{$_} \n" } sort keys %{$self->{reqs}->{$index}};
    }

    my $rc = $self->SUPER::evaluateTest();
    return $rc if ($rc);

    return 0;

}

sub buildXMLFile {
 
    my $self = shift;

#    print "Stack: buildXMLFile\n";
#    $self->{debug} = 1;
    $self->printLocation( (caller(0))[3] );

    my @mopts = split /,/, $self->{opts}->{display};

    ## Default: several columns, prefixed:mace as a col index, mawn as a row index
    if ( ! scalar(@mopts) ) { $self->SUPER::buildXMLFile(); }
    elsif ( $mopts[0] eq "1Col" ) {
	## this arg arg may be: "jind", "mace", "mawn" (as a row index)
	## "jind" stands for Job_index
	$self->SUPER::buildXMLFile1Col($mopts[1]);
    } else {
	## Now, 1st arg may be: "mace", "mawn" (as a col index)
	## while 2nd arg may be: "jind", "mace", "mawn" (as a row index)
	$self->SUPER::buildXMLFile($mopts[0], $mopts[1]);
    }
    return 0;

}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
