#!/usr/bin/perl

package TestUtils;

use strict;
require Exporter;

use vars qw(@ISA @EXPORT);
@ISA = qw(Exporter);
@EXPORT = qw(mytimeoutOLD mytimeout timeoutWithKill);

=head1 NAME

TestUtils.pm
# $Id: TestUtils.pm,v 1.1.1.2 2005/02/08 16:09:43 mbarroso Exp $

=head1 SYNOPSIS

TestUtils.pm - Module including some utility functions used in other test objects

=head1 DESCRIPTION 

B<TestUtils.pm> Module including some utility functions used in other test objects:

mytimeout() -  A utility function to execute a given command with a timeout. This is not intended to be called directly but to be used through the edg-tb-test command.

=head1 AUTHOR

Gonzalo Merino and Miquel Barcelo.  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.
ReWritten by Gilbert Grosdidier to include killing facilities (27/06/04).

=head1 LICENSE

Copyright (c) 2002 by Gonzalo Merino and Miquel Barcelo (IFAE).
All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

## A utility function to execute a given command with a timeout
## ***************************************************************************************
sub mytimeoutOLD {
  my ($command, $timeout) = @_;
  $timeout ||= 240;
#p  print STDERR " Timeout set to: $timeout \n";

    STDOUT->autoflush(1);
    STDERR->autoflush(1);

  my $rc = 0;
  my $output = '';

  my $r = rand;

  eval {
    local $SIG{ALRM} = sub {die "time out\n"};
    alarm $timeout;
    $output = `$command 2>&1`;
    $rc = $? >> 8;
    alarm 0;
  };

  if ($@) {
    die unless $@ eq "time out\n";   # propagate unexpected errors
    # timed out
    $rc = -1;
    print "Command timed out\n";
    $output .= "Command timed out\n"; ##  GG
  }

  $SIG{ALRM} = 'DEFAULT';

  return { rc=>$rc, out=>$output};

}

## Dummy interface to new timeoutWithKill
## ***************************************************************************************
sub mytimeout {
  my ($command, $timeout) = @_;
  $timeout ||= 240;

  my $href = timeoutWithKill($command, $timeout, 9);

#  return { rc=>$href->{rc}, out=>$href->{out} };
  return $href;

}

## Submit the command thru a fork but keep reading from it thru PH filehandle
## Args: command, timelapse (60), signal (9)
## ***************************************************************************************
sub timeoutWithKill {

    local *PH;

    my $command = shift;
    my $timeout = shift;
    $timeout ||= 60;
    my $gigKill = shift;
    $gigKill ||= 9;
    my $comadd  = shift;
    $comadd ||= " ";
    print " NEW Cmd: >$command 2>&1 $comadd< Timeout set to: $timeout ; Kill with: -$gigKill \n";
#p    print STDERR " NEW Cmd: >$command 2>&1 $comadd< Timeout set to: $timeout ; Kill with: -$gigKill \n";

    my $pid = 0;

    ## Fork, and read from the Child output
    die "cannot fork: $!" unless defined ( $pid = open ( PH, "-|" ) );

## TIP: "ps --no-headers --pid 19778" selects the right line 
## and provides the process name (4th col)
## that can be used in a "pkill -g 19778 pname"
## or in a "killall -g -s 9 -- pname" I think.

    if ( $pid ) {		# Parent  <=================================================================

##GOODFORTEST    print "Submit $pid $? \n";
	my $rc = 0;
	my $output = '';

	## Reading from Child output, while capturing it [and printing it to STDOUT as well ?]
	while (<PH>) {
	    $output .= $_;
##OPTIONAL	print STDOUT " Line: $_";
##GOOD	    if ( $comadd ne " " ) { print STDOUT " Line: $_"; }  ##  For testing period ??
	    if ( $comadd ne " " ) {
		if ( ! /GLOBALRESULT/ ) { print STDOUT "$_"; }  ##  For testing period ??
	    }
	}
	close PH;
##GOODFORTEST      my $res = `ps -weo "uid_hack,pid,ppid,pgid,c,ni,lstart,tname,etime,time,pcpu,vsize,cmd" | egrep 'grod|UID' >> kill.log 2>&1`;

	if ( 0 ) {
	    print " RC0: $rc \n";
	    print " RC1: $@ \n";
	    print " RC2: $? \n";
	    print " OUT: \n$output \n";
	}

	## Trying to spot if something went wrong, and flag accurately the "time out" case ...
	if ($?) {
	    if ( $output =~ "time out" ) {
		# timed out
		$rc = -1;
		#p	  print " IF: Command timed out\n";
		$output .= "Command timed out\n"; ##  GG
	    } else {		# propagate unexpected errors
		$rc = $? >> 8;
		#p	  print " IF: Command failed\n";
		$output .= "Command failed\n"; ##  GG
	    }
	}

	return { rc=>$rc, out=>$output };

    } else {			# Child  <=================================================================

	my $cpid = $$;
	STDOUT->autoflush(1);
	STDERR->autoflush(1);

	## To dup STDERR to STDOUT
	open STDERR, ">&STDOUT";

	## To create a new process group where the child is the leader
	setpgrp (0, 0);
#NONEED ??      sleep 1;

#GOOD      my $res = `ps -weo "uid_hack,pid,ppid,pgid,c,ni,lstart,tname,etime,time,pcpu,vsize,cmd" | egrep 'grod|UID' > kill.log 2>&1`;
	## No need, but for getting $pname (optional indeed)
	my @m = split /\s+/, `ps --no-headers --pid $cpid`;
	my $pname = $m[-1];
##GOODFORTEST      print STDERR " Showing  the CHILD process: $cpid for timeout: $timeout sec. To be killed: $pname \n";

	eval {
	    local $SIG{ALRM} = sub {
		print STDOUT "\n #####################################CTRL+C
 timeoutWithKill time out occured while running:
 Cmd: >$command 2>&1 $comadd<
 Signal -$gigKill is being sent to process group: $$
 #####################################CTRL+C \n\n";

#NONEED		print STDERR " ERR time out\n Signal -$gigKill sent: $$ \n";
		local $SIG{KILL} = 'IGNORE';
#GOODFORTEST      my $res = `ps -weo "uid_hack,pid,ppid,pgid,c,ni,lstart,tname,etime,time,pcpu,vsize,cmd" | egrep 'grod|UID' > kill.log 2>&1`;

		## To kill the child and its sibling (the - sign means the whole process group)
		kill -$gigKill => $cpid;

	    };
	    print STDERR " Starting the CHILD process: $$ for timeout: $timeout sec. To be killed: $pname \n";
	    alarm $timeout;

	    #	exec("$command 2>&1");
	    system("$command 2>&1 $comadd");
###GOOD	    system("$command  $comadd");

	    alarm 0;
	};

	exit;			## for safety ...

    }

}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
