#!/usr/bin/perl -w

# DAG version - Mario Reale  / EGEE JRA-1 Testing team June 2, 2005 

use POSIX qw(:signal_h :errno_h :sys_wait_h);
use File::Basename;
use IO::File;
use strict;

           use IPC::SysV qw(IPC_PRIVATE S_IRWXU IPC_CREAT);
           use IPC::Msg;


=head1 NAME
SubmitUtils.pm - Specific to the Stack subclass.
# $Id: SubmitUtilsDAG.pm,v 1.1 2005/06/06 17:58:54 reale Exp $

=head1 SYNOPSIS

This utility script is specific to the Stack subclass,
and MUST NOT be called directly.

=head1 DESCRIPTION 

B<SubmitUtils.pm> This specific utility script is used by the Stack subclass,
for generating the Storms. It must not be called directly in any way.
It is not a user callable function.

 Please refer to BaseTest::EDGLifecycle::Stack and/or to
BaseTest::EDGLifecycle::Stack::JobStorm for simple examples of Storms.

=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
Mario Reale ( Mario.Reale@cern.ch )
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

STDOUT->autoflush(1);
STDERR->autoflush(1);

my $jdl  = $ARGV[0];

chomp(my $date = `date`);
$date =~ s/\s+/_/g;

## Several sleeping times and polling periods ...
my $sleepT = $ENV{STACK_POLLING_PERIOD};

my $timeoutReq = $ENV{STACK_CHILD_TIMEOUT} - 2*$sleepT;

####my $timeoutReq = 150; 

my $useSemaphores = $ENV{STACK_SEMAPHORES};
my $timeoutStat = 100;
#my $timeoutStat = 30;
my $timeoutSub  = 300;

my $cmds = dgjob("submit");
my $cmdt = dgjob("status");
my $cmdo = dgjob("get-output");
my $cmdl = dgjob("get-logging-info");


my $stdOutput = "stdout.file";
my $stdError = "stderr.file";

my $bigFileName = `grep BigFileName $jdl | cut -d'"' -f2` || "";
chomp $bigFileName;

my $config = "";
my $index = 0;
my @mstore = ();
my @staStore = ();
my $mflag = 0;
my $jobId;
my $debugF = 0;
my $logFile;
my $outFile;
my $destination = "";
my $daddyspid = 0;
my $Aserie = "S";
my $Aindex = "I";

if ( $#ARGV > 0 ) {
    $index = $ARGV[1];
    $Aindex .= $index;
    $logFile = $ARGV[2];
    $outFile = $ARGV[3];
    $daddyspid = $ARGV[4];
}
if ( exists $ENV{STACK_SERIE} ) {
  $Aserie .= $ENV{STACK_SERIE};
}
## Local temporary dir
my $localDir = dirname($jdl);


my $sigusr1 = 0;
$SIG{USR1} = sub { print STDERR " Signal USR1 received0: $$ \n"; die "Timeout in SubmitUtils:"; };

## Only this one seems to be useful !
if (0) {
$SIG{ALRM} = sub { print STDERR " Signal ALRM received0: $$ \n"; 

		   &subReport();
		   printRES (" flag=>Status, index=>$index, jdl=>$jdl, date=>$date, macef=>$destination, stat2=>TIMEOUTGLOB, stat1=>[FAIL]\n");
		   die "Timeout in SubmitUtils:"; };
} else {
    $SIG{ALRM} = \&subAlarm;
}

## Slac specific, left as an example
if (0) {
if ( $#ARGV > 1 ) { 
  $config = $ARGV[2];
  print " Config: _${config}_ \n";
  if ( $config eq "gw39" ) {
    $config = "-config /u/eb/grodid/public/globus/grodid/edg-test/UI_ConfigENV_gw39.cfg";
  } elsif ( $config eq "cern" ) {
    $config = "-config /u/eb/grodid/public/globus/grodid/edg-test/UI_ConfigENV_cern.cfg";
  } elsif ( $config eq "gm04" ) {
    $config = "-config /u/eb/grodid/public/globus/grodid/edg-test/UI_ConfigENV_gm04.cfg";
  }
}
} else {
    ## This one is required by all commands
##    $config = "--config $ENV{EDG_TESTROOT}/etc/edg_wl_ui_cmd_var-lxshare0234.conf";
    ## But this one is only for job-submit, job-list-match, job-cancel, job-status
##    $configvo = "--config-vo "$ENV{EDG_TESTROOT}/etc/edg_wl_ui-lxshare0234.conf";
}

if (0) {
## Inserting the variable Arguments into the jdl file
my $njdl = "/tmp/$ENV{USER}-$$-" . "temp.jdl";
if (0) { ##old
  my $bid = `grep -v Arguments $jdl > $njdl`;
  $bid = `echo 'Arguments = "${Aserie} ${Aindex} rien"; ' >> $njdl`;
  $jdl = $njdl;
} else { ##new

## we are NOW using directly $jdl ...
}
}

{
    my ($rct, $rcc, $cmdFile, @mstor) = (0, 0, "", ());
    ($rct, $rcc, $cmdFile, @mstor) = timeoutWithFork("$cmds $config $jdl 2> /dev/null", $timeoutSub, $cmdFile);

     print STDERR "\n mstor = @mstor \n";

     print "\n rct = $rct \n";


    if ( $rct ) {
	printRES (" flag=>Trying_submission, index=>$index, none=>-$config-, jdl=>$jdl, date=>$date, mess=>Error, stat2=>TIMEOUTSUB, stat1=>[FAIL]\n");
	##exit 1;
	subExit(1, "", $cmdFile);
    } else {
	unlink $cmdFile;
	@mstore = @mstor;
    }
}

## For debugging the environment
if (0) {
    print STDERR " ENV_SUB: ", `env `;
}

foreach my $lout ( @mstore ) {
    $_ = $lout;  ##  For compat w/ old code

## For debugging the submission
if (0) {
    print STDERR " SUB Seen: $_";
}

## Special to simulate breaks or mistakes ... ================
    if ( $index == 11 && $debugF ) {
      $_ =~ s/Connecting/Error:/;
    }
## Special to simulate breaks or mistakes ... ================

  if ( /Error:/ ) {
    chomp ( my $line = $_ );
      printRES (" flag=>Trying_submission, index=>$index, none=>-$config-, jdl=>$jdl, date=>$date, mess=>$line, stat2=>UNSUBMITTED, stat1=>[UNTESTED]\n");
      $mflag = 1;
  }
  if ( /https/ ) {
    my @m = split /\s+/;
    $jobId = $m[2];
    my $eRes = `echo $jobId >> $localDir/jobids`;
    print "\n JobId: $index $jdl $date $jobId \n\n";
    print STDERR " JobId: $index $jdl $date $jobId \n";
    
    printRES (" flag=>Trying_submission, index=>$index, date=>$date, jobid=>$jobId\n");
    last;
  }
}


## Removing the temporary jdl file
##temp $bid = `rm $njdl`;
#maybenotafterall unlink $jdl;

if ( ! defined $jobId ) { 
    print " SUB Warning: Error_${index}_$date \n";

if (1) {
    open MES, "> $logFile";
    print MES @mstore;
    close MES;
}
    exit 1; 
}

if (0) {

#comm

} else {
    if ( $useSemaphores ) {
     my $msg = new IPC::Msg($useSemaphores, S_IRWXU);
    $msg->snd(1234567, "$index $jobId $$");
    my $buf;
    my $type = $msg->rcv($buf, 256, 1+$index);
    if ( $buf =~ /VAZY/ ) {
    }
}
}

my $dt     = $sleepT;
my $wait   = 0;
my $rct    = 0;
my $rcc;
my $done   = "NOT";
my $done2  = "NOT";
my $nready = 0;
my %nr     = ();
my $ndone  = 0;
my $stat2 = "";
my $cmdFile = "";
my $STAFlag = 0;
WSTA: while (1) {
    @mstore = ();
    $mflag = 0;
    sleep $dt;
    $wait += $dt;

    while ( $mflag <= 5 ) {
	($rct, $rcc, $cmdFile, @mstore) = timeoutWithFork("$cmdt $config $jobId 2> /dev/null", $timeoutStat, $cmdFile);
        print STDERR "\n mstore = @mstore \n";
	last if ( $rct == 0 );
	$mflag++;
  
    }
    if ( $rct != 0 ) {
	&subReport();
	$stat2 = "TIMEOUTSTAT";
	printRES (" flag=>Status, index=>$index, jdl=>$jdl, date=>$date, none=>$wait, macef=>$destination, stat2=>$stat2, stat1=>[FAIL]\n");
	subExit(1, "", $cmdFile);
    }
    if ( $rcc != 0 ) {
	if (0) {
	    open MES, "> $logFile";
	    print MES @mstore;
	    close MES;
	} else {
	    &subReport();
	}
	$stat2 = "DEADSTAT";
	printRES (" flag=>Status, index=>$index, jdl=>$jdl, date=>$date, none=>$wait, macef=>$destination, stat2=>$stat2, stat1=>[FAIL]\n");
	subExit(1, "", $cmdFile);
    }

    my $flagLoc = 0;
    foreach ( @mstore ) {
	my @m = split /\s+/;
	next unless $m[1];
	if ( $m[0] eq "Destination:" ) { $destination = $m[1]; }
	next unless $m[2];
	if ( $m[1] eq "Status:" && $m[2] eq "Done" && $m[3] eq "(Success)" ) { $flagLoc++; }
	next unless $m[3];
	if ( $m[1] eq "Reason:" && $m[2] eq "(nil)" ) { $flagLoc++; }
    }
    if ( $flagLoc == 2 ) { 
	print STDERR " >>>> Weirdness found: flagLoc == 2 in $jobId \n";
	next WSTA; 
    }

    foreach ( @mstore ) {
    if ( /Status/ ) {
      my @m = split /\s+/;

      if ( $m[1] ne "Status:" ) { next; }
      push @staStore, " STA  Status: $index $jdl $date $wait $m[2] \n";

      print " STA  Status: $index $wait @m[2..$#m] \n"; ## if ( $STAFlag++ == 0 );



###      print STDERR "\n  ====== m[2] = $m[2] \n"; 

      if ( $m[2] eq "Done" && $m[3] eq "(Success)" ) { $m[2] = "Done"; }
      elsif ( $m[2] eq "Done" && $m[3] eq "(Cancelled)" ) { 


	  $m[2] = "Cancelling";
	  print STDERR " >>>> Weirdness found: $m[2] in $jobId \n";
      }
      elsif ( $m[2] eq "Done" && $m[3] eq "(Failed)" ) { 
	  $m[2] = "Failing";
	  print STDERR " >>>> Weirdness found: $m[2] in $jobId \n";
      }
      elsif ( $m[2] eq "Done" && $m[3] eq "(Exit" ) {
           $m[2] = "Failing";
           print STDERR " >>>> Weirdness found: $m[2] in $jobId \n";
           printRES (" flag=>Status, index=>$index, jdl=>$jdl, date=>$date, none=>$wait, stat2=>$stat2, stat1=>[FAIL]\n");
           &subReport();
           subExit(1, "", $cmdFile);

      }
      if ( $m[2] ne "Done" ) {

	  if ( $m[2] =~ /Scheduled|Submitted|Ready|Running|Waiting|Cancelling|Failing/ ) { 
	      if ( $dt*$nr{$m[2]}++ < $timeoutReq ) {
		  next WSTA;
	      } else {
		  $stat2 = "TIMEOUTREQ";
		  $m[2] = "Aborted";
	      }
	  } elsif ( $m[2] ne "Aborted" ) {
	      print STDERR " >>>> VERY WEIRD keyword found: $m[2] in $jobId \n";
	  }
      }

      if ( $m[2] eq "Aborted" || $m[2] eq "Done" ) {
	$done = $m[2];
        $done2 = $m[3];
	if ( $done eq "Aborted" ) {
	    $stat2 = "ABORTED" if ( $stat2 eq "" );
	    if ( $sigusr1 != 0 ) { $stat2 = "TIMEOUT"; };
	    printRES (" flag=>Status, index=>$index, jdl=>$jdl, date=>$date, none=>$wait, stat2=>$stat2, stat1=>[FAIL]\n");
	}
        elsif ( $done eq "Done" && $done2 eq "(Exit" ) {
            $stat2 = "ABORTED" if ( $stat2 eq "" );
            if ( $sigusr1 != 0 ) { $stat2 = "TIMEOUT"; };
            printRES (" flag=>Status, index=>$index, jdl=>$jdl, date=>$date, none=>$wait, stat2=>$stat2, stat1=>[FAIL]\n");
        }
	last;
      }
      last;
    }
  }
  if ( $done ne "NOT" ) { last; }
} 

if ( $done eq "Aborted" ) { 

    &subReport();
    subExit(1, "", $cmdFile);
}

if ( $done eq "Done" && $done2 eq "(Exit" ) {

    &subReport();
    subExit(1, "", $cmdFile);
}



$mflag = 0;

my $dagsubdir = "nothing";
my $outName = "nothing";
my $outNameProv = "nothing";
my $outNameT = "nothing";
my $errName = "nothing";
my $dirName = "nothing";
my $bigName = "nothing";

my $dagsubdir2 = "nothing";
my $outName2 = "nothing";
my $outName3 = "nothing";
my $outNameProv2 = "nothing";
my $errName2 = "nothing";
my $dirName2 = "nothing";
my $bigName2 = "nothing";
my @outNameArr = "nothing"; 

GETOUT: while ( $mflag <= 5 ) {
    @mstore = ();
    open GET, "$cmdo -dir $localDir $config $jobId 2> /dev/null |";
    while (<GET>) {
	push @mstore, $_;
	if ( /NS_JOB_OUTPUT_NOT_READY/ ) {
	    my $sl = `sleep $sleepT`;
	    $mflag++;
	    next GETOUT;
	}

        my @spidf1 = split /\-/, $logFile;
        my $siwant1 = $spidf1[2];

        my @spidf2 = split /\//, $siwant1;
        my $siwant2 = $spidf2[0];

         my @spidf3 = split /\DAG/, $logFile;
         my $siwant3 = $spidf3[0];


        if ( /sandbox/ ) {
           my $temp = <GET>;
           push @mstore, $temp;
           my @m = split  /\//, $temp;
           $outNameProv = $m[3];
           chomp $outNameProv;

           $outNameT = "$siwant3"."/$ENV{USER}_$outNameProv";

           $dirName = $outName;
           $errName = "$outName"."/$stdError";
           $outName .= "/$stdOutput";
           $bigName = $dirName."/$bigFileName";
        }

        if ( /   Children/ ) {
           my $temp = <GET>;
           push @mstore, $temp;
           my @m = split /\//, $temp;
           $outName2 = $m[3];
           chomp $outName2;
        }

           
#         $outName = "/tmp/$ENV{USER}-$daddyspid-$siwant2"."/$ENV{USER}_$outNameProv"."/$ENV{USER}_$outName2"."/stdout.file";
          $outName = "$siwant3"."$ENV{USER}_$outNameProv"."/$ENV{USER}_$outName2";
           $dirName = $outName;
           $errName = "$outName" . "/$stdError";
           $outName .= "/$stdOutput";
           $bigName = $dirName . "/$bigFileName";
    }

    close GET;
    last;
}

    print STDERR "\n PID super = $daddyspid \n";
    print STDERR "\n outNameProv = $outNameProv \n";
    print STDERR "\n outName = $outName \n";
    print STDERR "\n outName2 = $outName2 \n";
    print STDERR "\n dirName = $dirName \n";
    print STDERR "\n errName = $errName \n";
    print STDERR "\n bigName = $bigName \n";

my $statLines;

my $stat1;

if ( $outName ne "nothing" ) {
 
#  print STDERR "\n outName ne nothing \n";

#  print STDERR "\n stderr   $errName \n";
#  print STDERR "\n outName  $outName \n";


  my $errLines = `wc $errName | awk '{print \$1}'`;
  my $outLines = `wc $outName | awk '{print \$1}'`;


#  print STDERR "\n errLines = $errLines \n";
#  print STDERR "\n outLines = $outLines \n";

  $statLines = " "; 

if ( $outLines*$errLines == 0 ) { 
    $statLines = "empty";
    if ( $errLines == 0 ) { $statLines = "err_" . "$statLines"; }
    if ( $outLines == 0 ) { $statLines = "out_" . "$statLines"; }
}
}

    print "\n GET OutName: $index $jdl $date $wait $outName \n";
    print STDERR "\n =========================== \n";

    print STDERR "\n outName = $outName \n";
    print STDERR "\n statLines = $statLines \n";

 #   if ( $outName eq "nothing" || $statLines ne " " ) { 
    if  ( $outName eq "nothing" || $statLines eq "out_empty" ) {

    printRES (" flag=>Warning, index=>$index, mess=>nothing_${index}_${date}_${statLines}_${mflag}, stat2=>MISSING_FILES, stat1=>[FAIL]\n");

    open MES, "> $logFile";
    print MES @mstore;
    close MES;

    if ( $outName eq "nothing" || $statLines eq "out_empty" ) { 

            $stat1 = '[FAIL]';
            $stat2 = 'OUTFILE';

  printRES (" flag=>Status, index=>$index, jdl=>$jdl, date=>$date, none=>$wait, macef=>$destination, none=>$statLines, stat2=>PARTFAIL, stat1=>[FAIL]\n");

     subExit(2, $dirName, $cmdFile);

    }
#}

	    $stat1 = '[OK]'; 
	    $stat2 = 'SUCCESS';

    my @m = "ok "; 

  printRES (" flag=>Status, index=>$index, jdl=>$jdl, date=>$date, none=>$wait, macef=>$destination, none=>$m[1], none=>$m[2], none=>$statLines, stat2=>$stat2, stat1=>$stat1\n");



print "\n";

subExit(0, $dirName, $cmdFile);
}

#========================================================================================================================

sub subExit {
    my ($rc, $dirName, $cmdFile) = @_;
    unlink $cmdFile unless ( $cmdFile eq "" );
    exit $rc;
}


sub printRES
  {
    my $str = shift;
    if (0) {
      open RES, ">> resume-loop.out";
      print RES $str;
      close RES;
    } else {
	chomp(my $date = `date`);
	$date =~ s/\s+/_/g;
      print " Summary  timest=>$date,$str";
    }
    return 0;
  }


# This will execute the given command subject to the given timeout (in
# seconds).  It returns an array reference with the first element
# being the return code and the second a scalar variable with the
# command output.
sub timeoutWithRC {
    my ($command, $timeout) = @_;
    $timeout ||= 60;

    my $rc = 0;
    my $crc = 0;
    my @output = ();

    my $r = rand;

    eval {
        local $SIG{ALRM} = sub {$rc = 1; die " Timeout in gg-job" }; 
        alarm $timeout;
        @output = `$command`;
        $crc = $?;
        alarm 0;
    };

    return ($rc, $crc, @output);
}


# This will execute the given command subject to the given timeout (in
# seconds).  It returns an array reference with the first element
# being the return code and the second a scalar variable with the
# command output.

sub timeoutWithFork {
    my ($command, $timeout, $cmdFile) = @_;
    $timeout ||= 60;

    my $rc = 0;
    my $crc = 0;
    my @output = ();

    if ( $cmdFile eq "" ) {
    my @cmd = <<"EOC";
#!/usr/bin/perl -w

    my  \$rc = 0;
    my  \$crc = 0;
    my  \@output = ();

     \$SIG{ALRM} = sub { \$rc = 1; die " Timeout in timeoutWithFork" }; 
    eval {
        alarm $timeout;
         \@output = `$command`;
         \$crc =  \$?;
        alarm 0;
    };

    die "Timeout in timeoutWithFork" if \$rc;

    print STDOUT "CRC:  \$crc \\n";
    print STDOUT  \@output;

    exit 0;
EOC

#    $cmdFile = "/tmp/cmd-$ENV{USER}-$$-timeout.pl";
    $cmdFile = "$localDir/cmd-${Aindex}-$$-timeout.pl";
    open CMD, ">$cmdFile";
    print CMD @cmd;
    close CMD;
    chmod 0755, $cmdFile;
}

    open OUT, "$cmdFile 2>&1 |";
    while (<OUT>) {
	if ( /Timeout/ ) { $rc = 1; last; }
	else {
	    if ( /CRC:/ ) { my @m = split /\s+/; $crc = $m[1]; }
	    else { push @output, $_; }
	}
    }
    close OUT;

#gg    return [$rc, $crc, $output];
    return ($rc, $crc, $cmdFile, @output);
}

#======================================================================================================================

sub dgjob {
    my $cmdn = shift;
    my $root = "-job-";
    if ( $ENV{EDG_FLAVOR} eq 'edg2' ) {
	$root = "edg".$root;
    } elsif ( $ENV{EDG_FLAVOR} eq 'glite1' ) {
 	$root = "glite".$root;
	$cmdn =~ s/get-//;
    }
    return "$root"."$cmdn";
}

#======================================================================================================================

sub subReport {

    open MES, "> $logFile";
    print MES "+0+--------------------------------------\n";
    print MES @staStore;
    print MES "+1+--------------------------------------\n";
    print MES @mstore;
    @mstore = `$cmdl $config -v 2 $jobId 2> /dev/null `;
    print MES "+2+--------------------------------------\n";
    print MES @mstore;
    print MES "+3+--------------------------------------\n";
    close MES;

    return;
}
#==================================================================================================================
sub subAlarm{ 

print STDERR " Signal ALRM received0: $$ \n"; 

		   &subReport();
		   &printRES (" flag=>Status, index=>$index, jdl=>$jdl, date=>$date, macef=>$destination, stat2=>TIMEOUTGLOB, stat1=>[FAIL]\n");
		   die "Timeout in SubmitUtils:";
}
1;
