#!/bin/bash
######################################################################
#
# Authors: Miquel Barcelo  - barcelo@ifae.es
#	   Gonzalo Merino  - merino@ifae.es
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
######################################################################

# some HTML functions used...

HTML_OUT=test.html
HTML_DIR=${HTML_DIR:-$EDG_TEST_HTML_OUTPUT}
HTML_STYLE=$EDG_TEST_HTML_STYLE

HTML_echo ()
{
    test -e $HTML_DIR/$HTML_OUT && echo $1 >> $HTML_DIR/$HTML_OUT
}

HTML_dump ()
{
    test -e $HTML_DIR/$HTML_OUT && cat $HTML_STYLE/$1 >> $HTML_DIR/$HTML_OUT
}

HTML_cat ()
{
    HTML_echo "<br><pre>"
    test -e $HTML_DIR/$HTML_OUT && cat $1 >> $HTML_DIR/$HTML_OUT
    HTML_echo "</pre><br>"
}

# HTML_validate: return codes
#                0 Test can continue normally
#                1 Some error produced while testing, test may continue
#                2 Critical error produced, test must stop
#
# NOTE: On case of returning 1, the parser should decide if continue the test
# or not, depending on configuration.

HTML_validate ()
{
# parse log to HTML link
   local RETVALUE=0
   case "$1" in
   $RET_PASSED)
	HTML_dump oktest_start.html
        HTML_echo "$HTML_LOG"
	HTML_dump oktest_end.html
        #Test step returned success. Continue normally
	RETVALUE=0
	;;
   $RET_FAIL)
	HTML_dump failtest_start.html
	HTML_echo "$HTML_LOG"
	HTML_dump failtest_end.html
	#Test step failed. May stop test
	RETVALUE=1
	;;
   $RET_WARNING)
	HTML_dump warntest_start.html
        HTML_echo "$HTML_LOG"
	HTML_dump warntest_end.html
	RETVALUE=0
	#Test step issued a warning. Test can continue
	;;
   $RET_XFAIL)
	HTML_dump xfailtest_start.html
        HTML_echo "$HTML_LOG"
        HTML_dump xfailtest_end.html
	#Test step failed. May stop test
	RETVALUE=1
        ;;
   $RET_UNRESOLVED)
	HTML_dump unresolved_start.html
        HTML_echo "$HTML_LOG"
        HTML_dump unresolved_end.html
	#Test didn't perform due a configuration error. Do not continue!
	#This can be due some test script / parser internal error,
	#testsuite misconfiguration or some "conceptual" contradiction, like:
	#-Using same host as test target and reference.
	RETVALUE=2
        ;;
   $RET_UNTESTED)
	HTML_dump untested_start.html
        HTML_echo "$HTML_LOG"
        HTML_dump untested_end.html
	#This test step is not implemented (yet), or test script decided to skip it.
	#Continue with the following ones
	RETVALUE=0
        ;;
   $RET_UNSUPPORTED)   
	HTML_dump nosupport_start.html
        HTML_echo "$HTML_LOG"
        HTML_dump nosupport_end.html
	#Some software version is incorrect. Test must end.
	RETVALUE=2
        ;;
   *)   HTML_echo "<tr><th>null"
	RETVALUE=0
	;;
   esac
   if [ ! -z "$2" ] ; then 
	HTML_dump substat_start.html
	HTML_echo "$2"
	HTML_dump substat_end.html
   fi
   HTML_dump test_end.html
   return $RETVALUE
}

HTML_create ()
{
    if [ -e $HTML_DIR/$HTML_OUT ] ; then rm -fr $HTML_DIR/$HTML_OUT ; fi
    mkdir -p $HTML_DIR
}
