#!/bin/sh
######################################################################
#
# Authors: Andrea Formica   - Andrea.Formica@cea.fr
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
# Based on INFN Testgroup scripts
#
######################################################################

# Create log directory for all info files
HOUR=`date`
EXEDIR=`pwd`

#Set printing variables
THIS_SCRIPT="InstallRPM.sh"
PRETXT="[${THIS_SCRIPT} ${HOUR}]:"
ISTEST=""

# Load environment variables
if [ -z $INSTALL_EDG_TESTROOT ]
then
    echo "INSTALL_EDG_TESTROOT is not defined "
    exit 4
fi

# Load functions

# Variables used between functions
RPMLIST=""
RPMNAME=""
distribution=""
donelist=""

# General Variables
INST_INFO_FILE=./inst-info
FAILED_FILE=./failed.txt
PRESTRING=""

export INST_INFO_FILE PRESTRING RPMLIST RPMNAME

#NODE=`$INSTALL_EDG_TESTROOT/myhostname.sh shortname`
NODE=`hostname --short`
export LOGDIR="./RPMS.${NODE}"

if [ ! -d $LOGDIR ];  then
	mkdir $LOGDIR
fi

olddone=""

ISTEST=
MAXLOOP=13
# overwrite maxloop if is a test
[ ! -z $ISTEST ] && MAXLOOP=2 && $ECHO "$THIS_SCRIPT: Is a test ==> overwrites MAXLOOP = $MAXLOOP"
 
#
# Get Argument for Installation or Restoring
#
# Initialise
ACTION=
DIRPKG=
ARGPKG=

# Reinit OPTIND to make getopts working
if [ $OPTIND -ne 1 ]; then
    OPTIND=1
fi

while getopts "o:p:d:h" myopt; do
	case  "$myopt" in
		p)
			ARGPKG=$OPTARG
		;;
		d)
			DIRPKG=$OPTARG
		;;
		o)
			echo "Iteration will " $OPTARG " the selected package..."
			ACTION=$OPTARG
		;;
		h)
			echo "$THIS_SCRIPT  : will try to {install | restore} from a list named"
                        echo "                <directory-path>/<package>all                    "
                        echo "                <package> can also be a list. If so, the script looks"
                        echo "                for every <package> in that list, and perform the wanted"
                        echo "                action on that list"
                        echo ""
			echo "Usage: $THIS_SCRIPT -p <package>           "
			echo "                    -o {install | restore} "
			echo "                    -d <directory-path>    "
			exit $ERROR
		;;
		*)
			echo "Try -h for help!"
			exit $ERROR
		;;
	esac
done

if [ -z $ACTION ]; then
	writeOutput "$THIS_SCRIPT: ERROR Empty action flag" 
	writeOutput "Type $THIS_SCRIPT -h for Help"
	exit 1
fi
if [ -z $DIRPKG ]; then
	writeOutput "$THIS_SCRIPT: ERROR Empty directory path name"
	writeOutput "Type $THIS_SCRIPT -h for Help"
	exit 1
fi
if [ -z "$ARGPKG" ]; then
	writeOutput "$THIS_SCRIPT: ERROR Empty package flag"
	writeOutput "Type $THIS_SCRIPT -h for Help"
	exit 1
fi

echo "$DIRPKG"
EDGNAME=`echo $DIRPKG | awk -F'/' '{print $NF}'`

# link the configuration and special installation files if they exist
INSTLIST=`ls $INSTALL_EDG_TESTROOT/*install`
CLEANLIST=`ls $INSTALL_EDG_TESTROOT/*clean`
RESTORELIST=`ls $INSTALL_EDG_TESTROOT/*restore`

if [ ! -d $EDG_DISTRIB_RPMS ]
then
    echo "Cannot find $EDG_DISTRIB_RPMS directory !" 
    exit 1
fi
cd $EDG_DISTRIB_RPMS
echo "$THIS_SCRIPT Linking pkg files in $PWD"
for msrc in ${INSTLIST} ; do
    echo "$THIS_SCRIPT Linking $msrc"
    ln -s $msrc . >& /dev/null
done
for msrc in ${CLEANLIST} ; do
    echo "$THIS_SCRIPT Linking $msrc"
    ln -s $msrc . >& /dev/null
done
for msrc in ${RESTORELIST} ; do
    echo "$THIS_SCRIPT Linking $msrc"
    ln -s $msrc . >& /dev/null
done
cd $EXEDIR



ALLPKG="$ARGPKG"


writeOutput "################################################"
writeOutput "##########      STARTING $ACTION        ########"
writeOutput "################################################"

LDIR=`echo $DIRPKG | awk -F'/' '{print $NF}'`
if [ ! -z $LDIR ]
then
    LOGDIR=${LOGDIR}/${LDIR}
    if [ ! -d $LOGDIR ];  then
	mkdir $LOGDIR
    fi   
fi

# INSTALLING EDG PKG
if [ $ACTION = "install" ]; then

for pkg in $ALLPKG; do

	lista="$DIRPKG/${pkg}all"
        writeOutput "${PRETXT} fileexist $lista"  
        fileexist $lista

	if [ $? != 0 ]; then
	    writeOutput "${PRETXT} WARNING: Cannot find $lista"
	    continue	
	fi
	if [ ! -s $lista ]; then
	    writeOutput "${PRETXT} WARNING: file $lista has zero length"
	    continue	
	fi
        
	if [ ! -e $LOGDIR/$pkg ]; then
	    mkdir  $LOGDIR/$pkg
	fi
	PKGLOGDIR="${LOGDIR}/${pkg}"
        
	LOGFILE=${pkg}.${ACTION}.log
        export EDG_LOGFILE=$LOGFILE
	[ -e $LOGFILE ] && (rm $LOGFILE; touch $LOGFILE)
	
# init variables
        intdistrib=""
        intdone=""        
        iterate=1
	until [ $iterate = $MAXLOOP ] ; do

                echo
                writeOutput "################################################"
                writeOutput "##########          ITERATION $iterate        #########"
                writeOutput "################################################"

                writeOutput "############# classify-rpms : input=[$lista] output=[${pkg}${iterate}]"
		classify_rpms $lista ${pkg}${iterate}
                print_status  
		# Check if there is something that can be installed              
                if [ ! -s ${pkg}${iterate}.canbeupgraded ] && [ ! -s ${pkg}${iterate}.canbeinstalled ]; then
		    writeOutput "${PRETXT} $ACTION ==> cannot DO more than this...sorry!" 
		    break
		fi
		
                writeOutput "############# create-list : input=[${pkg}${iterate}]"
		create_list ${pkg}${iterate}
                print_status
                
                writeOutput "############# install-rpm : input=[${pkg}${iterate}, $ACTION]"
		install_rpm ${pkg}${iterate} ${ACTION}
                print_status
                
        	[ -e ./info${pkg}${iterate} ] && (cp ./info${pkg}${iterate} $PKGLOGDIR)

                if [ $CLEANFILE ]
                then
                    writeOutput "############# cleaning files ${pkg}${iterate}.*"
		    $CP ${pkg}${iterate}.failedinstall $FAILED_FILE
                    res=`$RM ${pkg}${iterate}.*`
                    print_status                   
                fi 

                [ $DEBUG ] && writeOutput "############# get-list : input=[${pkg}${iterate}, DISTRIBUTION]"
		get_list "DISTRIBUTION" info${pkg}${iterate}
                distribution=`echo $RPMLIST | egrep -v none`
                [ $iterate = 1 ] && intdistrib=`echo "$distribution" | sed -e 's/ *$//'` 
                [ $DEBUG ] && writeOutput "############# get-list : input=[${pkg}${iterate}, DONE]"
		get_list "DONE" info${pkg}${iterate}
                donelist=`echo $RPMLIST | egrep -v none`
                intdone=`echo "$intdone $donelist" | sed -e 's/ *$//'`

                compare_list "$intdone" "$intdistrib"
                if [ $? = 0 ] 
		then
		    writeOutput "${PRETXT} $ACTION completed"
		    mail_status "${PRETXT} $ACTION completed $DIRPKG ${pkg}"
		    break
		fi
                compare_list "$intdone" "$olddone"
                if [ $? = 0 ] 
		then
		    writeOutput "${PRETXT} $ACTION ==> cannot DO more than this...sorry!"
		    mail_status "${PRETXT} $ACTION uncomplete $DIRPKG ${pkg}\n~r $FAILED_FILE" 1
		    break
                fi
                olddone="$intdone"
                
                [ -e $FAILED_FILE ] && res=`$RM $FAILED_FILE`
                iterate=$((iterate+1))
	done
	
# Now record log files	
        writeOutput "################################################"
        writeOutput "##########       RECORD LOG FILES      #########"
        writeOutput "################################################"
        echo $olddone >> $LOGFILE 
	[ -e $LOGFILE ] && (mv $LOGFILE $PKGLOGDIR)
	[ $? = 0 ] && (writeOutput "${PRETXT}  Succesfully transferred $LOGFILE in $PKGLOGDIR") || \
                     (writeOutput "${PRETXT}  Cannot transfer $LOGFILE")
# move restore log if exists    
    	if [ -e ${PKGLOGDIR}/${pkg}.restore.log ];
	then
	    mv  ${PKGLOGDIR}/${pkg}.restore.log  ${PKGLOGDIR}/${pkg}.restore.log.OLD
	fi	
done
fi

# RESTORE
#
if [ $ACTION = "restore" ]; then
for pkg in $ALLPKG; do

	PKGLOGDIR="${LOGDIR}/${pkg}"
	LOGFILE=${pkg}.${ACTION}.log
        export EDG_LOGFILE=$LOGFILE

	[ -e $LOGFILE ] && (rm $LOGFILE; touch $LOGFILE)
#
        writeOutput "${PRETXT} starting iterations" 

        iterate=$((MAXLOOP-1))
        until [ $iterate = 0 ] ; do

            echo
            writeOutput "################################################"
            writeOutput "##########        ITERATION $iterate      #########"
            writeOutput "################################################"

            if [ -e  ${PKGLOGDIR}/info${pkg}${iterate} ]
            then
	    	$CP ${PKGLOGDIR}/info${pkg}${iterate} ./info${pkg}${iterate}
                writeOutput "############# install-rpm : input=[${pkg}${iterate}, $ACTION]"
	        install_rpm ${pkg}${iterate} ${ACTION}
                print_status

        	[ -e ./info${pkg}${iterate} ] && (cp ./info${pkg}${iterate} ${PKGLOGDIR}/infoRES${pkg}${iterate})
            else
                writeOutput "${PRETXT} Info file ${PKGLOGDIR}/info${pkg}${iterate} does not exist"
            fi
            iterate=$((iterate-1))
        done
        writeOutput "${PRETXT} $ACTION completed"
# Now record log files	
        writeOutput "################################################"
        writeOutput "##########       RECORD LOG FILES      #########"
        writeOutput "################################################"
	[ -e $LOGFILE ] && (mv $LOGFILE $PKGLOGDIR)
	[ $? = 0 ] && (echo "${PRETXT}  Succesfully transferred $LOGFILE in $PKGLOGDIR") || \
                     (echo "${PRETXT}  Cannot transfer $LOGFILE")
# move install log if exists    
    	if [ -e ${PKGLOGDIR}/${pkg}.install.log ];
	then
	    mv  ${PKGLOGDIR}/${pkg}.install.log  ${PKGLOGDIR}/${pkg}.install.log.OLD
	fi	
done
fi

echo "### END of $THIS_SCRIPT"
exit 0
#
#
