#!/bin/sh
######################################################################
#
# Authors: Andrea Formica   - Andrea.Formica@cea.fr
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
# Based on INFN Testgroup scripts
#
######################################################################
## Ex: InstallWEB.sh -d beta9 -p external -g RB -i install -c 0
#
# Create log directory for all info files
HOUR=`date`
EXEDIR=`pwd`

echo "Working in $EXEDIR"

# Load environment variables

# Working Directory for EDG Installation
# EDG_INSTALL=/home/appli/edg/1.2/INSTALL
[ ! -d $EDG_INSTALL ] && mkdir $EDG_INSTALL
cd $EDG_INSTALL

# Delete log file
[ -e $EDG_LOGFILE ] && $RM $EDG_LOGFILE

GET_WEBLIST=$EDG_TEST_FUNCTIONS/get_RPM_list-script.sh

# some variables
THIS_SCRIPT="InstallWEB.sh"
PRETXT="[${THIS_SCRIPT} ${HOUR}]:"

#
# Get Argument for Installation or Restoring
#
# Initialise
EDGNAME=""
VERPKG=""
EDGPKG=""
FROMWEB=1
DOCONFIG=
ACTION="none"
while getopts "g:p:d:i:wh" myopt; do
	case  "$myopt" in
		g)
			EDGNAME=$OPTARG
		;;
		d)
			VERPKG=$OPTARG
		;;
		p)
			EDGPKG=$OPTARG
		;;
		i)
			ACTION=$OPTARG
		;;
		w)
			FROMWEB=
		;;
		h)
			echo "Usage: $THIS_SCRIPT :"
                        echo "                    -d <version> [beta9,..]                  Default="
			echo "                    -p <pkg>     [external,edg,globus,..]    Default="
			echo "                    -g <name>    [WN, SE, UI, RB, CE]        Default="
			echo "                    -w           [Do not read list from WEB] Default=read list from WEB"
			echo "                    -i           [install | (requires -p option)" 
			echo "                                  restore | (requires -p option)" 
			echo "                                  config  | (no -p option required)"
			echo "                                  none     ]                 Default=none"
			exit 1
		;;
		*)
			echo "Try -h for help!"
			exit 1
		;;
	esac
done

[ ! -d $VERPKG ] && mkdir $VERPKG
[ ! -d $VERPKG/$EDGNAME ] && mkdir $VERPKG/$EDGNAME

writeOutput "################################################"
writeOutput "##########    STARTING $THIS_SCRIPT     ########"
writeOutput "################################################"

writeOutput "${PRETXT} Downloading lists for version $VERPKG and package $EDGPKG in group $EDGNAME"
writeOutput "${PRETXT} fileexist $GET_WEBLIST"
fileexist $GET_WEBLIST
print_status
if [ $? = 0 ] && [ $FROMWEB ]
then
    [ ! -e ${VERPKG}/${EDGNAME}/${EDGPKG}all ] && $GET_WEBLIST $EDGNAME ${EDGPKG}-${VERPKG} > ${VERPKG}/${EDGNAME}/${EDGPKG}all
else
    writeOutput "${PRETXT} Using dir  $EDG_INSTALL/${VERPKG}/${EDGNAME}  and package ${EDGPKG}" 
fi
if [ "$ACTION" = "install" ] || [ "$ACTION" = "restore" ]; then

    writeOutput "################################################"
    writeOutput "##########    STARTING InstallRPM.sh    ########"
    writeOutput "################################################"
    res=0
#
    writeOutput "${PRETXT} InstallRPM  -d $EDG_INSTALL/${VERPKG}/${EDGNAME} -p ${EDGPKG} -o $ACTION"
    . $EDG_TEST_FUNCTIONS/InstallRPM.sh -d $EDG_INSTALL/${VERPKG}/${EDGNAME} -p ${EDGPKG} -o $ACTION
    print_status
    res=$?
#
elif [ "$ACTION" = "config" ]; then

    writeOutput "################################################"
    writeOutput "##########    STARTING  ${EDGNAME}.config   ########"
    writeOutput "################################################"
    res=0
#
    . $INSTALL_EDG_TESTROOT/${EDGNAME}.config $EDG_TESTROOT/etc/install_tools_lists 
    print_status
    res=$?
else
    writeOutput "${PRETXT} ERROR: unknown action $ACTION!"
    exit 1
fi

echo "### END of $THIS_SCRIPT"
exit 0
#
#
