#!/usr/bin/env bash
######################################################################
#
# Authors: Rene Metery  -  rene.metery@c-s.fr 
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
#######################################################################

follow_progress()
{
    if test -z "$1" 
    then
	message 1 "$0: wrong number of arguments." > /dev/stderr
	exit 4
    fi
    $TAIL --follow=name $TMP_LOG_FILE 2>/dev/null | $GREP $1  &
    follow_pid=$!
    export follow_pid=$(($follow_pid - 1))
}

###################### check_step() ###########################
check_step()
{
 ### Count how many lines have the <step> tag that
 ### indicates the number of test
 NB=$($GREP -c '<step>' $DEF_FILE)
 $ECHO $NB
}


##################### check_title() ###########################
####################  Function for XML ########################
check_title()
{
 ### Count how many lines have the <title> tag that
 ### indicates the number of title
 NB=$($GREP -c '<title>' $DEF_FILE)
 $ECHO $(($NB - 1))
}


###################### grep_info() ############################
####################  Function for XML ########################
grep_info()
{
 # search info
 INFO_BEGIN_POSITION=$($GREP -n '<info>' $DEF_FILE | $CUT -d: -f1)
 INFO_END_POSITION=$($GREP -n '</info>' $DEF_FILE | $CUT -d: -f1)
 
 # get the selected step
 NB_LINES=$(($INFO_END_POSITION - $INFO_BEGIN_POSITION))
 $HEAD -n $(($INFO_END_POSITION - 1)) $DEF_FILE | $TAIL -n $(($NB_LINES - 1)) > $TMP_LOG_STEP_FILE
}


################### grep_info_title() #########################
###################  Function for XML #########################
grep_info_title()
{
 # grep the info
 grep_info
 
 # grep the title
 grep_selected_title
 
 # print the title
 $CAT $TMP_LOG_INFO_FILE
}


#################### grep_info_brf() ##########################
####################  Function for XML ########################
grep_info_brf()
{
 # clean the log file
 $ECHO " " > $TMP_LOG_INFO_FILE
 
 # grep all the titles
 NB_STEP=$(check_step)
 i=1
 while [ $i -le $NB_STEP ]
 do
      grep_selected_step $i
      grep_selected_title
      i=$(($i + 1))           
 done
 
 # print all the title separated with an '#'
 $CAT $TMP_LOG_INFO_FILE | $SED s/##*/#/
}


#################### grep_info_det() ##########################
####################  Function for XML ########################
grep_info_det()
{
 # clean the log file
 $ECHO " " > $TMP_LOG_INFO_FILE
  
 # grep all the det_info
 NB_STEP=$(check_step)
 i=1
 while [ $i -le $NB_STEP ]
 do
       grep_selected_step $i
       
       # search the det_info
       INFO_BEGIN_POSITION=$($GREP -n '<det_info>' $TMP_LOG_STEP_FILE | $CUT -d: -f1)
       INFO_END_POSITION=$($GREP -n '</det_info>' $TMP_LOG_STEP_FILE | $CUT -d: -f1)      
       
       # check if there is a title              
       grep '<title>' $TMP_LOG_STEP_FILE >/dev/null 2>&1
       if test $? = 0
       then
             $ECHO -e "\n#####" >> $TMP_LOG_INFO_DET_FILE
       fi		    
       
       # get the det_info
       NB_LINES=$(($INFO_END_POSITION - $INFO_BEGIN_POSITION))
       $HEAD -n $(($INFO_END_POSITION - 1)) $TMP_LOG_STEP_FILE | $TAIL -n $(($NB_LINES - 1)) >> $TMP_LOG_INFO_DET_FILE
       
       i=$(($i + 1))
 done
 $ECHO -e "\n#####" >> $TMP_LOG_INFO_DET_FILE
}


################ grep_selected_info_det() #####################
####################  Function for XML ########################
grep_selected_info_det()
{
 NB=$1 
 # search the info_det
 INFO_DET_POSITION=$($GREP -n '#####' $TMP_LOG_INFO_DET_FILE | $CUT -d: -f1)
 
 SELECTED_INFO_DET_BEGIN=$($ECHO $INFO_DET_POSITION | $CUT -d" " -f$1 | $CUT -d: -f1)
 SELECTED_INFO_DET_END=$($ECHO $INFO_DET_POSITION | $CUT -d" " -f$(($1 + 1)) | $CUT -d: -f1)      
 
 # get the 
 NB_LINES=$(($SELECTED_INFO_DET_END - $SELECTED_INFO_DET_BEGIN))
 $HEAD -n $(($SELECTED_INFO_DET_END - 2)) $TMP_LOG_INFO_DET_FILE | $TAIL -n $(($NB_LINES - 2))
}


################ grep_selected_command() ######################
grep_selected_command()
{
 # search the command
 COMMAND_BEGIN_POSITION=$($GREP -n '<command>' $TMP_LOG_STEP_FILE | $CUT -d: -f1)
 COMMAND_END_POSITION=$($GREP -n '</command>' $TMP_LOG_STEP_FILE | $CUT -d: -f1)
   
 # get the command
 NB_LINES=$(($COMMAND_END_POSITION - $COMMAND_BEGIN_POSITION))
 $ECHO $LANG_PATH > $TMP_LOG_CMD_FILE

 # get det_info for the header
 INFO_BEGIN_POSITION=$($GREP -n '<brf_info>' $TMP_LOG_STEP_FILE | $CUT -d: -f1)
 INFO_END_POSITION=$($GREP -n '</brf_info>' $TMP_LOG_STEP_FILE | $CUT -d: -f1)
 NB_INFO_LINES=$(($INFO_END_POSITION - $INFO_BEGIN_POSITION))
 INFO=$($HEAD -n $(($INFO_END_POSITION - 1)) $TMP_LOG_STEP_FILE | $TAIL -n $(($NB_INFO_LINES - 1)))

if [ $LANG_NAME = "bash" ] 
then
 $ECHO "echo -e \"\\n\\n\"" >> $TMP_LOG_CMD_FILE
 $ECHO "echo \"************************************************\"" >> $TMP_LOG_CMD_FILE
 $ECHO "echo \"***   Beginning Test\"" >> $TMP_LOG_CMD_FILE
 $ECHO "echo \"************************************************\"" >> $TMP_LOG_CMD_FILE
 $ECHO "echo \"***   $($ECHO $INFO)\"" >> $TMP_LOG_CMD_FILE
 $ECHO "echo \"************************************************\"" >> $TMP_LOG_CMD_FILE
 $ECHO "export RETVALUE=\$RET_DEFVAL" >> $TMP_LOG_CMD_FILE
fi
 $HEAD -n $(($COMMAND_END_POSITION - 1)) $TMP_LOG_STEP_FILE | $TAIL -n $(($NB_LINES - 1)) >> $TMP_LOG_CMD_FILE
if [ $LANG_NAME = "bash" ] 
then
 $ECHO "STATUS=\$?" >> $TMP_LOG_CMD_FILE
 $ECHO "echo \"************************************************\"" >> $TMP_LOG_CMD_FILE
 $ECHO "echo \"***   Test ending status is  :  \"\$STATUS" >> $TMP_LOG_CMD_FILE
 $ECHO "exit \$STATUS" >> $TMP_LOG_CMD_FILE
fi
}


############## grep_selected_pre_command() ####################
grep_selected_pre_command()
{
 # search the command
 $GREP '<precommand>' $TMP_LOG_STEP_FILE > /dev/null 2>&1
 if ! test "$?" = 0
 then
       $ECHO "echo -n \".\"" > $TMP_LOG_CMD_FILE
       return 0
 fi
 PRE_COMMAND_BEGIN_POSITION=$($GREP -n '<precommand>' $TMP_LOG_STEP_FILE | $CUT -d: -f1)
 PRE_COMMAND_END_POSITION=$($GREP -n '</precommand>' $TMP_LOG_STEP_FILE | $CUT -d: -f1)
   
 # get the command
 NB_LINES=$(($PRE_COMMAND_END_POSITION - $PRE_COMMAND_BEGIN_POSITION))
 $ECHO $LANG_PATH > $TMP_LOG_CMD_FILE
 $ECHO "echo \"\"" >> $TMP_LOG_CMD_FILE
 $HEAD -n $(($PRE_COMMAND_END_POSITION - 1)) $TMP_LOG_STEP_FILE | $TAIL -n $(($NB_LINES - 1)) >> $TMP_LOG_CMD_FILE
 #$ECHO "echo -n \"......\"" >> $TMP_LOG_CMD_FILE
}
       

################## grep_selected_step() #######################
grep_selected_step()
{
 # search all steps
 STEP_BEGIN_POSITION=$($GREP -n '<step>' $DEF_FILE)
 STEP_END_POSITION=$($GREP -n '</step>' $DEF_FILE)
   
 # search the selected step
 SELECTED_BEGIN_STEP=$($ECHO $STEP_BEGIN_POSITION | $CUT -d" " -f$1 | $CUT -d: -f1)
 SELECTED_END_STEP=$($ECHO $STEP_END_POSITION | $CUT -d" " -f$1 | $CUT -d: -f1)
      
 # get the selected step
 NB_LINES=$(($SELECTED_END_STEP - $SELECTED_BEGIN_STEP))
 $HEAD -n $(($SELECTED_END_STEP - 1)) $DEF_FILE | $TAIL -n $(($NB_LINES - 1)) > $TMP_LOG_STEP_FILE
}


################ grep_selected_brf_info() #####################
grep_selected_brf_info()
{
 # searh the brf_info
 INFO_BEGIN_POSITION=$($GREP -n '<brf_info>' $TMP_LOG_STEP_FILE | $CUT -d: -f1)
 INFO_END_POSITION=$($GREP -n '</brf_info>' $TMP_LOG_STEP_FILE | $CUT -d: -f1)
   
 # get the brf_info
 NB_LINES=$(($INFO_END_POSITION - $INFO_BEGIN_POSITION))
 INFO=$($HEAD -n $(($INFO_END_POSITION - 1)) $TMP_LOG_STEP_FILE | $TAIL -n $(($NB_LINES - 1)))
      
 # print the det_info
 $ECHO $INFO
}


################ grep_selected_det_info() #####################
grep_selected_det_info()
{
 # search the det_info
 INFO_BEGIN_POSITION=$($GREP -n '<det_info>' $TMP_LOG_STEP_FILE | $CUT -d: -f1)
 INFO_END_POSITION=$($GREP -n '</det_info>' $TMP_LOG_STEP_FILE | $CUT -d: -f1)
 
 # get the det_info
 NB_LINES=$(($INFO_END_POSITION - $INFO_BEGIN_POSITION))
 $HEAD -n $(($INFO_END_POSITION - 1)) $TMP_LOG_STEP_FILE | $TAIL -n $(($NB_LINES - 1)) >> $TMP_LOG_INFO_FILE
 $ECHO "" >> $TMP_LOG_INFO_FILE
}

################ grep_selected_substatus_info() #####################
grep_selected_substatus_info()
{
# search the substatus
 $GREP '<substatus_info>' $TMP_LOG_STEP_FILE > /dev/null 2>&1 
 if ! test "$?" = 0
 then
       return 0
 fi
 # search the substatus_info
 INFO_BEGIN_POSITION=$($GREP -n '<substatus_info>' $TMP_LOG_STEP_FILE | $CUT -d: -f1)
 INFO_END_POSITION=$($GREP -n '</substatus_info>' $TMP_LOG_STEP_FILE | $CUT -d: -f1)
 
# get the substatus_info
 NB_LINES=$(($INFO_END_POSITION - $INFO_BEGIN_POSITION))
 INFO=$($HEAD -n $(($INFO_END_POSITION - 1)) $TMP_LOG_STEP_FILE | $TAIL -n $(($NB_LINES - 1)))
      
 # print the substatus_info
 $ECHO $INFO
}

################## grep_selected_title() ######################
grep_selected_title()
{
 # searh the title
 TITLE_BEGIN_POSITION=$($GREP -n '<title>' $TMP_LOG_STEP_FILE | $CUT -d: -f1)
 TITLE_END_POSITION=$($GREP -n '</title>' $TMP_LOG_STEP_FILE | $CUT -d: -f1)
 
 if ! test "$TITLE_BEGIN_POSITION" = "" || test "TITLE_BEGIN_POSITION" = ""
 then
       # get the title
       NB_LINES=$(($TITLE_END_POSITION - $TITLE_BEGIN_POSITION))
       $HEAD -n $(($TITLE_END_POSITION - 1)) $TMP_LOG_STEP_FILE | $TAIL -n $(($NB_LINES - 1)) >> $TMP_LOG_INFO_FILE
			  
       $ECHO "" >> $TMP_LOG_INFO_FILE
       return 0
 else
       return 1
 fi
}


###################### insert_title() #########################
insert_title()
{
 ! test -e $EDG_TEST_CONF_FILE && source $EDG_TEST_CONF_FILE
 grep_selected_title
 if test $? = 0
 then
       TITLE="$($TR "\n" " " < $TMP_LOG_INFO_FILE)"
       
       HTML_echo "######## $TITLE"
       HTML_dump first_col_end.html
       HTML_dump col_start.html
       HTML_dump col_end.html
       HTML_dump row_end.html
       HTML_dump row_start.html
       HTML_dump first_col_start.html
 fi
}


###################### print_info() ###########################
print_info()
{
 STEP_NUM=$1
 NB_STEP=$(check_step)
 if test -n "$STEP_NUM"
 then
       ### Print brief info about the selected step
       grep_selected_step $STEP_NUM
       insert_title
       $ECHO -n "Step $STEP_NUM "
       grep_selected_brf_info
 else
       ### Print all detailed infos
       i=1
       while [ $i -le $NB_STEP ]
       do
           grep_selected_step $i
	   grep_selected_title
	   grep_selected_det_info $i
	   i=$(($i + 1))
       done
       $CAT $TMP_LOG_INFO_FILE
 fi
}

