#!/bin/bash
#$Id: checkCron.sh,v 1.1.1.2 2005/02/08 16:09:44 mbarroso Exp $
######################################################################
#
# Authors: Frederique.Chollet   - Frederique.Chollet@lapp.in2p3.fr
#
# Copyright 2001-2002-2003 EU DataGrid project (2003), WP6 TestBed
#                    http://www.edg.org/license.html
#
#
#  Modified : Apr.08 03 TSTG
#            Frederique Chollet  chollet@lapp.in2p3.fr
#             
#            This test makes use of sudo if EDG_TEST_USER is not set to root
#	     In this case, EDG_TEST_USER must be run by a permitted 
#            user specified in the sudoers file
######################################################################

##################################################################
# FUNCTION: checkcron <cronjob>
#
# ARGUMENTS: <cronjob> {string} (mandatory)
#
# DESCRIPTION: check if <cronjob> is in the root crontab
#
# RETURN VALUE: 0 - if <service> is running 
#               1 - if <service> is not running 
#               4 - no mandatory parameter is passed
##################################################################

checkcron()
{
   if [ -z $1 ]
   then
        [ $DEBUG ] && $ECHO "checkcron(): no mandatory parameter is passed"
	return 4
   fi

   local status

   # first check in /etc/crontab

  
   if [ -e /etc/crontab ]
   then

        status=`$CAT /etc/crontab | $GREP $1 > /dev/null ; echo $?`

	if [ $status = 0 ]
	then
   		[ $DEBUG ] && $ECHO "checkcron(): entry for cron job $1 found in /etc/crontab"
		return 0
	fi

   fi
   
   # second check with 'crontab -l' - needs privileges 
   if ! test "$EDG_TEST_USER" = "root" 
   then
   # see if we can use $SUDO
	which $SUDO > /dev/null 2>&1
	if [ $? = 1 ]
	then
	    [ $DEBUG ] && $ECHO "checkcron(): $SUDO is not installed in this computer" 1>&2
	    return 5
	fi
   else
	if ! test `whoami` = "$EDG_TEST_USER"
	then
	    [ $DEBUG ] && $ECHO "checkcron(): should be run as EDG_TEST_USER="$EDG_TEST_USER
	    return 4
	fi
   fi

   if ! test "$EDG_TEST_USER" = "root" 
   then
	status=`$SUDO -p "checkcron(): you are not a permitted user for this test" -S $CRONTAB -l -u root </dev/null | $GREP $1 | $WC -l `   
   else 
	status=`$CRONTAB -l -u root </dev/null | $GREP $1 | $WC -l `
   fi

   if [ $status = 0 ]
   then
       [ $DEBUG ] && $ECHO "checkcron(): $1 not found in root crontab"
       return 1
   else
       [ $DEBUG ] && $ECHO "checkcron(): $1 found in root crontab"
       return 0
   fi
  
}
