#!/bin/sh
#
######################################################################
#
# Authors: Andrea Formica   - Andrea.Formica@cea.fr
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
# Based on INFN Testgroup scripts
#
######################################################################

##################################################################
# FUNCTION: dolist-install <flag> <list> 
#           dolist-restore   "       "
#
# ARGUMENTS: <flag>    {string} (mandatory)
#            <list>    {string} 
#
# Input:  <list>              Write out a list of EDG packages marked by <flag>
#                             and write the list to the output file indicated     
#                             by <output file>. <list> parameter can be empty
#
#         <flag>              {install: Uvh,ivh} {restore: }
#
# RETURN VALUE: 0 - list 
#               1 - file not found
#	        4 - no mandatory parameter is passed
#
##################################################################
#
dolist_install()
{

    PRESTRING="${PRESTRING}+"
    local THIS_SCRIPT="dolist-install.sh"
    local HOUR=`date | awk '{print $4}'`
    local PRETXT="[${THIS_SCRIPT} ${HOUR}]:"
    
    if [ $LEVEL -gt 1 ]
    then
        PRETXT="$PRESTRING [${THIS_SCRIPT} ${HOUR}]:"
    fi

    if [ -z $1 ]
    then
        [ $DEBUG ] && writeOutput "dolist-install(): no mandatory parameter is passed"
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: input  $1={flag}"
        echo
        PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
	return 4
    fi

    
    local FLAG="$1"
    local mylist="$2"
    local res
    local status=0

    RPMLIST=""

    if [ "$mylist" = "" ]; then
	writeOutput "${PRETXT} Empty list"
    else
        for rpm in $mylist; do
#            echo "RPM === " $rpm
            irpm=$EDG_DISTRIB_RPMS/$rpm
            NAME=`rpm --queryformat "%{NAME}" -qp $irpm 2>&1`

            INSTALL_FILE=`echo $NAME | awk -F'-' '{print $1}'`
            INSTALL_SCRIPT=$EDG_DISTRIB_RPMS/${INSTALL_FILE}.install
            CLEAN_SCRIPT=$EDG_DISTRIB_RPMS/${INSTALL_FILE}.clean

            CLEAN_FUNC=${INSTALL_FILE}-clean
            INST_FUNC=${INSTALL_FILE}-install
            
            [ $DEBUG ] && writeOutput "${PRETXT} fileexist  $CLEAN_SCRIPT" 
            fileexist $CLEAN_SCRIPT
            if [ $? = 0 ]
            then
                $CLEAN_FUNC; [ $? != 0 ] && status=$[ status + 1 ]
#            print_status
            fi

            [ $DEBUG ] && writeOutput "${PRETXT} fileexist  $INSTALL_SCRIPT" 
            fileexist $INSTALL_SCRIPT
            if [ $? = 0 ] && [ ! -e $INSTALL_SCRIPT.${NODE}.done ]; then
                [ $LEVEL -gt 0 ] && writeOutput "${PRETXT} Special installation for $irpm...skip checks"
                $INST_FUNC
            elif [ $? = 0 ] && [ -e $INSTALL_SCRIPT.${NODE}.done ]; then
                [ $LEVEL -gt 0 ] && writeOutput "${PRETXT} Special installation already done for $irpm"
            else
                [ $LEVEL -gt 0 ] && writeOutput "${PRETXT} install $rpm with $FLAG"
                res=`rpm -$FLAG $ISTEST $irpm > /dev/null 2>&1`
            fi
            if [ $? = 0 ]
            then
                RPMLIST="$RPMLIST $rpm"
                print_status
            else
                [ $LEVEL -gt 0 ] && writeOutput "${PRETXT} ALERT in install $rpm with $FLAG"
                status=$[ status + 1 ] 
            fi
        done    
    fi
# Check the status
    if [ $status -ge 1 ]
    then
        PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
        return 1        
    fi

    [ $DEBUG ] && writeOutput "${PRETXT} ### END" 
    PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
    
    return 0
}

