#!/bin/sh
#
######################################################################
#
# Authors: Andrea Formica   - Andrea.Formica@cea.fr
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
# Based on INFN Testgroup scripts
#
######################################################################

##################################################################
# FUNCTION: dolist-install <flag> <list> 
#           dolist-restore   "       "
#
# ARGUMENTS: <flag>    {string} (mandatory)
#            <list>    {string} 
#
# Input:  <list>              Write out a list of EDG packages marked by <flag>
#                             and write the list to the output file indicated     
#                             by <output file>. <list> parameter can be empty
#
#         <flag>              {install: Uvh,ivh} {restore: }
#
# RETURN VALUE: 0 - list 
#               1 - file not found
#	        4 - no mandatory parameter is passed
#
##################################################################
#
dolist_restore()
{

    PRESTRING="${PRESTRING}+"
    local THIS_SCRIPT="dolist-restore.sh"
    local HOUR=`date | awk '{print $4}'`
    local PRETXT="[${THIS_SCRIPT} ${HOUR}]:"
    
    if [ $LEVEL -gt 1 ]
    then
        PRETXT="$PRESTRING [${THIS_SCRIPT} ${HOUR}]:"
    fi

    if [ -z $1 ]
    then
        [ $DEBUG ] && writeOutput "dolist-restore(): no mandatory parameter is passed"
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: input  $1={flag}"
        PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
	return 4
    fi

    
    local FLAG="$1"
    local mylist="$2"
    local type=0
    local res
    local status=0

    local isdone=""

    RPMLIST=""

    if [ $FLAG = "Uvh" ]
    then
        FLAG="Uvh --oldpackage" 
        type=0
    elif [ $FLAG = "ivh" ]
    then
        FLAG="e"     
        type=1
    else
        [ $DEBUG ] && writeOutput "dolist-restore(): wrong input $1"
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: input  $1={flag = Uvh, ivh}"
        PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
	return 2
    fi

    if [ "$mylist" = "" ]; then
	[ $DEBUG ] && writeOutput "${PRETXT} Empty list"
        PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
        return 1
    else
        for rpm in $mylist; do
#            echo "RPM === " $rpm
            irpm=$EDG_DISTRIB_RPMS/$rpm
            NAME=`rpm --queryformat "%{NAME}" -qp $irpm 2>&1`
            RNAME=`rpm -qp $irpm`
            
            isdone=`echo $donelist | egrep $rpm`
            [ $? != 0 ] && writeOutput "${PRETXT} ERROR file $rpm is not DONE ==> SKIP RESTORE" \
                        && continue
            
            if [ $type = 0 ]
            then
                getpkg-list "$NAME" "$RH62_DIS_RPMS_DIR" "$original"
                irpm="$RH62_DIS_RPMS_DIR/$RPMNAME"
            else
                irpm=$RNAME
            fi

            INSTALL_FILE=`echo $NAME | awk -F'-' '{print $1}'`
            INSTALL_SCRIPT=$EDG_DISTRIB_RPMS/${INSTALL_FILE}.install
            RESTORE_SCRIPT=$EDG_DISTRIB_RPMS/${INSTALL_FILE}.restore
            CLEAN_SCRIPT=$EDG_DISTRIB_RPMS/${INSTALL_FILE}.clean

            CLEAN_FUNC=${INSTALL_FILE}-clean            
            REST_FUNC=${INSTALL_FILE}-restore
                                   
            [ $DEBUG ] && writeOutput "${PRETXT} fileexist  $INSTALL_SCRIPT" 
            fileexist $RESTORE_SCRIPT
            if [ $? = 0 ] && [ ! -e $RESTORE_SCRIPT.${NODE}.done ]; then
                [ $LEVEL -gt 0 ] && writeOutput "${PRETXT} Special restoring for $irpm...skip checks"
                $REST_FUNC $irpm
            elif [ $? = 0 ] && [ -e $RESTORE_SCRIPT.${NODE}.done ]; then
                [ $LEVEL -gt 0 ] && writeOutput "${PRETXT} Special restoring already done for $irpm"
                $REST_FUNC $irpm
            else
                [ $LEVEL -gt 0 ] && writeOutput "${PRETXT} restore $rpm with $FLAG"
                res=`rpm -$FLAG $ISTEST $irpm > /dev/null 2>&1`
            fi
            if [ $? = 0 ]
            then
                RPMLIST="$RPMLIST $rpm"
                print_status
            else
                [ $LEVEL -gt 0 ] && writeOutput "${PRETXT} ALERT in restore $rpm with $FLAG"
                status=$[ status + 1 ] 
            fi
            
            [ $DEBUG ] && writeOutput "${PRETXT} fileexist $CLEAN_SCRIPT" 
            fileexist $CLEAN_SCRIPT
            if [ $? = 0 ]
            then
                $CLEAN_FUNC $CLEANARG ; [ $? != 0 ] && status=$[ status + 1 ]
#            print_status
            fi
        done    
    fi

# Check the status
    if [ $status -ge 1 ]
    then
        PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
        return 1        
    fi

    [ $DEBUG ] && writeOutput "${PRETXT} ### END" 
    PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
    
    return 0
}
