#!/bin/bash
######################################################################
#
# Authors: Gennaro Tortone   - Gennaro.Tortone@na.infn.it
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
######################################################################
# Modified : F.Chollet 19/03/03 test if file exists added
##################################################################
# FUNCTION: getfileparam <variable> <file> <value>
#
# ARGUMENTS: <variable> {string} (mandatory)
#	     <file> {string} (mandatory)
#            <value> [string] (optional)
#
# DESCRIPTION: check if <variable> is defined inside <file> 
#	       and returns value; optional check if value is correctly set
#
# RETURN VALUE: 0 - if <variable> is defined (value in output)
#		1 - if <variable> is not defined
#	        4 - no mandatory parameter is passed
#               5 - untested because file doest not exist
##################################################################

getfileparam()
{
   if [ -z $1 ]
   then
        [ $DEBUG ] && $ECHO "getfileparam(): no mandatory parameter is passed" 1>&2
	return 4
   fi

   if [ -z $2 ]
   then
        [ $DEBUG ] && $ECHO "getfileparam(): no mandatory parameter is passed" 1>&2
	return 4
   fi

   if [ -e $2 ]
   then 
	source $2
   else 
	[ $DEBUG ] && $ECHO "getfileparam(): file passed does not exist" 1 >&2
	return 5
   fi	
  
   local varname=$1
   eval local varvalue=$`$ECHO $1`

   if [ "$varvalue" ]
   then
        if [ "$3" ]
        then
        	if [ "$3" = "$varvalue" ]
		then
			 [ $DEBUG ] && $ECHO "getfileparam(): variable is defined in $2 ($varname = $varvalue) and equal to $3" 1>&2
        		 $ECHO "$varvalue"
        		 return 0
		else
			[ $DEBUG ] && $ECHO "getfileparam(): variable is defined in $2 ($varname = $varvalue) NOT equal to $3" 1>&2
                         $ECHO "$varvalue"
                         return 1
		fi
	fi

	[ $DEBUG ] && $ECHO "getfileparam(): variable is defined in $2 ($varname = $varvalue)" 1>&2
	$ECHO "$varvalue" 
	return 0
   else
	[ $DEBUG ] && $ECHO "getfileparam(): variable $varname is NOT defined!" 1>&2
   	$ECHO ""	
	return 1
   fi
}
