#!/bin/bash
######################################################################
#
# Authors: Miquel Barcelo  - barcelo@ifae.es
#	   Gonzalo Merino  - merino@ifae.es
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
######################################################################

##################################################################
# FUNCTION: getLogEnd_GDMP <logid> <server> <port> <vo> <logfile>
#
# ARGUMENTS: <logid>   {string} (mandatory)
#            <server>  {string} (mandatory)
#            <port>    {string} (mandatory)
#            <vo>      {string} (mandatory)
#            <logfile> {string} (optional)
#
# DESCRIPTION: checks if a process on GDMP server has ended
#              using Log ID 
#                      
# RETURN VALUE: 0 - if process has ended
#		1 - if still running / cannot check status
#	        4 - no mandatory parameter is passed
##################################################################

getLogEnd_GDMP()
{
   if [ -z "$1" ] || [ -z "$2" ] || [ -z "$3" ] || [ -z "$4" ]
   then
        message 1 "getLogEnd_GDMP(): no mandatory parameter is passed"
	return 4
   fi    

   if [ -z "$5" ]
   then
       local TMPLOGFILE=command.log
   else
       local TMPLOGFILE="$5"
   fi
   
   local OLD_VERBOSE_LEVEL=$VERBOSE_LEVEL
   runCommand "gdmp_job_status -L $1 -S $2 -P $3 -V $4" $TMPLOGFILE
   local RET=$?
   
   if [ $VERBOSE_LEVEL -le 1 ] 
   then
	VERBOSE_LEVEL=0
   fi
   
   VERBOSE_LEVEL=$OLD_VERBOSE_LEVEL
   
   local rc=`grep "END LOGGING OUTPUT" $TMPLOGFILE | wc -l`

   if [ -z "$5" ]
   then
       rm $TMPLOGFILE
   fi

   if [ $RET -ne 0 ]
   then
       message 1 "getLogEnd_GDMP(): Failed to get process status"
       return 1
   fi

   if [ $rc = 2 ]
   then
       message 1 "getLogEnd_GDMP(): Process $1 has ended"
       return 0
   else
       message 1 "getLogEnd_GDMP(): Still waiting process $1 to end"
       return 1
   fi

}
