#!/bin/bash
######################################################################
#
# Authors: Miquel Barcelo  - barcelo@ifae.es
#	   Gonzalo Merino  - merino@ifae.es
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
##################################################################

##################################################################
# FUNCTION: getLogID_GDMP <logfile> <returnvar>
#
# ARGUMENTS: <logfile>    {string} (mandatory)
#            <returnvar>  {string} (mandatory)
#
#
# DESCRIPTION: gets Log ID from an asynchronous GDMP command output
#                       
#
# RETURN VALUE: 0 - if could get Log ID 
#		1 - if could not get an Log ID 
#	        4 - no mandatory parameter is passed
##################################################################

getLogID_GDMP()
{
   if [ -z "$1" ] || [ -z "$2" ]
   then
        message 1 "getLogID_GDMP(): no mandatory parameter is passed"
	return 4
   fi    

   local logid
   logid=`grep "Server Log ID=" $1 | awk -F "=" '{print $2}'| awk '{print $1}'`

   if [ -z "$logid" ]
   then
       message 1 "getLogID_GDMP(): Command did not return a Log ID"
       return 1
   else
       message 1 "getLogID_GDMP(): Log ID is $logid"
       eval $2=$logid
       return 0
   fi

}

