#!/bin/bash
######################################################################
#
# Authors: F.Chollet          - chollet@lapp.in2p3.fr
#                               
# from the orginal getfileparamtail function written by
# Mario Reale     -      Mario.Reale@cnaf.infn.it
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
# Modified : Apr.08 03 TSTG
#            Frederique Chollet  chollet@lapp.in2p3.fr
#             
#            This test makes use of sudo if EDG_TEST_USER is not set to root
#	     In this case, EDG_TEST_USER must be run by a permitted 
#            user specified in the sudoers file
######################################################################
# 
##################################################################
# FUNCTION: getpfileparamtail <filename>
#
# ARGUMENTS: <filename> {string} (mandatory)
#            <query string> (string) (mandatory)
#
# DESCRIPTION: check if a parameter is defined inside a file.
#               if multiple lines found, last line ("tail") is
#               considered
#
# RETURN VALUE: 0 - if <query string> found in <filename>
#               1 - if <query string> not fond in <filename>
#               4 - no mandatory parameter is passed
#               5 - untested if EDG_TEST_USER not root and $SUDO uninstalled
##################################################################


getpfileparamtail()
{ 
   if ! test "$EDG_TEST_USER" = "root" 
   then
   # see if we can use $SUDO
	which $SUDO > /dev/null 2>&1
	if [ $? = 1 ]
	then
	    [ $DEBUG ] && $ECHO "getpfileparamtail(): $SUDO is not installed in this computer" 1>&2
	    return 5
	fi
   else
	if ! test `whoami` = "$EDG_TEST_USER"
	then
	    [ $DEBUG ] && $ECHO "checkuserenv(): should be run as EDG_TEST_USER="$EDG_TEST_USER
	    return 4
	fi
   fi

   if [ -z $1 ]
   then
        $ECHO "getpfileparamtail(): no mandatory parameter is passed"
        return 4
   fi

   if [ -z $2 ]
   then
        $ECHO "getpfileparamtail(): no mandatory parameter is passed" 
        return 4
   fi


  what=$2;
  where=$1;

  if ! test "$EDG_TEST_USER" = "root" 
  then  
      a=`$SUDO -p "getpfileparamtail(): you are not a permitted user for this test" -S $GREP $what $where </dev/null | awk 'BEGIN { FS < $what }; $0 ~ // {print $2}'  | head -1`
  else
      a=`$GREP $what $where </dev/null | awk 'BEGIN { FS < $what }; $0 ~ // {print $2}'  | head -1`
  fi

   if [ "$a" ]
         then
         b=`expr length $a`
        
        if  [ "$b" -eq 1 ]
	    then
	    if ! test "$EDG_TEST_USER" = "root" 
		then  
		a=`$SUDO -p "getpfileparamtail(): you are not a permitted user for this test" -S $GREP $what $where </dev/null | awk 'BEGIN { FS < $what }; $0 ~ // {print $3}' | tail -1`
	    else
		a=`$GREP $what $where </dev/null | awk 'BEGIN { FS < $what }; $0 ~ // {print $3}' | tail -1`
	    fi	
       	
             b=`expr length $a`;
         fi

         c=`expr index $a =`
         d=`expr $b - $c`
         f=`expr $c + 1`
         e=`expr substr $a $f $b`
        	$ECHO "getpfileparamtail(): param " $2 "=" $e 
	        return 0
       else
        	$ECHO "getpfileparamtail(): param "$2"  is NOT defined in file "  $1 " !"
   	        return 1

     fi

}
