#!/bin/bash

######################################################################
#
# Authors: Mario Reale    -  Mario.Reale@cnaf.infn.it
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
# Modified 15/01/03 TSTG F.Chollet chollet@lapp.in2p3.fr
#                        Check EDG_LOCATION may be used to access rb.conf
#                        Force stdout redirection and the return value
######################################################################


##################################################################
# FUNCTION: getportnumber_rb_conf <variable>
#
# ARGUMENTS: <variable> {string} (mandatory)
#
# DESCRIPTION: gets <service> port number as defined in /opt/edg/etc/rb.conf
#	       and returns the value of the corresponding port number
#
# RETURN VALUE: 0 - if <service> is defined (value in output)
#		1 - if <service> is not defined
#               4 - if no mandatory parameter is passed  
####################################################################################


function getportnumber_rb_conf( ) {

 if [ -z $1 ]
   then
        [ $DEBUG ] && $ECHO "getportnumber_rb_conf(): no mandatory parameter is passed" 1>&2
	return 4
   fi

 if [ -z "$EDG_LOCATION" ]
   then
	 [ $DEBUG ] && $ECHO "getportnumber_rb_conf(): EDG_LOCATON is undefined." 1>&2  
	 if fileexist "/etc/profile.d/edg.sh"; then 
	    source /etc/profile.d/edg.sh
	    [ $DEBUG ] && $ECHO "EDG_LOCATION has been defined by sourcing /etc/profile.d/edg.sh." 1>&2 
	 fi
 fi	

 a=`cat $EDG_LOCATION/etc/rb.conf | grep $1 | awk ' BEGIN { FS = "=" } $0 ~ //  {print $2}' | awk  ' BEGIN { FS = ";" } $0 ~ //  {print $1}'`

   if [ "$a" ]
   then
        [ $DEBUG ] && $ECHO -e "getportnumber_rb_conf(): according to $EDG_LOCATION/etc/rb.conf  $1 should run on port $a"    1>&2
        $ECHO "$a"
        return 0
   else
		[ $DEBUG ] && $ECHO "getportnumber_rb_conf(): service $1 not found in $EDG_LOCATION/etc/rb.conf " 1>&2
        $ECHO ""
        return 1
   fi


}

