#!/bin/bash

######################################################################
#
# Authors: Alessandro Cavalli  - cavalli@cnaf.infn.it
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
######################################################################

######################################################################
# FUNCTION: grisqueryse <host>
#
# ARGUMENTS: <host> {string} (optional)
#
# DESCRIPTION: Queries the local GRIS or the one specified with <host>
#              of a StorageElement and checks if it's responding and
#              if it returns complete data.
#
# RETURN VALUE: 0 - if everything is OK.
#               1 - if required data is missin
#               4 - if the GRIS doesn't respond
######################################################################

grisqueryse()
{

    OPTION=$1

    HOST=${OPTION:-`/bin/hostname`}
    PORT=2135
    DN="mds-vo-name=local,o=grid"

    SEIDOK=no
    SEPROTOK=no
    SESTATOK=no
    MISSING=no
    MISSING_MAY=no

    SE_ATTR_LIST=`echo "SEId CloseCE SEtypearchitecture SEsize SEResourceContactString SEvo" | tr 'A-Z' 'a-z'`

    SE_ATTR_LIST_MAY=`echo "SESubjectName SEmaxfilesize SEmaxdata SEmaxnumfile" | tr 'A-Z' 'a-z'`

    SE_PROT_ATTR_LIST=`echo "SEProtocol SEId" | tr 'A-Z' 'a-z'`

    SE_PROT_ATTR_LIST_MAY=`echo "Port" | tr 'A-Z' 'a-z'`

    SE_STAT_ATTR_LIST=`echo "SEfreespace" | tr 'A-Z' 'a-z'`

    SE_STAT_ATTR_LIST_MAY=`echo "SEId SEaccesstime SEfilequeuelen" | tr 'A-Z' 'a-z'`

    echo

    grid-info-search -LLL -h ${HOST} -p ${PORT} -b "${DN}" -x | tr 'A-Z' 'a-z' > /tmp/$$_all 2>&1

    ERR=`grep -i "ldap_bind: Can't contact LDAP server" /tmp/$$_all`

    if [ "$ERR" = "ldap_bind: Can't contact LDAP server" ]
	then
	echo "GRIS isn't responding at ${HOST}:${PORT}/${DN} "
	echo
	rm -f /tmp/$$_all
	return 4
    fi


    while read attribute value
      do
      cat /dev/null > /tmp/$$_entry

      if [ "$attribute" = "dn:" ] ; then
      	
	  if [ "`echo $value | cut -c1-5`" = "seid=" ] ; then
	      echo "Analizyng seId: " `echo ${value} |sed -e "s/,.*//g" `
	      SEIDOK=yes

	      while read attribute value
		do
		echo ${attribute} ${value} >> /tmp/$$_entry

		if [ -z "${attribute}" -a -z "${value}" ] ; then
               
		    for attr in ${SE_ATTR_LIST} ; do
			if [ -z  "`grep -i ${attr}: /tmp/$$_entry | sed -n -e 1p | sed -e \"s/.*: //g\"`" ] ; then
			    echo "Attribute '${attr}' not defined"
			    MISSING=yes
			fi
		    done

		    for attr in ${SE_ATTR_LIST_MAY} ; do
			if [ -z  "`grep -i ${attr}: /tmp/$$_entry | sed -n -e 1p | sed -e \"s/.*: //g\"`" ] ; then
			    echo "Optional attribute '${attr}' not defined"
			    MISSING_MAY=yes
			fi
		    done
               
		    echo
		    continue 2
		fi
	      done
	  fi

	  if [ "`echo $value | cut -c1-5`" = "sepro" ] ; then
	      echo "Analizyng SeProtocol: ${value}"
	      SEPROTOK=yes
	      PROT_FILE=no

	      while read attribute value
		do
		echo ${attribute} ${value} >> /tmp/$$_entry

		if [ "${attribute}" = "seprotocol:" -a "${value}" = "file" ] ; then
		    PROT_FILE=yes
		fi

		if [ -z "${attribute}" -a -z "${value}" ] ; then

		    for attr in ${SE_PROT_ATTR_LIST} ; do
			if [ -z  "`grep -i ${attr}: /tmp/$$_entry | sed -n -e 1p | sed -e \"s/.*: //g\"`" ] ; then
			    echo "Attribute '${attr}' not defined"
			    MISSING=yes
			fi
		    done

		    for attr in ${SE_PROT_ATTR_LIST_MAY} ; do
			if [ -z  "`grep -i ${attr}: /tmp/$$_entry | sed -n -e 1p | sed -e \"s/.*: //g\"`" ] ; then
			    if [ "${PROT_FILE}" != "yes" ] ; then
				echo "Optional attribute '${attr}' not defined"
				MISSING_MAY=yes
			    fi
			fi
		    done
               
		    echo
		    continue 2
		fi
	      done
	  fi

	  if [ "`echo $value | cut -c1-5`" = "in=st" ] ; then
	      echo "Analizyng SeStatus: ${value}"
	      SESTATOK=yes
 
	      while read attribute value
		do
		echo ${attribute} ${value} >> /tmp/$$_entry
 
		if [ -z "${attribute}" -a -z "${value}" ] ; then
 
		    for attr in ${SE_STAT_ATTR_LIST} ; do
			if [ -z  "`grep -i ${attr}: /tmp/$$_entry | sed -n -e 1p | sed -e \"s/.*: //g\"`" ] ; then
			    echo "Attribute '${attr}' not defined"
			    MISSING=yes
			fi
		    done
 
		    for attr in ${SE_STAT_ATTR_LIST_MAY} ; do
			if [ -z  "`grep -i ${attr}: /tmp/$$_entry | sed -n -e 1p | sed -e \"s/.*: //g\"`" ] ; then
			    echo "Optional attribute '${attr}' not defined"
			    MISSING_MAY=yes
			fi
		    done
 
		    echo
		    continue 2
		fi
	      done
	  fi
      fi
    done < /tmp/$$_all

    rm -f /tmp/$$_all
    rm -f /tmp/$$_entry

    if [ "${SEIDOK}${SEPROTOK}${SESTATOK}${MISSING}${MISSING_MAY}" = "yesyesyesnono" ] ; then
        echo "The GRIS for the Storage Element is up"
        echo "and data returned is complete"
	echo
	return 0
    fi

    if [ "${MISSING}${MISSING_MAY}" = "noyes" ] ; then
        echo "The GRIS for the Storage Element is up"
        echo "and all mandatory attributes are defined"
	echo "But one or more OPTIONAL attributes are not defined"
	echo
	return 0
    fi

    if [ "${SEIDOK}" = "no" ] ; then
	echo "ERROR: no SEID data returned from GRIS."
    fi

    if [ "${SEPROTOK}" = "no" ] ; then
	echo "ERROR: no SeProtocol data returned from GRIS."
    fi

    if [ "${SESTATOK}" = "no" ] ; then
	echo "ERROR: no SeStatus data returned from GRIS."
    fi

    if [ "${MISSING}" = "yes" ] ; then
	echo "ERROR: missing attribute(s)"
    fi

    echo
    return 1
}
