#!/bin/sh
#
######################################################################
#
# Authors: Andrea Formica   - Andrea.Formica@cea.fr
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
# Based on INFN Testgroup scripts
#
######################################################################

##################################################################
# FUNCTION: get-list <flag1> <flag2> <input file> <line>
#
# ARGUMENTS: <flag1>    {string} (mandatory)
#            <flag2>    {string} (mandatory)
#            <input file> {string} (mandatory)
#            <line> {string} (mandatory)
#
# Input:  <flag>          Replace all lines between <flag1> and <flag2>
#                         with <line> 
#                             
#
#
# RETURN VALUE: 0 - list 
#               1 - file not found
#	        4 - no mandatory parameter is passed
#
##################################################################
#
replace()
{

    local THIS_SCRIPT="replace.sh"
    local HOUR=`date | awk '{print $4}'`
    local PRETXT="$PRESTRING [${THIS_SCRIPT} ${HOUR}]:"

    if [ -z $1 ]
    then
        [ $DEBUG ] && writeOutput "replace(): no mandatory parameter is passed"
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: input  $1={flag}"
        echo
	return 4
    fi
    if [ -z $2 ]
    then
        [ $DEBUG ] && writeOutput "replace(): no mandatory parameter is passed"
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: input  $2={flag}"
        echo
	return 4
    fi
    if [ -z $3 ]
    then
        [ $DEBUG ] && writeOutput "replace(): no mandatory parameter is passed"
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: input  $3={input file}"
        echo
	return 4
    fi
    if [ -z "$4" ]
    then
        [ $DEBUG ] && writeOutput "replace(): no mandatory parameter is passed"
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: input  $4={line}"
        echo
	return 4
    fi

    
    local fl=$1
    local fm=$2
    local INST_INFO_FILE=$3
    local lline=$4
    
#    commonenv
    [ $DEBUG ] && writeOutput "${PRETXT} fileexist $INST_INFO_FILE" 
    fileexist $INST_INFO_FILE
    [ $? != 0 ] && return 1

    firstline=`cat $INST_INFO_FILE | \\
               egrep -n "$fl" | head -1 | awk -F':' '{print $1}'`
    lastline=`cat $INST_INFO_FILE | \\
               egrep -n "$fm" | head -1 | awk -F':' '{print $1}'`
    nlines=`wc $INST_INFO_FILE | awk '{print $1}'`
    cmd="1,${firstline}p"
    cat $INST_INFO_FILE | sed -n $cmd > $INST_INFO_FILE.head
    $ECHO "$lline" >> $INST_INFO_FILE.head
    cmd="${lastline},${nlines}p"
    cat $INST_INFO_FILE | sed -n $cmd >> $INST_INFO_FILE.head
    mv $INST_INFO_FILE.head $INST_INFO_FILE

    return 0
}
