#!/bin/bash
#$Id: rpmCheckerNotInstalledList.sh,v 1.1.1.2 2005/02/08 16:09:44 mbarroso Exp $
######################################################################
#
# Authors: Mario Reale   - Mario.Reale@cnaf.infn.it
#          Flavia Donno  - Flavia.Donno@pi.infn.it
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
######################################################################

##################################################################
# FUNCTION: rpmchecker <filename>
#
# ARGUMENTS: <filename> {string} (mandatory)
#            <filelist-not-installed> (string)  (optional)
#
# DESCRIPTION: It checks if the list of rpms specified in <filename>
#              are installed on the machine 
#              It allows a set of RPMs specified in the file 
#              <filelist-not-installed> to be specied , so that
#              non finding these RMPs will not give an error code 
#              in any case
#
# RETURN VALUE: 0 - if all rpms are installed
#               1 - if any one of the specified rpms is not installed
#               4 - no mandatory parameter is passed or
#		    if the input file specified is empty
##################################################################

rpmcheckernilist()
{
  
   if [ -z $1 ]
   then
        $ECHO "rpmchecker(): no mandatory parameter is passed"
        return 4
   fi

   if [ -f $1 ]
   then

     nlines=`wc -l $1 |  awk '{print $1}'`
     errflg=0
     returval=0
     k=1
      until [ $k -gt $nlines ] ; do
            f=`sed -n ${k}p $1`
            k=$[ k + 1 ]
            rpm -q $f > /dev/null 2>&1
            if [ $? -eq 0 ]; then
                 [ $DEBUG ] && $ECHO "rpmchecker(): $f is installed"
                 errflg=0
            else
                errflg=1
                if [ $2 ]
                then

                   kf=1
                   nfk=`wc -l $2  |  awk '{print $1}'`
                   errflg=1 
                   until [ $kf -gt $nfk ]; do
                       ffli=`sed -n ${kf}p $2`
                      kf=$[ kf + 1 ] 
                       if [ "$ffli" = "$f" ]
                       then
                          errflg=0               
                        $ECHO "rpmcheckernilist(): $f is NOT  installed - but this is anyhow expected according to the list $2"
                       fi
                   done
                fi

                if [ $errflg -eq 1 ]; then
                   $ECHO " ================================== rpmchecker(): $f NOT INSTALLED !"
                   returval=1
                fi  


            fi
      done 
      

   else  
      [ $DEBUG ] && $ECHO "rpmchecker(): the input file specified is empty"
      return 4
   fi

#   $ECHO "return code = $returval"

   return $returval
}




