#!/bin/bash
######################################################################
#
# Authors: Miquel Barcelo  - barcelo@ifae.es
#	   Gonzalo Merino  - merino@ifae.es
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
######################################################################

##################################################################
# FUNCTION: runCommand <command> <logfile>
#
# ARGUMENTS: <command> {string} (mandatory)
#            <logfile> {string} (optional)
#
# DESCRIPTION: runs command, checks result and optionally log output
#
# RETURN VALUE: $? - return value of the called <command> 
#		1  - if <command> timeout
#               4  - if mandatory parameter is missing
#               
##################################################################
# Default timeout is 4 minutes. Can be changed by overwritting this 
# global variable in the caller script.
TIMINGOUT=240

runCommand()
{
   if [ -z "$1" ]
   then
        message 1 "runCommand(): no mandatory parameter is passed"
	return 4
   fi    

   message 2 "'*'"
   message 1 "runCommand(): Now running: \"$1\""
   if [ -z "$2" ]
   then
       message 2 "runCommand(): No logging"
       $EDG_TESTROOT/sbin/timeout-script.exp $TIMINGOUT "$1"
       local RETVAL=$?
   else
       local TMPLOGFILE=$2
       message 2 "runCommand(): Logging output to \"$TMPLOGFILE\""
       message 0 "'***************************************'" > $TMPLOGFILE
       message 0 "'*' Log for: $1" >> $TMPLOGFILE
       message 0 "'***************************************'" >> $TMPLOGFILE

       TMPFILE1=/tmp/runCommand_$$_1.tmp
       TMPFILE2=/tmp/runCommand_$$_2.tmp

       $EDG_TESTROOT/sbin/timeout-script.exp $TIMINGOUT "$1 &> $TMPFILE1" &> $TMPFILE2
       local RETVAL=$?
       cat $TMPFILE1 >> $TMPLOGFILE ; rm $TMPFILE1
       cat $TMPFILE2 >> $TMPLOGFILE ; rm $TMPFILE2
       message 0 " " >> $TMPLOGFILE
   fi

   if [ "$RETVAL" = 0 ]
   then
	message 2 "runCommand(): command ran succesfully"
   else
	message 1 "runCommand(): command failed"
   fi

   return $RETVAL
}




