#!/bin/bash
######################################################################
#
# Authors: Andrea Formica
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
######################################################################

##################################################################
# FUNCTION: testrpm <rpmname> <resfile> {upgrade|install}
#
# ARGUMENTS: <rpmname> {string} (mandatory)
#            <resfile> {string} (mandatory)
#            {Uvh|ivh}
#
# DESCRIPTION: check if a rpm can be upgraded or installed
#
# RETURN VALUE: 0 - if rpmname can be upgraded or installed
#		1 - if rpm cannot be upgraded or installed	
#	        4 - no mandatory parameter is passed, or rpmname does not exist
##################################################################

testrpm()
{
    if [ -z $1 ]
    then
	[ $DEBUG ] && writeOutput "testrpm(): no mandatory parameter is passed"
	return 4 
    fi
    if [ -z $2 ]
    then
	[ $DEBUG ] && writeOutput "testrpm(): no mandatory parameter is passed"
	return 4 
    fi
    if [ ! -e $1 ] 
    then
	return 4
    fi
    if [ "$3" != "Uvh" ] && [ "$3" != "ivh" ]
    then
        return 4 
    fi

    canstring="canbeinstalled"
    failedstring="failedinstall"
    if [ "$3" = "Uvh" ] 
    then
        canstring="canbeupgraded"
        failedstring="failedupgrade"
    fi

    MSG=`rpm -$3 --test $1 2>&1`
    if [ $? = 0 ]; then
        $ECHO $rpm >> $2.$canstring
        return 0
    else
	$ECHO "" >> $2.$failedstring
	$ECHO $rpm >> $2.$failedstring
	$ECHO -e "  Reason:" $MSG >> $2.$failedstring
        return 1
    fi
     

}
