#!/bin/sh
#
######################################################################
#
# Authors: Andrea Formica   - Andrea.Formica@cea.fr
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
# Based on INFN Testgroup scripts
#
######################################################################

##################################################################
# FUNCTION: write-list <flag> <output file> <list> 
#
# ARGUMENTS: <list>    {string} 
#            <flag>    {string} (mandatory)
#            <output file> {string} (mandatory)
#
# Input:  <list>              Write out a list of EDG packages marked by <flag>
#                             and write the list to the output file indicated     
#                             by <output file>. <list> parameter can be empty
#
#
# RETURN VALUE: 0 - list written to output
#               1 - file not found
#	        4 - no mandatory parameter is passed
#
##################################################################
#
write_list()
{

    PRESTRING="${PRESTRING}+"
    local THIS_SCRIPT="write_list.sh"
    local HOUR=`date | awk '{print $4}'`
    local PRETXT="[${THIS_SCRIPT} ${HOUR}]:"
    
    if [ $LEVEL -gt 1 ]
    then
        PRETXT="$PRESTRING [${THIS_SCRIPT} ${HOUR}]:"
    fi

    if [ -z $1 ]
    then
        [ $DEBUG ] && writeOutput "write_list(): no mandatory parameter is passed"
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: input  $1={flag}"
	return 4
    fi
    if [ -z $2 ]
    then
        [ $DEBUG ] && writeOutput "write_list(): no mandatory parameter is passed"
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: input  $2={output file}"
	return 4
    fi

    
    local FLAG="$1"
    local INST_INFO_FILE=$2
    local mylist="$3"

    [ $DEBUG ] && writeOutput "${PRETXT} fileexist $INST_INFO_FILE" 
    fileexist $INST_INFO_FILE

    [ $? != 0 ] && (PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`; return 1)

    echo "[$FLAG]" >> $INST_INFO_FILE

    if [ "$mylist" = "" ]; then
	echo "none" >> $INST_INFO_FILE
    else
        for rpm in $mylist; do
#            echo "adding RPM ==> " $rpm
	    echo $rpm >> $INST_INFO_FILE
        done    
    fi

    echo "[END-$FLAG]" >> $INST_INFO_FILE

    [ $DEBUG ] && writeOutput "${PRETXT} ### END" 
    PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
    
    return 0
}
