#!/usr/bin/env bash
######################################################################
#
# Authors: Rene Metery  -  rene.metery@c-s.fr 
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
#######################################################################
# Last Modification : 21/03/03 F.Chollet
#######################################################################
# Requires :
#       - The EDG_TESTROOT environment variable being defined
#       - getopt (installed by default on most systems)
#
##################################################################
#
 usage()
 {
    message 0 "Usage: $NAME"
    message 0 "       $NAME [-h] -n"
    message 0 "       $NAME [-h] -i [-s int]"
    message 0 "       $NAME [-h] [-s int] [-l string]"
    message 0 " "
    message 0 "Without arguments, the script runs the entire test (all steps)."
    message 0 "Options/Arguments:"
    message 0 " "
    message 0 "       -h/--help           Shows help."
    message 0 "       -n/--numstep        Prints the number of selectable steps on the test."
    message 0 "       -s/--step           Run only the specified step, not the entire test."
    message 0 "       -i/--info           Prints brief info about the selected step using ""-s""."
    message 0 "                           Prints detailed info about the entire test if no"
    message 0 "                           step selected."
    message 0 "       -l/--logfile        Log test results to this file. By default is dumped"
    message 0 "                           to stdout."
 }
#
# DESCRIPTION:  User Interface installation & configuration tests.
#               
#
# RETURN VALUE: 0 PASSED
#	 	1 FAIL
#		2 WARNING
#		3 XFAIL
#		4 UNRESOLVED
#		5 UNTESTED
#		6 UNSUPPORTED
#               
##################################################################


# We should take a look for an $EDG_TESROOT/etc/edg-test.conf or 
# ~/.edg-tsft/edg-test.conf for configuration file

if test -z "$EDG_TESTROOT"
then
      $ECHO "EDG_TESTROOT environment variable must be defined!"
      exit 4
fi    


. $EDG_TEST_FUNCTIONS/sourcer.sh
. $EDG_TESTROOT/tests/InstallConfig/CE/sourcer.sh
sourcetestsiteconffile

export DEBUG=1 # debug mode on
# unset DEBUG   #debug mode off

export DEF_FILE=$EDG_TESTROOT/tests/InstallConfig/WN/WN_config_test.def

export TMP_LOG_STEP_FILE=$EDG_TEST_TMP/WN_cfg_step$$.tmp
export TMP_LOG_INFO_FILE=$EDG_TEST_TMP/WN_cfg_info$$.tmp
export TMP_LOG_CMD_FILE=$EDG_TEST_TMP/WN_cfg_cmd$$.tmp

###################### check_step() ###########################
check_step()
{
 ### Count how many lines have the <step> tag that
 ### indicates the number of test
 NB=$(grep -c '<step>' $DEF_FILE )
 $ECHO $NB
}

################ grep_selected_command() ######################
grep_selected_command()
{
 # search the command
 COMMAND_BEGIN_POSITION=$(grep -n '<command>' $TMP_LOG_STEP_FILE | cut -d: -f1)
 COMMAND_END_POSITION=$(grep -n '</command>' $TMP_LOG_STEP_FILE | cut -d: -f1)

 # get the command
 NB_LINES=$(($COMMAND_END_POSITION - $COMMAND_BEGIN_POSITION))
 $HEAD -n $(($COMMAND_END_POSITION - 1)) $TMP_LOG_STEP_FILE | $TAIL -n $(($NB_LINES - 1)) > $TMP_LOG_CMD_FILE
}


################## grep_selected_step() #######################
grep_selected_step()
{
 # search all steps
 STEP_BEGIN_POSITION=$(grep -n '<step>' $DEF_FILE)
 STEP_END_POSITION=$(grep -n '</step>' $DEF_FILE)

 # search th selected step
 SELECTED_BEGIN_STEP=$(echo $STEP_BEGIN_POSITION | cut -d" " -f$1 | cut -d: -f1)
 SELECTED_END_STEP=$(echo $STEP_END_POSITION | cut -d" " -f$1 | cut -d: -f1)

 # get the selected step
 NB_LINES=$(($SELECTED_END_STEP - $SELECTED_BEGIN_STEP))
 $HEAD -n $(($SELECTED_END_STEP - 1)) $DEF_FILE | $TAIL -n $(($NB_LINES - 1)) > $TMP_LOG_STEP_FILE
}


################ grep_selected_brf_info() #####################
grep_selected_brf_info()
{
 # searh the brf_info
 INFO_BEGIN_POSITION=$(grep -n '<brf_info>' $TMP_LOG_STEP_FILE | cut -d: -f1)
 INFO_END_POSITION=$(grep -n '</brf_info>' $TMP_LOG_STEP_FILE | cut -d: -f1)
 
 # get the brf_info
 NB_LINES=$(($INFO_END_POSITION - $INFO_BEGIN_POSITION))
 INFO=$($HEAD -n $(($INFO_END_POSITION - 1)) $TMP_LOG_STEP_FILE | $TAIL -n $(($NB_LINES - 1)))

 # print the det_info
 $ECHO $INFO
}


################ grep_selected_det_info() #####################
grep_selected_det_info()
{
 NB=$1
 # searh the det_info
 INFO_BEGIN_POSITION=$(grep -n '<det_info>' $TMP_LOG_STEP_FILE | cut -d: -f1)
 INFO_END_POSITION=$(grep -n '</det_info>' $TMP_LOG_STEP_FILE | cut -d: -f1)
 
 # check if there is a title
 INFO=$($HEAD -n $(($INFO_END_POSITION - 1)) $TMP_LOG_STEP_FILE | $TAIL -n $(($NB_LINES - 1)))
 $ECHO $INFO | grep '#########' >/dev/null 2>&1
 if ! test $? = 0
 then
       $ECHO -e "Step $1:" >> $TMP_LOG_INFO_FILE
 fi

 # get the det_info
 NB_LINES=$(($INFO_END_POSITION - $INFO_BEGIN_POSITION))
 $HEAD -n $(($INFO_END_POSITION - 1)) $TMP_LOG_STEP_FILE | $TAIL -n $(($NB_LINES - 1)) >> $TMP_LOG_INFO_FILE
 $ECHO "" >> $TMP_LOG_INFO_FILE
}


################## grep_selected_title() ######################
grep_selected_title()
{
 # searh the title
 TITLE_BEGIN_POSITION=$(grep -n '<title>' $TMP_LOG_STEP_FILE | cut -d: -f1)
 TITLE_END_POSITION=$(grep -n '</title>' $TMP_LOG_STEP_FILE | cut -d: -f1)
 
 if ! test "$TITLE_BEGIN_POSITION" = "" || test "TITLE_BEGIN_POSITION" = ""
 then
       # get the title
       NB_LINES=$(($TITLE_END_POSITION - $TITLE_BEGIN_POSITION))
       $HEAD -n $(($TITLE_END_POSITION - 1)) $TMP_LOG_STEP_FILE | $TAIL -n $(($NB_LINES - 1)) >> $TMP_LOG_INFO_FILE

       $ECHO "" >> $TMP_LOG_INFO_FILE
       return 0
 else
       return 1
 fi
}


###################### insert_title() #########################
insert_title()
{
# source $EDG_TEST_CONF_FILE 
 grep_selected_title
 if test $? = 0
 then
       TITLE="$($TR "\n" " " < $TMP_LOG_INFO_FILE)"       
       
       HTML_echo "######## $TITLE"
       HTML_dump first_col_end.html
       HTML_dump col_start.html
       HTML_dump col_end.html
       HTML_dump row_end.html
       HTML_dump row_start.html
       HTML_dump first_col_start.html
 fi
}


###################### print_info() ###########################
print_info()
{
 STEP_NUM=$1
 NB_STEP=$(check_step)
 if test -n "$STEP_NUM"
 then
       ### Print brief info about the selected step
       grep_selected_step $STEP_NUM
       insert_title
       $ECHO -n "Step $STEP_NUM "
       grep_selected_brf_info
 else
       ### Print all detailed infos
       i=1
       while [ $i -le $NB_STEP ]
       do
           grep_selected_step $i
           grep_selected_title
           grep_selected_det_info $i
           i=$(($i + 1))
       done
       $CAT $TMP_LOG_INFO_FILE
 fi 
}

trace_status() 
# trace the returned status or the status passed as argument 
# in the log file
# in the compound final status RETVALUE   
{
 ret=$?
 if [ ! -z "$1" ]; then
   ret=$1 
 fi
 case "$ret" in
 "$RET_PASSED" | "$RET_OK" ) 
	 echo_success >> $TMP_LOG_FILE 2>&1
#        echo_passed could be used as well
	 ;;
 "$RET_FAIL" )
	 echo_failure >> $TMP_LOG_FILE 2>&1
	 ;;
 "$RET_WARNING" )	 
	 echo_warning >> $TMP_LOG_FILE 2>&1
 	 ;;	 
 "$RET_UNTESTED" )
         echo_untested >> $TMP_LOG_FILE 2>&1
	 ;;
 "$RET_XFAIL" )
         echo_xfail >> $TMP_LOG_FILE 2>&1
	 ;;
       * )
         echo_unresolved  >> $TMP_LOG_FILE 2>&1
	 ;;
 esac

 if [ $ret -lt $RETVALUE ] && [ $ret -ne $RET_PASSED ]; then RETVALUE=$ret ;fi 
}

return_status()
{
   if [ $RETVALUE = $RET_DEFVAL ]; then RETVALUE=$RET_PASSED; fi             
   return $RETVALUE
}

export -f trace_status
export -f return_status

####################### do_test() #############################
do_test()
{
    ### Check function argument
    if test -z "$1" 
    then
	message 1 "$0: wrong number of arguments."
	exit 4
    fi
 
    ### Log file initialization
 
    if [ -z "$2" ] ; then
        export TMP_LOG_FILE=.tmplog_$PPID
    else
        export TMP_LOG_FILE=$2.new
    fi    
   
    ### Exit code initialization -cf testRetCodes.sh
    ### export RET_PASSED=0 is done ther
    export RET_OK=0
    export RET_ERROR=1
    export RET_WARNING=2
    export RET_XFAIL=3
    export RET_UNRESOLVED=4
    export RET_UNTESTED=5
    export RET_UNSUPPORTED=6
    export RET_DEFVAL=7
    export RETURN_CODE=$RET_DEFVAL
    export RETVALUE=$RET_DEFVAL
    grep_selected_step $1
    grep_selected_command
    chmod 700 $TMP_LOG_CMD_FILE

    [ $DEBUG ] && $ECHO "----DEBUG set----------------------------------------"
    [ $DEBUG ] && $ECHO $0

    $ECHO "----Testing `hostname`, Step $1                                      " > $TMP_LOG_FILE
    $ECHO -n "----" >> $TMP_LOG_FILE 2>&1
    grep_selected_brf_info >> $TMP_LOG_FILE 2>&1
    $TMP_LOG_CMD_FILE 
    RETURN_CODE=$?

    if [ -z $2 ] ; then 
#	format the file into columns delimited with character %  (cf echoFunc.sh)  
	cat $TMP_LOG_FILE | column -s % -t 
    else
	cat $TMP_LOG_FILE | column -s % -t  >> $2
    fi
    rm -f $TMP_LOG_FILE

    return $RETURN_CODE
}




################# Parse the command line ######################
PROG_NAME="`/bin/basename $0`"

ARGS_SHORT="hins:l:"
ARGS_LONG="help,info,numstep,step:,logfile:"

GETOPT_OUT=`$GETOPT -o $ARGS_SHORT --long $ARGS_LONG -n $PROG_NAME -- "$@"`

if ! test $? = 0 
then 
      $ECHO "Try \`WN_config_test.sh -h' for more informations."
      exit 4
fi

eval set -- "$GETOPT_OUT"
while true ; do
    case "$1" in
	-h|--help)      HELP=1                   ;;
	-i|--info)      INFO=1                   ;;
	-n|--numstep)   NB_STEP=1                ;;
	-s|--step)      SEL_STEP=$2      ; shift ;;
	-l|--logfile)   LOGOUT=$2        ; shift ;;
	--)             shift            ; break ;;
	*)              $ECHO "Internal error!" ; exit 4 ;;
    esac
    shift
done


### Show the help
if test -n "$HELP"
then
      usage
      exit 0
fi
  
### Show the total number of steps
if test -n "$NB_STEP"
then   
    NB_TEST=$(check_step)
    $ECHO "$NB_TEST"
    exit 0
fi

### Show infos about steps
if test -n "$INFO"
then  
    print_info "$SEL_STEP"
    $RM -f $TMP_LOG_INFO_FILE
    $RM -f $TMP_LOG_STEP_FILE
    exit 0
fi     


######################### Main ################################
RETVAL=0
RETVALNEW=0
NB_TEST=$(check_step)
rm -f $LOGOUT

if test -z "$SEL_STEP" 
then
       ### Launch all tests
       k=1
       while [ $k -le $NB_TEST ]
       do
           do_test $k $LOGOUT
           RETVALNEW=$?
           if [ $RETVAL -ne 1 ] ; then RETVAL=$RETVALNEW ; fi 
           k=$(($k + 1))
       done
else
      ### Launch selected test
      do_test $SEL_STEP $LOGOUT
      RETVAL=$?
fi    

$RM -f $TMP_LOG_CMD_FILE
$RM -f $TMP_LOG_INFO_FILE
$RM -f $TMP_LOG_STEP_FILE

exit $RETVAL
