package EDG_Job;
use Time::Local;

sub new {
    my $self = shift;
    my $class = ref($self) || $self;
    my $jdl = shift;
    my $jobid = shift;
    my $job = bless {}, $class;

# Initialize all the class variables to the null string
    $job->{jdl} = '';
    $job->{jobid} = '';
    $job->{status} = '';
    $job->{status_reason} = '';
    $job->{last_update_time} = '';
    $job->{reached_on} = '';
    $job->{status_enter_time} = '';
    $job->{location} = '';
    $job->{owner} = '';
    $job->{submit_error} = '';
    $job->{status_error} = '';
    $job->{get_output_error} = '';

# The constructor takes as arguments the JDL and optionally the job ID
    $job->{jdl} = $jdl if (defined $jdl);
    $job->{jobid} = $jobid if (defined $jobid);
    return $job;
}

sub jdl {
    my $job = shift;
    my $jdl = $job->{jdl};
    return $jdl;
}

sub jobid {
    my $job = shift;
    my $jobid = $job->{jobid};
    return $jobid;
}

sub status {
    my $job = shift;
    my $status = $job->{status};
    return $status;
}

sub status_reason {
    my $job = shift;
    my $status_reason = $job->{status_reason};
    return $status_reason;
}

sub last_update_time {
    my $job = shift;
    my $time = $job->{last_update_time};
    $time = &compact_time($time) if ( defined $time  and $time ne "" );
    return $time;
}

sub reached_on {
    my $job = shift;
    my $time = $job->{reached_on};
    $time = &compact_time($time) if ( defined $time  and $time ne "" );
    return $time;
}

sub status_enter_time {
    my $job = shift;
    my $time = $job->{status_enter_time};
    $time = &compact_time($time) if ( defined $time and $time ne "" );
    return $time;
}

sub location {
    my $job = shift;
    my $location = $job->{location};
    return $location;
}

sub owner {
    my $job = shift;
    my $owner = $job->{owner};
    return $owner;
}

sub submit_error {
    my $job = shift;
    my $submit_error = $job->{submit_error};
    return $submit_error;
}

sub status_error {
    my $job = shift;
    my $status_error = $job->{status_error};
    return $status_error;
}

sub get_output_error {
    my $job = shift;
    my $get_output_error = $job->{get_output_error};
    return $get_output_error;
}

sub submit {
    my $job = shift;
    my $jdl = $job->{jdl};
    my $jdl_file = "/tmp/$$.jdl";
    my $submit_error;
    my @cargs = ();
    my %args = ( @_ );
    my @keys = keys %args;
    foreach my $key ( @keys ) {
	if ( $key eq 'notify' ) {
	    push @cargs, "--notify $args{$key} ";
	} elsif ( $key eq 'hours' ) {
	    my $dir = $args{$key};
	    push @cargs, "--hours $args{$key} ";
	} elsif ( $key eq 'config' ) {
	    my $dir = $args{$key};
	    push @cargs, "--config $args{$key} ";
	} elsif ( $key eq 'output' ) {
	    my $dir = $args{$key};
	    push @cargs, "--output $args{$key} ";
	} elsif ( $key eq 'vo' ) {
	    my $dir = $args{$key};
	    push @cargs, "--vo $args{$key} ";
	}
    }
    open(JDL,">$jdl_file");
    print JDL $jdl;
    close(JDL);
    my @submit_out = `edg-job-submit @cargs $jdl_file`;
    unlink $jdl_file;
# Here derive job id
    foreach ( @submit_out ) {
	chomp;
	if ( /https/ ) {
	    my $jobid = $_;
	    $jobid =~ s/.*https/https/;
	    $job->{jobid} = $jobid;
	} elsif ( /^\*{4} Error: / ) {
	    my $submit_error = $_;
	    $submit_error =~ s/^\*{4} Error: //;
	    $submit_error =~ s/ \*{4}\s*$//;	    
	    $job->{submit_error} = $submit_error;
	}
    }
    return @submit_out;
}

sub update_status {
    my $edg_version = 2;
    my $job = shift;
    my $jobid = $job->jobid;
    my @status_out = `edg-job-status $jobid`;
    foreach ( @status_out ) {
	chomp;
#	if ( $edg_version == 1 ) {
#	    my ($key, $value) = split /\s+=\s+/, $_, 2;
#	} elsif ( $edg_version == 2 ) {
	    my ($key, $value) = split /\s*:\s*/, $_, 2;
#	}
	if ( defined $key and defined $value ) {
	    if ( "$key " eq 'Status ' ) {
		$job->{status} = $value;
	    } elsif ( "$key " eq 'Current Status ' ) {
		$job->{status} = $value;
	    } elsif ( $key eq 'reached on' ) {
		$job->{reached_on} = $value;
	    } elsif ( $key eq 'Last Update Time (UTC)' ) {
		$job->{last_update_time} = $value;
	    } elsif ( $key eq 'Status Reason' ) {
		$job->{status_reason} = $value;
	    } elsif ( $key eq 'Job Owner' ) {
		$job->{owner} = $value;
	    } elsif ( $key eq 'Status Enter Time (UTC)' ) {
		$job->{status_enter_time} = $value;
	    } elsif ( $key eq 'Destination' ) {
		$job->{location} = $value;
	    } elsif ( $key eq 'Job Destination' ) {
		$job->{location} = $value;
	    }
	}
	if ( /^\*{4} Error: / ) {
	    $status_error = $_;
	    $status_error =~ s/^\*{4} Error: //;
	    $status_error =~ s/ \*{4}\s*$//;
	    $job->{status_error} = $status_error;
	}
    }
    return @status_out;
}

sub get_output {
    my $job = shift;
    my $jobid = $job->jobid;
    my @cargs = ();
    my %args = ( @_ );
    my @keys = keys %args;
    foreach my $key ( @keys ) {
	if ( $key eq 'input' ) {
	    push @cargs, "--input $args{$key} ";
	} elsif ( $key eq 'dir' ) {
	    my $dir = $args{$key};
	    push @cargs, "--dir $args{$key} ";
	}
    }
    my @get_output = `edg-job-get-output @cargs $jobid`;
    foreach ( @get_output ) {
	chomp;
	if ( /^\*{4} Error: / ) {
	    $get_output_error = $_;
	    $get_output_error =~ s/^\*{4} Error: //;
	    $get_output_error =~ s/ \*{4}\s*$//;
	    $job->{get_output_error} = $get_output_error;
	}
    }
    return @get_output;
}

sub compact_time {
    my %months = ('Jan', '01', 'Feb', '02', 'Mar', '03', 'Apr', '04', 'May', '05', 'Jun', '06', 'Jul', '07', 'Aug', '08', 'Sep', '09', 'Oct', 10, 'Nov', 11, 'Dec', 12);
    my $time = shift;
    my ($wday, $mon, $day, $hour, $year) = split /\s+/, $time;
    $mon = $months{$mon};
    $time = "$day-$mon-$year $hour";
    return $time;
}    
1;
