# Define the widths of the fields in the job database
open(INC, "prod.inc") or die "Cannot find prod.inc";
@fields = %width = ();
while (<INC>) {
    my ($var, $width) = split;
    $width{"$var"} = $width;
    push @fields, $var;
}
close INC;

%width = ('Run'           => 10,
	  'Jobid'         => 100,
	  'SubErr'        => 40,
	  'Status'        => 15,
	  'StatErr'       => 40,
	  'Reason'        => 60,
	  'LUT'           => 20,
	  'SET'           => 20,
	  'Location'      => 60,
	  'Host'          => 30,
	  'WallClockTime' => 10,
	  'OutputSE'      => 30,
	  'StorDir'       => 50,
	  'LFN'           => 100
	  );

# Field names
@fields = ('Run', 'Jobid', 'SubErr', 'Status', 'StatErr', 'Reason', 'LUT', 'SET', 'Location', 'Host', 'WallClockTime', 'OutputSE', 'StorDir', 'LFN');

# Pack/unpack string
$pack_string = '';
foreach ( @fields ) {
    $pack_string .= "A$width{$_} ";
}

# Initialize the memory resident job database, i.e. the hashes
# %Jobid, %Status, etc.
@runs = ();
foreach (@fields) {
    %$_ = () unless (/Run/);
}
1;

# Loads into memory the job database
sub jobdb_load {
    open(DB, "$database") or die "Cannot open the job database\n";
    @runs = ();
    my @f = @fields;
    shift @f;
    while (<DB>) {
	next if (/^#/);
	my @record = unpack($pack_string, $_);
	my $run = shift @record;
	push @runs, $run;
	foreach my $f (@f) {
	    my $value = shift @record;
	    $$f{$run} = $value;
	}
    }
    close DB;
}

# Saves the job database from memory to file
sub jobdb_save {
    open(DB, ">$tmpfile") or die "Cannot open the job database\n";

# Copy any line beginning with a # in the old database (if any)
    if (-r $database) {
	open(OLDDB,"$database") or die "Cannot open the old job database\n";
	while (<OLDDB>) {
	    print DB $_ if (/^#/);
	}
	close OLDDB;
    }
    foreach my $run (@runs) {
	my @record = ();
	push @record, $run;
	my @f = @fields;
	shift @f;
	foreach my $field (@f) {
	    if (defined $$field{$run}) {
		push @record, $$field{$run};
	    } else {
		push @record, '';
	    }
	}
	print DB pack($pack_string, @record), "\n";
    }
    close DB;
    rename $tmpfile, $database;
}

# Print to stdout the status summary for a given run/jobid
sub status_update {
    my $run = shift;
    my $jobid = shift;
    my $job = EDG_Job->new("", $jobid);
    $job->update_status;
    $Status{$run} = $job->status;
    $StatErr{$run} = $job->status_error;
    $Reason{$run} = $job->status_reason;
    $LUT{$run} = $job->last_update_time;
    $SET{$run} = $job->status_enter_time;
    $Location{$run} = $job->location;
}
