#!/bin/sh

# Define some variables
HOST=`hostname`
WDIR=`pwd`
echo "insert Host $HOST"                                      # Host

# Source the card file
. ./*.cards

# Name arguments
RUN=$1
DATA="${DATA_NAME}${RUN}.dat"

# Run the fake event generator and calculate the wall clock time
chmod +x Test.exe
starttime=`date +%s`
./Test.exe $EVENTS $DATA_SIZE
mv dummy_data $DATA
endtime=`date +%s`
difftime=`expr $endtime - $starttime`
echo "insert WallClockTime $difftime"                         # WallClockTime

# Copy locally the input data, if required
if [ -n "$INPUT_DATA" -a "$INPUT_DATA" != 'NO' ] ; then
    INPUT="${INPUT_DATA}${RUN}.dat"
    edg-replica-manager-listReplicas -l $INPUT -c $RC_CONFIG_FILE > ./replicas
    CLOSE_SE=`edg-brokerinfo getCloseSEs | head -1`
    LOCAL_REPLICA=`cat ./replicas | grep $CLOSE_SE | cut -d: -f2- | sed 's/^ //'`
    if [ -n "$LOCAL_REPLICA" ] ; then
	REPLICA=$LOCAL_REPLICA
    else
	REPLICA=`cat ./replicas | grep '^location 1' | cut -d: -f2-`
    fi
    if [ -z "$REPLICA" ] ; then
	echo "ERROR: NO_REPLICA_FOR_INPUT_DATA" >&2
	exit 1
    else
	echo "globus-url-copy gsiftp://$REPLICA file://${HOST}${WDIR}/input"
	globus-url-copy gsiftp://$REPLICA file://${HOST}${WDIR}/input
	if [ -r ${WDIR}/input ] ; then
	    echo "Input file copy succeeded!"
	fi
    fi
fi

# Store data in a SE
if [ "$COPY2SE" = YES ] ; then

# Check that the RM commands are there
    which edg-replica-manager > /dev/null 2> /dev/null
    if [ ! $? ] ; then
	echo "ERROR: RM_MISSING" >&2
	exit 1
    fi
    which edg-rc > /dev/null 2> /dev/null
    if [ ! $? ] ; then
	echo "ERROR: RC_MISSING" >&2
	exit 1
    fi
    which edg-gridftp-ls > /dev/null 2> /dev/null
    if [ ! $? ] ; then
	echo "ERROR: GRIDFTP_MISSING" >&2
	exit 1
    fi

# Use the (first) close SE
    if [ "$OUTPUT_SE" = 'CLOSE' ] ; then
	SE=`edg-brokerinfo getCloseSEs | head -1`
	if [ -z "$SE" ] ; then
	    echo "ERROR: NO_CLOSE_SE" >&2
	    echo "insert OutputSE NO_CLOSE_SE"
	    exit 1
	else
	    OUTPUT_SE=$SE
	fi
    fi
    echo "insert OutputSE $OUTPUT_SE"

# Find the storage directory
    STORAGE_DIR=`ldapsearch -H ldap://${OUTPUT_SE}:2135 -x -b 'Mds-Vo-Name=local,o=grid' 'objectclass=StorageElement' | grep "SEvo: $VO" | cut -d: -f3`
# If fails, try with the GLUE schema
    if [ -z "$STORAGE_DIR" ] ; then
	STORAGE_DIR=`ldapsearch -H ldap://${OUTPUT_SE}:2135 -x -b 'Mds-Vo-Name=local,o=grid' "GlueSAAccessControlBaseRule=$VO" | grep 'GlueSARoot:' | cut -d: -f2`
    fi
    if [ -z "$STORAGE_DIR" ] ; then
	echo "ERROR: NO_STOR_DIR" >&2
	echo "insert StorDir NO_STOR_DIR"
	exit 1
    fi
    STORAGE_DIR=`echo $STORAGE_DIR | tr -d '[:blank:]'`
    echo "insert StorDir $STORAGE_DIR"

# Copy the file to the SE
    OUTPUT_DIR=${STORAGE_DIR}/${LFN_DIR}

# Check that the destination directory exists; if not create it
    edg-gridftp-exists gsiftp://${OUTPUT_SE}${OUTPUT_DIR}
    if [ 1 ] ; then
	edg-gridftp-mkdir gsiftp://${OUTPUT_SE}${OUTPUT_DIR}
# Check again
	edg-gridftp-exists gsiftp://${OUTPUT_SE}${OUTPUT_DIR}
	if [ ! $? ] ; then
	    echo "ERROR: LFN_DIR_CREATION_FAILED" >&2
	    exit 1
	fi
    fi

# Use LDAP Replica Catalog
    if [ X$RLS = 'Xnone' ] ; then
	REG_OUT=`edg-replica-manager-copyAndRegisterFile -l ${LFN_DIR}/${DATA} -d ${OUTPUT_SE}${OUTPUT_DIR}/${DATA} -s ${HOST}${WDIR}/${DATA}`
	ERR=$?
    else

# Use RLS
	globus-url-copy file://${HOST}${WDIR}/${DATA} gsiftp://${OUTPUT_SE}${OUTPUT_DIR}/${DATA}
	if [ ! $? ] ; then
	    echo "ERROR: GLOBUS_URL_COPY_FAILED" >&2
	    echo "insert LFN !COPY_FAILED"
	    exit 1
	fi
	REG_OUT=`edg-replica-manager-registerEntry -l ${LFN_DIR}/${DATA} -s ${OUTPUT_SE}${OUTPUT_DIR}/${DATA}`
	ERR=$?
    fi
    REG_OUT=`echo $REG_OUT | grep "The program was successfully executed"`
    if [ ! $ERR -o -z "$REG_OUT" ] ; then
	echo "ERROR: COPY_AND_REG_FAILED" >&2
	echo "insert LFN !COPY_AND_REG_FAILED"
	exit 1
    fi
    echo "insert LFN ${LFN_DIR}/${DATA}"

# Replicate the file to another SE
    if [ "$REPLICATION" = 'YES' -a -n "$REP_SE" ] ; then
	edg-replica-manager-replicateFile -l ${LFN_DIR}/${DATA} -s ${OUTPUT_SE}${OUTPUT_DIR}/${DATA} -d $REP_SE
	if [ $? != 0 ] ; then
	    echo "ERROR: REPLICATE_FAILED"
	else
	    insert RepSE $REP_SE
	fi
    fi
fi

